@@ -36,8 +36,6 @@ from lava_dispatcher.errors import (
     OperationFailed,
 )
 
-from master import MasterCommandRunner
-
 
 class VexpressTarget(BootloaderTarget):
 
@@ -169,12 +167,6 @@ class VexpressTarget(BootloaderTarget):
                                                          firmware, bl0, bl1, bl2, bl31, rootfstype, bootloadertype,
                                                          target_type, qemu_pflash=qemu_pflash)
 
-    def boot_master_image(self):
-        super(VexpressTarget, self).boot_master_image()
-        runner = MasterCommandRunner(self)
-        runner.run("touch /forcefsck")
-        runner.run("sync")
-
     def _load_test_firmware(self):
         with self._mcc_setup() as mount_point:
             self._install_test_firmware(mount_point)
