@@ -24,7 +24,8 @@
 # pylint: disable=unused-import
 
 from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMU
-from lava_dispatcher.pipeline.actions.boot.pyocd import BootMonitorPyOCD
+from lava_dispatcher.pipeline.actions.boot.pyocd import PyOCD
+from lava_dispatcher.pipeline.actions.boot.cmsis_dap import CMSIS
 from lava_dispatcher.pipeline.actions.boot.dfu import DFU
 from lava_dispatcher.pipeline.actions.boot.u_boot import UBoot
 from lava_dispatcher.pipeline.actions.boot.kexec import BootKExec
@@ -35,4 +36,5 @@ from lava_dispatcher.pipeline.actions.boot.lxc import BootLxc
 from lava_dispatcher.pipeline.actions.boot.ipxe import IPXE
 from lava_dispatcher.pipeline.actions.boot.grub import Grub
 from lava_dispatcher.pipeline.actions.boot.iso import BootIsoInstaller
-from lava_dispatcher.pipeline.actions.boot.bootloader_defaults import BootloaderDefaults
+from lava_dispatcher.pipeline.actions.boot.minimal import Minimal
+from lava_dispatcher.pipeline.actions.boot.uefi import UefiShell
