@@ -23,20 +23,22 @@ import shutil
 import subprocess
 from lava_dispatcher.pipeline.action import (
     Action,
-    Pipeline,
     InfrastructureError,
-    JobError
+    JobError,
+    LAVABug,
+    Pipeline,
 )
 from lava_dispatcher.pipeline.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.pipeline.utils.constants import (
-    LXC_PATH,
-    RAMDISK_FNAME
+    RAMDISK_FNAME,
+    UBOOT_DEFAULT_HEADER_LENGTH,
 )
 from lava_dispatcher.pipeline.utils.installers import (
     add_late_command,
     add_to_kickstart
 )
 from lava_dispatcher.pipeline.utils.filesystem import (
+    lxc_path,
     mkdtemp,
     prepare_guestfs,
     copy_in_overlay,
@@ -50,6 +52,9 @@ from lava_dispatcher.pipeline.utils.compression import (
 )
 from lava_dispatcher.pipeline.utils.strings import substitute
 from lava_dispatcher.pipeline.utils.network import dispatcher_ip
+from lava_dispatcher.pipeline.actions.deploy.prepare import PrepareKernelAction
+
+# pylint: disable=superfluous-parens,too-many-statements
 
 
 class ApplyOverlayGuest(Action):
@@ -70,10 +75,10 @@ class ApplyOverlayGuest(Action):
         if 'guest' not in self.job.device['actions']['deploy']['methods']['image']['parameters']:
             self.errors = "Device configuration does not specify size of guest filesystem."
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
         if not overlay_file:
-            raise RuntimeError("Unable to find the overlay")
+            raise LAVABug("Unable to find the overlay")
         self.logger.debug("Overlay: %s", overlay_file)
         guest_dir = self.mkdtemp()
         guest_file = os.path.join(guest_dir, self.guest_filename)
@@ -94,13 +99,15 @@ class ApplyOverlayImage(Action):
         self.summary = "apply overlay to test image"
         self.description = "apply overlay via guestfs to the test image"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
         if overlay_file:
             self.logger.debug("Overlay: %s", overlay_file)
-            decompressed_image = self.get_namespace_data(action='download_action', label='image', key='file')
+            decompressed_image = self.get_namespace_data(action='download-action', label='image', key='file')
             self.logger.debug("Image: %s", decompressed_image)
-            root_partition = self.parameters['image']['root_partition']
+            root_partition = self.parameters['image'].get('root_partition')
+            if root_partition is None:
+                raise JobError("Unable to apply the overlay image without 'root_partition'")
             self.logger.debug("root_partition: %s", root_partition)
             copy_in_overlay(decompressed_image, root_partition, overlay_file)
         else:
@@ -108,28 +115,29 @@ class ApplyOverlayImage(Action):
         return connection
 
 
-class ApplyOverlaySparseRootfs(Action):
+class ApplyOverlaySparseImage(Action):
 
-    def __init__(self):
-        super(ApplyOverlaySparseRootfs, self).__init__()
-        self.name = "apply-overlay-sparse-rootfs"
-        self.summary = "apply overlay to sparse rootfs image"
-        self.description = "apply overlay to sparse rootfs image"
+    def __init__(self, image_key):
+        super(ApplyOverlaySparseImage, self).__init__()
+        self.name = "apply-overlay-sparse-image"
+        self.summary = "apply overlay to sparse image"
+        self.description = "apply overlay to sparse image"
+        self.image_key = image_key  # the sparse image key in the parameters
 
     def validate(self):
-        super(ApplyOverlaySparseRootfs, self).validate()
+        super(ApplyOverlaySparseImage, self).validate()
         self.errors = infrastructure_error('/usr/bin/simg2img')
         self.errors = infrastructure_error('/bin/mount')
         self.errors = infrastructure_error('/bin/umount')
         self.errors = infrastructure_error('/usr/bin/img2simg')
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         overlay_file = self.get_namespace_data(action='compress-overlay',
                                                label='output', key='file')
         if overlay_file:
             self.logger.debug("Overlay: %s", overlay_file)
             decompressed_image = self.get_namespace_data(
-                action='download_action', label='rootfs', key='file')
+                action='download-action', label=self.image_key, key='file')
             self.logger.debug("Image: %s", decompressed_image)
             copy_overlay_to_sparse_fs(decompressed_image, overlay_file)
         else:
@@ -154,13 +162,15 @@ class PrepareOverlayTftp(Action):
         self.internal_pipeline.add_action(ExtractRamdisk())  # idempotent, checks for a ramdisk parameter
         self.internal_pipeline.add_action(ExtractModules())  # idempotent, checks for a modules parameter
         self.internal_pipeline.add_action(ApplyOverlayTftp())
+        if 'kernel' in parameters and 'type' in parameters['kernel']:
+            self.internal_pipeline.add_action(PrepareKernelAction())
         self.internal_pipeline.add_action(ConfigurePreseedFile())  # idempotent, checks for a preseed parameter
         self.internal_pipeline.add_action(CompressRamdisk())  # idempotent, checks for a ramdisk parameter
 
-    def run(self, connection, args=None):
-        connection = super(PrepareOverlayTftp, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(PrepareOverlayTftp, self).run(connection, max_end_time, args)
         ramdisk = self.get_namespace_data(
-            action='download_action',
+            action='download-action',
             label='file',
             key='ramdisk'
         )
@@ -183,61 +193,82 @@ class ApplyOverlayTftp(Action):
         self.summary = "apply lava overlay test files"
         self.description = "unpack the overlay into the nfsrootfs or ramdisk"
 
-    def run(self, connection, args=None):  # pylint: disable=too-many-branches
-        connection = super(ApplyOverlayTftp, self).run(connection, args)
+    def validate(self):
+        super(ApplyOverlayTftp, self).validate()
+        persist = self.parameters.get('persistent_nfs', None)
+        if persist:
+            if not isinstance(persist, dict):
+                self.errors = "Invalid persistent_nfs parameter."
+            if 'address' not in persist:
+                self.errors = "Missing address for persistent NFS"
+
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-branches
+        connection = super(ApplyOverlayTftp, self).run(connection, max_end_time, args)
         directory = None
-        nfs_url = None
+        nfs_address = None
         overlay_file = None
+        namespace = self.parameters.get('namespace', None)
         if self.parameters.get('nfsrootfs', None) is not None:
             if not self.parameters['nfsrootfs'].get('install_overlay', True):
-                self.logger.info("Skipping applying overlay to NFS")
+                self.logger.info("[%s] Skipping applying overlay to NFS", namespace)
                 return connection
             overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
             directory = self.get_namespace_data(action='extract-rootfs', label='file', key='nfsroot')
-            self.logger.info("Applying overlay to NFS")
-        elif self.parameters.get('nfs_url', None) is not None:
-            if not self.parameters['nfs_url'].get('install_overlay', True):
-                self.logger.info("Skipping applying overlay to persistent NFS")
+            if overlay_file:
+                self.logger.info("[%s] Applying overlay to NFS", namespace)
+        elif self.parameters.get('images', {}).get('nfsrootfs', None) is not None:
+            if not self.parameters['images']['nfsrootfs'].get('install_overlay', True):
+                self.logger.info("[%s] Skipping applying overlay to NFS", namespace)
                 return connection
-            nfs_url = self.parameters.get('nfs_url')
-            self.logger.info("Applying overlay to persistent NFS")
+            overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
+            directory = self.get_namespace_data(action='extract-rootfs', label='file', key='nfsroot')
+            if overlay_file:
+                self.logger.info("[%s] Applying overlay to NFS", namespace)
+        elif self.parameters.get('persistent_nfs', None) is not None:
+            if not self.parameters['persistent_nfs'].get('install_overlay', True):
+                self.logger.info("[%s] Skipping applying overlay to persistent NFS", namespace)
+                return connection
+            overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
+            nfs_address = self.parameters['persistent_nfs'].get('address')
+            if overlay_file:
+                self.logger.info("[%s] Applying overlay to persistent NFS address %s", namespace, nfs_address)
             # need to mount the persistent NFS here.
             # We can't use self.mkdtemp() here because this directory should
             # not be removed if umount fails.
             directory = mkdtemp(autoremove=False)
             try:
-                subprocess.check_output(['mount', '-t', 'nfs', nfs_url, directory])
+                subprocess.check_output(['mount', '-t', 'nfs', nfs_address, directory])
             except subprocess.CalledProcessError as exc:
                 raise JobError(exc)
         elif self.parameters.get('ramdisk', None) is not None:
             if not self.parameters['ramdisk'].get('install_overlay', True):
-                self.logger.info("Skipping applying overlay to ramdisk")
+                self.logger.info("[%s] Skipping applying overlay to ramdisk", namespace)
                 return connection
             overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
             directory = self.get_namespace_data(action='extract-overlay-ramdisk', label='extracted_ramdisk', key='directory')
-            self.logger.info("Applying overlay %s to ramdisk", overlay_file)
+            if overlay_file:
+                self.logger.info("[%s] Applying overlay %s to ramdisk", namespace, overlay_file)
         elif self.parameters.get('rootfs', None) is not None:
             overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
             directory = self.get_namespace_data(action='apply-overlay', label='file', key='root')
         else:
-            self.logger.debug("No overlay directory")
+            self.logger.debug("[%s] No overlay directory", namespace)
             self.logger.debug(self.parameters)
         if self.parameters.get('os', None) == "centos_installer":
             # centos installer ramdisk doesnt like having anything other
             # than the kickstart config being inserted. Instead, make the
             # overlay accessible through tftp. Yuck.
-            tftp_dir = os.path.dirname(self.get_namespace_data(action='download_action', label='ramdisk', key='file'))
+            tftp_dir = os.path.dirname(self.get_namespace_data(action='download-action', label='ramdisk', key='file'))
             shutil.copy(overlay_file, tftp_dir)
             suffix = self.get_namespace_data(action='tftp-deploy', label='tftp', key='suffix')
             if not suffix:
                 suffix = ''
-            self.set_namespace_data(action=self.name, label='file', key='overlay', value=os.path.join(suffix, os.path.basename(overlay_file)))
-        if nfs_url:
-            subprocess.check_output(['umount', directory])
-            os.rmdir(directory)  # fails if the umount fails
+            self.set_namespace_data(action=self.name, label='file', key='overlay',
+                                    value=os.path.join(suffix, "ramdisk", os.path.basename(overlay_file)))
         if overlay_file:
+            self.logger.debug("[%s] Applying overlay %s to directory %s", namespace, overlay_file, directory)
             untar_file(overlay_file, directory)
-            if nfs_url:
+            if nfs_address:
                 subprocess.check_output(['umount', directory])
                 os.rmdir(directory)  # fails if the umount fails
         return connection
@@ -258,11 +289,11 @@ class ExtractRootfs(Action):  # pylint: disable=too-many-instance-attributes
         self.use_tarfile = True
         self.use_lzma = False
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if not self.parameters.get(self.param_key, None):  # idempotency
             return connection
-        connection = super(ExtractRootfs, self).run(connection, args)
-        root = self.get_namespace_data(action='download_action', label=self.param_key, key='file')
+        connection = super(ExtractRootfs, self).run(connection, max_end_time, args)
+        root = self.get_namespace_data(action='download-action', label=self.param_key, key='file')
         root_dir = self.mkdtemp()
         untar_file(root, root_dir)
         self.set_namespace_data(action='extract-rootfs', label='file', key=self.file_key, value=root_dir)
@@ -287,7 +318,7 @@ class ExtractNfsRootfs(ExtractRootfs):
         if not self.parameters.get(self.param_key, None):  # idempotency
             return
         if not self.get_namespace_data(
-                action='download_action', label=self.param_key, key='file'):
+                action='download-action', label=self.param_key, key='file'):
             self.errors = "no file specified extract as %s" % self.param_key
         if not os.path.exists('/usr/sbin/exportfs'):
             raise InfrastructureError("NFS job requested but nfs-kernel-server not installed.")
@@ -298,10 +329,10 @@ class ExtractNfsRootfs(ExtractRootfs):
             if not prefix.endswith('/'):
                 self.errors = 'prefix must be a directory and end with /'
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if not self.parameters.get(self.param_key, None):  # idempotency
             return connection
-        connection = super(ExtractNfsRootfs, self).run(connection, args)
+        connection = super(ExtractNfsRootfs, self).run(connection, max_end_time, args)
 
         if 'prefix' in self.parameters[self.param_key]:
             prefix = self.parameters[self.param_key]['prefix']
@@ -332,11 +363,11 @@ class ExtractModules(Action):
         self.summary = "extract kernel modules"
         self.description = "extract supplied kernel modules"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if not self.parameters.get('modules', None):  # idempotency
             return connection
-        connection = super(ExtractModules, self).run(connection, args)
-        modules = self.get_namespace_data(action='download_action', label='modules', key='file')
+        connection = super(ExtractModules, self).run(connection, max_end_time, args)
+        modules = self.get_namespace_data(action='download-action', label='modules', key='file')
         if not self.parameters.get('ramdisk', None):
             if not self.parameters.get('nfsrootfs', None):
                 raise JobError("Unable to identify a location for the unpacked modules")
@@ -365,7 +396,7 @@ class ExtractModules(Action):
         try:
             os.unlink(modules)
         except OSError as exc:
-            raise RuntimeError("Unable to remove tarball: '%s' - %s" % (modules, exc))
+            raise InfrastructureError("Unable to remove tarball: '%s' - %s" % (modules, exc))
         return connection
 
 
@@ -393,18 +424,18 @@ class ExtractRamdisk(Action):
             self.skip = True
             return
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if not self.parameters.get('ramdisk', None):  # idempotency
             return connection
-        ramdisk = self.get_namespace_data(action='download_action', label='ramdisk', key='file')
+        ramdisk = self.get_namespace_data(action='download-action', label='ramdisk', key='file')
         if self.skip:
             self.logger.info("Not extracting ramdisk.")
             suffix = self.get_namespace_data(action='tftp-deploy', label='tftp', key='suffix')
-            filename = os.path.join(suffix, os.path.basename(ramdisk))
+            filename = os.path.join(suffix, "ramdisk", os.path.basename(ramdisk))
             # declare the original ramdisk as the name to be used later.
             self.set_namespace_data(action='compress-ramdisk', label='file', key='ramdisk', value=filename)
             return
-        connection = super(ExtractRamdisk, self).run(connection, args)
+        connection = super(ExtractRamdisk, self).run(connection, max_end_time, args)
         ramdisk_dir = self.mkdtemp()
         extracted_ramdisk = os.path.join(ramdisk_dir, 'ramdisk')
         os.mkdir(extracted_ramdisk, 0o755)
@@ -414,12 +445,12 @@ class ExtractRamdisk(Action):
             suffix = ".%s" % compression
         ramdisk_compressed_data = os.path.join(ramdisk_dir, RAMDISK_FNAME + suffix)
         if self.parameters['ramdisk'].get('header', None) == 'u-boot':
-            # TODO: 64 bytes is empirical - may need to be configurable in the future
-            cmd = ('dd if=%s of=%s ibs=64 skip=1' % (ramdisk, ramdisk_compressed_data)).split(' ')
+            cmd = ('dd if=%s of=%s ibs=%s skip=1' % (
+                ramdisk, ramdisk_compressed_data, UBOOT_DEFAULT_HEADER_LENGTH)).split(' ')
             try:
                 self.run_command(cmd)
             except:
-                raise RuntimeError('Unable to remove uboot header: %s' % ramdisk)
+                raise LAVABug('Unable to remove uboot header: %s' % ramdisk)
         else:
             # give the file a predictable name
             shutil.move(ramdisk, ramdisk_compressed_data)
@@ -428,7 +459,7 @@ class ExtractRamdisk(Action):
         os.chdir(extracted_ramdisk)
         cmd = ('cpio -iud -F %s' % ramdisk_data).split(' ')
         if not self.run_command(cmd):
-            raise JobError('Unable to extract cpio arcive: %s - missing header definition (i.e. u-boot)?' % ramdisk_data)
+            raise JobError('Unable to extract cpio archive: %s - missing header definition (i.e. u-boot)?' % ramdisk_data)
         os.chdir(pwd)
         # tell other actions where the unpacked ramdisk can be found
         self.set_namespace_data(action=self.name, label='extracted_ramdisk', key='directory', value=extracted_ramdisk)
@@ -469,20 +500,20 @@ class CompressRamdisk(Action):
                 else:
                     self.errors = "ramdisk: add_header: unknown header type"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals
         if not self.parameters.get('ramdisk', None):  # idempotency
             return connection
         if self.skip:
             return connection
-        connection = super(CompressRamdisk, self).run(connection, args)
+        connection = super(CompressRamdisk, self).run(connection, max_end_time, args)
         ramdisk_dir = self.get_namespace_data(
             action='extract-overlay-ramdisk', label='extracted_ramdisk', key='directory')
         ramdisk_data = self.get_namespace_data(
             action='extract-overlay-ramdisk', label='ramdisk_file', key='file')
         if not ramdisk_dir:
-            raise RuntimeError("Unable to find unpacked ramdisk")
+            raise LAVABug("Unable to find unpacked ramdisk")
         if not ramdisk_data:
-            raise RuntimeError("Unable to find ramdisk directory")
+            raise LAVABug("Unable to find ramdisk directory")
         if self.parameters.get('preseed', None):
             if self.parameters["deployment_data"].get("preseed_to_ramdisk", None):
                 # download action must have completed to get this far
@@ -492,19 +523,19 @@ class CompressRamdisk(Action):
                 filename = self.parameters["deployment_data"]["preseed_to_ramdisk"]
                 self.logger.info("Copying preseed file into ramdisk: %s", filename)
                 shutil.copy(self.get_namespace_data(
-                    action='download_action', label='preseed', key='file'),
-                    os.path.join(ramdisk_dir, filename))
+                    action='download-action', label='preseed',
+                    key='file'), os.path.join(ramdisk_dir, filename))
                 self.set_namespace_data(action=self.name, label='file', key='preseed_local', value=filename)
         pwd = os.getcwd()
         os.chdir(ramdisk_dir)
-        self.logger.debug("Building ramdisk %s containing %s",
-                          ramdisk_data, ramdisk_dir)
+        self.logger.info("Building ramdisk %s containing %s",
+                         ramdisk_data, ramdisk_dir)
         cmd = "find . | cpio --create --format='newc' > %s" % ramdisk_data
         try:
             # safe to use shell=True here, no external arguments
             log = subprocess.check_output(cmd, shell=True, stderr=subprocess.STDOUT)
         except OSError as exc:
-            raise RuntimeError('Unable to create cpio filesystem: %s' % exc)
+            raise InfrastructureError('Unable to create cpio filesystem: %s' % exc)
         # lazy-logging would mean that the quoting of cmd causes invalid YAML
         self.logger.debug("%s\n%s" % (cmd, log))  # pylint: disable=logging-not-lazy
 
@@ -513,14 +544,14 @@ class CompressRamdisk(Action):
         final_file = compress_file(ramdisk_data, compression)
         os.chdir(pwd)
         tftp_dir = os.path.dirname(self.get_namespace_data(
-            action='download_action', label='ramdisk', key='file'))
+            action='download-action', label='ramdisk', key='file'))
 
         if self.add_header == 'u-boot':
             ramdisk_uboot = final_file + ".uboot"
             self.logger.debug("Adding RAMdisk u-boot header.")
             cmd = ("mkimage -A %s -T ramdisk -C none -d %s %s" % (self.mkimage_arch, final_file, ramdisk_uboot)).split(' ')
             if not self.run_command(cmd):
-                raise RuntimeError("Unable to add uboot header to ramdisk")
+                raise InfrastructureError("Unable to add uboot header to ramdisk")
             final_file = ramdisk_uboot
 
         shutil.move(final_file, os.path.join(tftp_dir, os.path.basename(final_file)))
@@ -532,7 +563,7 @@ class CompressRamdisk(Action):
                 suffix = ''
             self.set_namespace_data(
                 action=self.name,
-                label='file', key='ramdisk', value=os.path.join(suffix, os.path.basename(final_file)))
+                label='file', key='ramdisk', value=os.path.join(suffix, "ramdisk", os.path.basename(final_file)))
         else:
             self.set_namespace_data(
                 action=self.name,
@@ -555,8 +586,8 @@ class ApplyLxcOverlay(Action):
         super(ApplyLxcOverlay, self).validate()
         self.errors = infrastructure_error('tar')
 
-    def run(self, connection, args=None):
-        connection = super(ApplyLxcOverlay, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(ApplyLxcOverlay, self).run(connection, max_end_time, args)
         overlay_file = self.get_namespace_data(action='compress-overlay', label='output', key='file')
         if overlay_file is None:
             self.logger.debug("skipped %s", self.name)
@@ -564,27 +595,28 @@ class ApplyLxcOverlay(Action):
         lxc_name = self.get_namespace_data(
             action='lxc-create-action',
             label='lxc', key='name')
-        lxc_path = os.path.join(LXC_PATH, lxc_name, 'rootfs')
-        if not os.path.exists(lxc_path):
-            raise JobError("Lxc container rootfs not found")
-        tar_cmd = ['tar', '--warning', 'no-timestamp', '-C', lxc_path, '-xaf',
-                   overlay_file]
+        lxc_default_path = lxc_path(self.job.parameters['dispatcher'])
+        lxc_rootfs_path = os.path.join(lxc_default_path, lxc_name, 'rootfs')
+        if not os.path.exists(lxc_rootfs_path):
+            raise LAVABug("Lxc container rootfs not found")
+        tar_cmd = ['tar', '--warning', 'no-timestamp', '-C', lxc_rootfs_path,
+                   '-xaf', overlay_file]
         command_output = self.run_command(tar_cmd)
         if command_output and command_output is not '':
             raise JobError("Unable to untar overlay: %s" %
-                           command_output)  # FIXME: JobError needs a unit test
+                           command_output)
 
         # FIXME: Avoid copying this special 'lava-test-runner' which does not
         #        have 'sync' in cleanup. This should be handled during the
         #        creation of the overlay instead. Make a special case to copy
         #        lxc specific scripts, with distro specific versions.
         fname = os.path.join(self.lava_test_dir, 'lava-test-runner')
-        output_file = '%s/bin/%s' % (lxc_path, os.path.basename(fname))
+        output_file = '%s/bin/%s' % (lxc_rootfs_path, os.path.basename(fname))
         self.logger.debug("Copying %s", output_file)
         try:
             shutil.copy(fname, output_file)
         except IOError:
-            raise JobError("Unable to copy: %s" % output_file)
+            raise InfrastructureError("Unable to copy: %s" % output_file)
 
         return connection
 
@@ -596,19 +628,20 @@ class ConfigurePreseedFile(Action):
         self.summary = "add commands to installer config"
         self.description = "add commands to automated installers, to copy the lava test overlay to the installed system"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if 'deployment_data' not in self.parameters:
             return connection
         if self.parameters["deployment_data"].get('installer_extra_cmd', None):
             if self.parameters.get('os', None) == "debian_installer":
-                add_late_command(self.get_namespace_data(action='download_action', label='preseed', key='file'),
+                add_late_command(self.get_namespace_data(action='download-action', label='preseed', key='file'),
                                  self.parameters["deployment_data"]["installer_extra_cmd"])
             if self.parameters.get('os', None) == "centos_installer":
+                ip_addr = dispatcher_ip(self.job.parameters['dispatcher'])
                 overlay = self.get_namespace_data(
-                    action='download_action', label='file', key='overlay')
+                    action='download-action', label='file', key='overlay')
                 substitutions = {
-                    '{OVERLAY_URL}': 'tftp://' + dispatcher_ip() + '/' + overlay
+                    '{OVERLAY_URL}': 'tftp://' + ip_addr + '/' + overlay
                 }
                 post_command = substitute([self.parameters["deployment_data"]["installer_extra_cmd"]], substitutions)
-                add_to_kickstart(self.get_namespace_data(action='download_action', label='preseed', key='file'), post_command[0])
+                add_to_kickstart(self.get_namespace_data(action='download-action', label='preseed', key='file'), post_command[0])
         return connection
