@@ -22,6 +22,7 @@
 # This class is used for all downloads, including images and individual files for tftp.
 # python2 only
 
+import errno
 import math
 import os
 import sys
@@ -36,7 +37,9 @@ import lzma
 import zlib
 from lava_dispatcher.pipeline.action import (
     Action,
+    InfrastructureError,
     JobError,
+    LAVABug,
     Pipeline,
 )
 from lava_dispatcher.pipeline.logical import RetryAction
@@ -44,7 +47,6 @@ from lava_dispatcher.pipeline.utils.compression import untar_file
 from lava_dispatcher.pipeline.utils.constants import (
     FILE_DOWNLOAD_CHUNK_SIZE,
     HTTP_DOWNLOAD_CHUNK_SIZE,
-    HTTP_DOWNLOAD_TIMEOUT,
     SCP_DOWNLOAD_CHUNK_SIZE,
 )
 
@@ -67,9 +69,10 @@ class DownloaderAction(RetryAction):
     """
     def __init__(self, key, path):
         super(DownloaderAction, self).__init__()
-        self.name = "download_retry"
+        self.name = "download-retry"
         self.description = "download with retry"
         self.summary = "download-retry"
+        self.max_retries = 3
         self.key = key  # the key in the parameters of what to download
         self.path = path  # where to download
 
@@ -105,40 +108,37 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
 
     def __init__(self, key, path, url):
         super(DownloadHandler, self).__init__()
-        self.name = "download_action"
+        self.name = "download-action"
         self.description = "download action"
         self.summary = "download-action"
-        self.proxy = None
+
         self.url = url
         self.key = key
-        self.path = path
+        # Store the files in a sub-directory to keep the path unique
+        self.path = os.path.join(path, key)
         self.size = -1
 
     def reader(self):
-        raise NotImplementedError
+        raise LAVABug("'reader' function unimplemented")
 
-    def cleanup(self):
-        nested_tmp_dir = os.path.join(self.path, self.key)
-        self.logger.debug("%s cleanup", self.name)
-        if os.path.exists(nested_tmp_dir):
-            self.logger.debug("Cleaning up temporary tree.")
-            shutil.rmtree(nested_tmp_dir)
-        self.set_namespace_data(action='download_action', label=self.key, key='file', value='')
-        super(DownloadHandler, self).cleanup()
+    def cleanup(self, connection):
+        if os.path.exists(self.path):
+            self.logger.debug("Cleaning up download directory: %s", self.path)
+            shutil.rmtree(self.path)
+        self.set_namespace_data(action='download-action', label=self.key, key='file', value='')
+        super(DownloadHandler, self).cleanup(connection)
 
     def _url_to_fname_suffix(self, path, modify):
         filename = os.path.basename(self.url.path)
         parts = filename.split('.')
-        suffix = parts[-1]
-        if not modify:
-            filename = os.path.join(path, filename)
-            suffix = None
-        elif len(parts) == 1:  # handle files without suffixes, e.g. kernel images
-            filename = os.path.join(path, ''.join(parts[-1]))
-            suffix = None
+        # Handle unmodified filename
+        # Also files without suffixes, e.g. kernel images
+        if not modify or len(parts) == 1:
+            return (os.path.join(path, filename),
+                    None)
         else:
-            filename = os.path.join(path, '.'.join(parts[:-1]))
-        return filename, suffix
+            return (os.path.join(path, '.'.join(parts[:-1])),
+                    parts[-1])
 
     @contextlib.contextmanager
     def _decompressor_stream(self):  # pylint: disable=too-many-branches
@@ -153,6 +153,8 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                 compression = self.parameters[self.key].get('compression', False)
 
         fname, _ = self._url_to_fname_suffix(self.path, compression)
+        if os.path.isdir(fname):
+            raise JobError("Download '%s' is a directory, not a file" % fname)
         if os.path.exists(fname):
             os.remove(fname)
 
@@ -172,30 +174,29 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             if decompressor:
                 try:
                     buff = decompressor.decompress(buff)
-                except (EOFError) as exc:
+                except EOFError as eof_exc:
                     # EOFError can be raised when decompressing a bz2 archive
                     # generated by pbzip2. If there is something in unsused_data
                     # try to continue decompression.
-                    if compression == 'bz2':
-                        if decompressor.unused_data:
-                            buff = decompressor.unused_data
-                        else:
-                            self.logger.exception(str(exc))
-                            raise JobError(exc)
+                    if compression == 'bz2' and decompressor.unused_data:
+                        buff = decompressor.unused_data
                     else:
-                        self.logger.exception(str(exc))
-                        raise JobError(exc)
-                except (IOError, lzma.error, zlib.error) as exc:
-                    self.logger.exception(str(exc))
-                    raise JobError(exc)
+                        error_message = str(eof_exc)
+                        self.logger.exception(error_message)
+                        raise JobError(error_message)
+                except (IOError, lzma.error, zlib.error) as exc:  # pylint: disable=no-member
+                    error_message = str(exc)
+                    self.logger.exception(error_message)
+                    raise JobError(error_message)
             dwnld_file.write(buff)
 
         try:
-            dwnld_file = open(fname, 'wb')
-            yield (write, fname)
-        finally:
-            if dwnld_file:
-                dwnld_file.close()
+            with open(fname, 'wb') as dwnld_file:
+                yield (write, fname)
+        except (IOError, OSError) as exc:
+            msg = "Unable to open %s: %s" % (fname, exc.strerror)
+            self.logger.error(msg)
+            raise InfrastructureError(msg)
 
     def validate(self):
         super(DownloadHandler, self).validate()
@@ -207,9 +208,9 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             image_name, _ = self._url_to_fname_suffix(self.path, compression)
             image_arg = image.get('image_arg', None)
             overlay = image.get('overlay', False)
-            self.set_namespace_data(action='download_action', label=self.key,
+            self.set_namespace_data(action='download-action', label=self.key,
                                     key='file', value=image_name)
-            self.set_namespace_data(action='download_action', label=self.key,
+            self.set_namespace_data(action='download-action', label=self.key,
                                     key='image_arg', value=image_arg)
         else:
             self.url = lavaurl.urlparse(self.parameters[self.key]['url'])
@@ -217,12 +218,12 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             archive = self.parameters[self.key].get('archive', None)
             overlay = self.parameters.get('overlay', False)
             fname, _ = self._url_to_fname_suffix(self.path, compression)
-            self.set_namespace_data(action='download_action', label=self.key, key='file', value=fname)
+            self.set_namespace_data(action='download-action', label=self.key, key='file', value=fname)
 
         if overlay:
-            self.set_namespace_data(action='download_action', label=self.key, key='overlay', value=overlay)
+            self.set_namespace_data(action='download-action', label=self.key, key='overlay', value=overlay)
         if compression:
-            if compression not in ['gz', 'bz2', 'xz']:
+            if compression not in ['gz', 'bz2', 'xz', 'zip']:
                 self.errors = "Unknown 'compression' format '%s'" % compression
         if archive:
             if archive not in ['tar']:
@@ -230,27 +231,37 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
         # pass kernel type to boot Action
         if self.key == 'kernel' and ('kernel' in self.parameters):
             self.set_namespace_data(
-                action='download_action', label='type', key=self.key,
+                action='download-action', label='type', key=self.key,
                 value=self.parameters[self.key].get('type', None))
 
-    def run(self, connection, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
-        def progress_unknown_total(downloaded_size, last_value):
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+        def progress_unknown_total(downloaded_sz, last_val):
             """ Compute progress when the size is unknown """
-            condition = downloaded_size >= last_value + 25 * 1024 * 1024
-            return (condition, downloaded_size,
-                    "progress %dMB" % (int(downloaded_size / (1024 * 1024))) if condition else "")
+            condition = downloaded_sz >= last_val + 25 * 1024 * 1024
+            return (condition, downloaded_sz,
+                    "progress %dMB" % (int(downloaded_sz / (1024 * 1024))) if condition else "")
 
-        def progress_known_total(downloaded_size, last_value):
+        def progress_known_total(downloaded_sz, last_val):
             """ Compute progress when the size is known """
-            percent = math.floor(downloaded_size / float(self.size) * 100)
-            condition = percent >= last_value + 5
+            percent = math.floor(downloaded_sz / float(self.size) * 100)
+            condition = percent >= last_val + 5
             return (condition, percent,
-                    "progress %3d%% (%dMB)" % (percent, int(downloaded_size / (1024 * 1024))) if condition else "")
+                    "progress %3d%% (%dMB)" % (percent, int(downloaded_sz / (1024 * 1024))) if condition else "")
 
-        connection = super(DownloadHandler, self).run(connection, args)
+        connection = super(DownloadHandler, self).run(connection, max_end_time, args)
         # self.cookies = self.job.context.config.lava_cookies  # FIXME: work out how to restore
         md5 = hashlib.md5()
         sha256 = hashlib.sha256()
+
+        # Create a fresh directory if the old one has been removed by a previous cleanup
+        # (when retrying inside a RetryAction)
+        try:
+            os.makedirs(self.path, 0o755)
+        except OSError as exc:
+            if exc.errno != errno.EEXIST:
+                raise InfrastructureError("Unable to create %s: %s" % (self.path, str(exc)))
+
+        # Download the file
         with self._decompressor_stream() as (writer, fname):
 
             if 'images' in self.parameters and self.key in self.parameters['images']:
@@ -260,7 +271,8 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             md5sum = remote.get('md5sum', None)
             sha256sum = remote.get('sha256sum', None)
 
-            self.logger.info("downloading %s as %s" % (remote['url'], fname))
+            self.logger.info("downloading %s", remote['url'])
+            self.logger.debug("saving as %s", fname)
 
             downloaded_size = 0
             beginning = time.time()
@@ -292,12 +304,11 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
                               round(downloaded_size / (1024 * 1024 * (ending - beginning)), 2)))
 
         # set the dynamic data into the context
-        self.set_namespace_data(action='download_action', label=self.key, key='file', value=fname)
-        self.set_namespace_data(action='download_action', label=self.key, key='md5', value=md5.hexdigest())
-        self.set_namespace_data(action='download_action', label=self.key, key='sha256', value=sha256.hexdigest())
+        self.set_namespace_data(action='download-action', label=self.key, key='file', value=fname)
+        self.set_namespace_data(action='download-action', label=self.key, key='md5', value=md5.hexdigest())
+        self.set_namespace_data(action='download-action', label=self.key, key='sha256', value=sha256.hexdigest())
 
         # handle archive files
-        archive = False
         if 'images' in self.parameters and self.key in self.parameters['images']:
             archive = self.parameters['images'][self.key].get('archive', False)
         else:
@@ -313,26 +324,26 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
             if archive == 'tar':
                 untar_file(origin, None, member=target_fname,
                            outfile=target_fname_path)
-                self.set_namespace_data(action='download_action', label=self.key, key='file', value=target_fname_path)
-                self.set_namespace_data(action='download_action', label='file', key=self.key, value=target_fname)
+                self.set_namespace_data(action='download-action', label=self.key, key='file', value=target_fname_path)
+                self.set_namespace_data(action='download-action', label='file', key=self.key, value=target_fname)
             self.logger.debug("Using %s archive" % archive)
 
         if md5sum is not None:
-            chk_md5sum = self.get_namespace_data(action='download_action', label=self.key, key='md5')
+            chk_md5sum = self.get_namespace_data(action='download-action', label=self.key, key='md5')
             if md5sum != chk_md5sum:
                 self.logger.error("md5sum of downloaded content: %s" % chk_md5sum)
                 self.logger.info("sha256sum of downloaded content: %s" % (
-                    self.get_namespace_data(action='download_action', label=self.key, key='sha256')))
+                    self.get_namespace_data(action='download-action', label=self.key, key='sha256')))
                 self.results = {'fail': {
                     'md5': md5sum, 'download': chk_md5sum}}
                 raise JobError("MD5 checksum for '%s' does not match." % fname)
             self.results = {'success': {'md5': md5sum}}
 
         if sha256sum is not None:
-            chk_sha256sum = self.get_namespace_data(action='download_action', label=self.key, key='sha256')
+            chk_sha256sum = self.get_namespace_data(action='download-action', label=self.key, key='sha256')
             if sha256sum != chk_sha256sum:
                 self.logger.info("md5sum of downloaded content: %s" % (
-                    self.get_namespace_data(action='download_action', label=self.key, key='md5')))
+                    self.get_namespace_data(action='download-action', label=self.key, key='md5')))
                 self.logger.error("sha256sum of downloaded content: %s" % chk_sha256sum)
                 self.results = {'fail': {
                     'sha256': sha256sum, 'download': chk_sha256sum}}
@@ -341,26 +352,24 @@ class DownloadHandler(Action):  # pylint: disable=too-many-instance-attributes
 
         # certain deployments need prefixes set
         if self.parameters['to'] == 'tftp':
-            suffix = self.get_namespace_data(action='tftp-deploy', label='tftp', key='suffix')
-            if not suffix:
-                suffix = ''
-            self.set_namespace_data(
-                action='download_action', label='file', key=self.key,
-                value=os.path.join(suffix, os.path.basename(fname)))
+            suffix = self.get_namespace_data(action='tftp-deploy', label='tftp',
+                                             key='suffix')
+            self.set_namespace_data(action='download-action', label='file', key=self.key,
+                                    value=os.path.join(suffix, self.key, os.path.basename(fname)))
         elif self.parameters['to'] == 'iso-installer':
-            suffix = self.get_namespace_data(action='deploy-iso-installer', label='iso', key='suffix')
-            if not suffix:
-                suffix = ''
-            self.set_namespace_data(
-                action='download_action', label='file', key=self.key,
-                value=os.path.join(suffix, os.path.basename(fname)))
+            suffix = self.get_namespace_data(action='deploy-iso-installer',
+                                             label='iso', key='suffix')
+            self.set_namespace_data(action='download-action', label='file', key=self.key,
+                                    value=os.path.join(suffix, self.key, os.path.basename(fname)))
         else:
-            self.set_namespace_data(
-                action='download_action', label='file', key=self.key, value=fname)
-        self.logger.info("md5sum of downloaded content: %s" % (self.get_namespace_data(action='download_action',
-                                                                                       label=self.key, key='md5')))
-        self.logger.info("sha256sum of downloaded content: %s" % (self.get_namespace_data(action='download_action',
-                                                                                          label=self.key, key='sha256')))
+            self.set_namespace_data(action='download-action', label='file', key=self.key, value=fname)
+        self.results = {
+            'label': self.key,
+            'md5sum': str(self.get_namespace_data(
+                action='download-action', label=self.key, key='md5')),
+            'sha256sum': str(self.get_namespace_data(
+                action='download-action', label=self.key, key='sha256'))
+        }
         return connection
 
 
@@ -371,7 +380,7 @@ class FileDownloadAction(DownloadHandler):
 
     def __init__(self, key, path, url):
         super(FileDownloadAction, self).__init__(key, path, url)
-        self.name = "file_download"
+        self.name = "file-download"
         self.description = "copy a local file"
         self.summary = "local file copy"
 
@@ -391,8 +400,7 @@ class FileDownloadAction(DownloadHandler):
                 yield buff
                 buff = reader.read(FILE_DOWNLOAD_CHUNK_SIZE)
         except IOError as exc:
-            # TODO: improve error message
-            raise JobError(exc)
+            raise InfrastructureError("Unable to write to %s: %s" % (self.url.path, str(exc)))
         finally:
             if reader is not None:
                 reader.close()
@@ -405,42 +413,49 @@ class HttpDownloadAction(DownloadHandler):
 
     def __init__(self, key, path, url):
         super(HttpDownloadAction, self).__init__(key, path, url)
-        self.name = "http_download"
+        self.name = "http-download"
         self.description = "use http to download the file"
         self.summary = "http download"
 
     def validate(self):
         super(HttpDownloadAction, self).validate()
+        res = None
         try:
             self.logger.debug("Validating that %s exists", self.url.geturl())
-            res = requests.head(self.url.geturl(), allow_redirects=True, timeout=HTTP_DOWNLOAD_TIMEOUT)
+            res = requests.head(self.url.geturl(), allow_redirects=True)
             if res.status_code != requests.codes.OK:  # pylint: disable=no-member
                 # try using (the slower) get for services with broken redirect support
                 self.logger.debug("Using GET because HEAD is not supported properly")
+                res.close()
                 res = requests.get(
-                    self.url.geturl(), allow_redirects=True, stream=True,
-                    timeout=HTTP_DOWNLOAD_TIMEOUT)
+                    self.url.geturl(), allow_redirects=True, stream=True)
                 if res.status_code != requests.codes.OK:  # pylint: disable=no-member
                     self.errors = "Resources not available at '%s'" % (self.url.geturl())
 
             self.size = int(res.headers.get('content-length', -1))
         except requests.Timeout:
+            self.logger.error("Request timed out")
             self.errors = "'%s' timed out" % (self.url.geturl())
         except requests.RequestException as exc:
-            # TODO: find a better way to report the error
-            self.errors = str(exc)
+            self.logger.error("Ressource not available")
+            self.errors = "Unable to get '%s': %s" % (self.url.geturl(), str(exc))
+        finally:
+            if res is not None:
+                res.close()
 
     def reader(self):
         res = None
         try:
-            res = requests.get(self.url.geturl(), allow_redirects=True, stream=True, timeout=HTTP_DOWNLOAD_TIMEOUT)
+            res = requests.get(self.url.geturl(), allow_redirects=True,
+                               stream=True)
             if res.status_code != requests.codes.OK:  # pylint: disable=no-member
-                raise JobError("Unable to download '%s'" % (self.url.geturl()))
+                # This is an Infrastructure error because the validate function
+                # checked that the file does exist.
+                raise InfrastructureError("Unable to download '%s'" % (self.url.geturl()))
             for buff in res.iter_content(HTTP_DOWNLOAD_CHUNK_SIZE):
                 yield buff
         except requests.RequestException as exc:
-            # TODO: improve error reporting
-            raise JobError(exc)
+            raise InfrastructureError("Unable to download '%s': %s" % (self.url.geturl(), str(exc)))
         finally:
             if res is not None:
                 res.close()
@@ -453,7 +468,7 @@ class ScpDownloadAction(DownloadHandler):
 
     def __init__(self, key, path, url):
         super(ScpDownloadAction, self).__init__(key, path, url)
-        self.name = "scp_download"
+        self.name = "scp-download"
         self.description = "Use scp to copy the file"
         self.summary = "scp download"
 
@@ -499,28 +514,35 @@ class QCowConversionAction(Action):
 
     def __init__(self, key):
         super(QCowConversionAction, self).__init__()
-        self.name = "qcow2_convert"
+        self.name = "qcow2-convert"
         self.description = "convert qcow image using qemu-img"
         self.summary = "qcow conversion"
         self.key = key
 
-    def run(self, connection, args=None):
-        connection = super(QCowConversionAction, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(QCowConversionAction, self).run(connection, max_end_time, args)
         fname = self.get_namespace_data(
-            action='download_action',
+            action='download-action',
             label=self.key,
             key='file'
         )
         origin = fname
-        # Change the extension only if the file ends with '.qcow2'
+        # Remove the '.qcow2' extension and add '.img'
         if fname.endswith('.qcow2'):
-            fname = fname[:-5] + "img"
-        else:
-            fname += ".img"
+            fname = fname[:-6]
+        fname += ".img"
 
         self.logger.debug("Converting downloaded image from qcow2 to raw")
-        subprocess.check_call(['qemu-img', 'convert', '-f', 'qcow2',
-                               '-O', 'raw', origin, fname])
+        try:
+            subprocess.check_output(['qemu-img', 'convert',
+                                     '-f', 'qcow2',
+                                     '-O', 'raw', origin, fname],
+                                    stderr=subprocess.STDOUT)
+        except subprocess.CalledProcessError as exc:
+            self.logger.error("Unable to convert the qcow2 image")
+            self.logger.error(exc.output)
+            raise JobError(exc.output)
+
         self.set_namespace_data(action=self.name, label=self.key, key='file', value=fname)
         self.set_namespace_data(action=self.name, label='file', key=self.key, value=fname)
         return connection
