@@ -18,7 +18,8 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-from lava_dispatcher.pipeline.action import Pipeline
+import os
+from lava_dispatcher.pipeline.action import Pipeline, InfrastructureError, Action
 from lava_dispatcher.pipeline.logical import Deployment
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.deploy.download import (
@@ -31,6 +32,10 @@ from lava_dispatcher.pipeline.actions.deploy.overlay import (
     CustomisationAction,
     OverlayAction,
 )
+from lava_dispatcher.pipeline.actions.deploy.apply_overlay import ApplyOverlayTftp
+from lava_dispatcher.pipeline.utils.compression import untar_file
+
+# pylint: disable=too-many-instance-attributes
 
 
 class DeployImagesAction(DeployAction):  # FIXME: Rename to DeployPosixImages
@@ -46,17 +51,13 @@ class DeployImagesAction(DeployAction):  # FIXME: Rename to DeployPosixImages
         path = self.mkdtemp()
         if 'uefi' in parameters:
             uefi_path = self.mkdtemp()
-            download = DownloaderAction('uefi', uefi_path)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
+            self.internal_pipeline.add_action(DownloaderAction('uefi', uefi_path))
             # uefi option of QEMU needs a directory, not the filename
             self.set_namespace_data(action=self.name, label='image', key='uefi_dir', value=uefi_path, parameters=parameters)
             # alternatively use the -bios option and standard image args
         for image in parameters['images'].keys():
             if image != 'yaml_line':
-                download = DownloaderAction(image, path)
-                download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-                self.internal_pipeline.add_action(download)
+                self.internal_pipeline.add_action(DownloaderAction(image, path))
                 if parameters['images'][image].get('format', '') == 'qcow2':
                     self.internal_pipeline.add_action(QCowConversionAction(image))
         if self.test_needs_overlay(parameters):
@@ -67,34 +68,16 @@ class DeployImagesAction(DeployAction):  # FIXME: Rename to DeployPosixImages
             self.internal_pipeline.add_action(DeployDeviceEnvironment())
 
 
-class DeployMonitoredAction(DeployAction):
-
-    def __init__(self):
-        super(DeployMonitoredAction, self).__init__()
-        self.name = 'deploy-monitor'
-        self.description = "deploy images without POSIX"
-        self.summary = "deploy without requiring POSIX"
-
-    def populate(self, parameters):
-        self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-        path = self.mkdtemp()
-        for image in parameters['images'].keys():
-            if image != 'yaml_line':
-                download = DownloaderAction(image, path)
-                download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-                self.internal_pipeline.add_action(download)
-
-
-class DeployMonitoredPyOCD(Deployment):
+class DeployQemuNfs(Deployment):
     """
-    Strategy class for a PyOCD deployment not using
-    the POSIX Lava Test Shell overlays.
+    Strategy class for a kernel & NFS QEMU deployment.
+    Does not use GuestFS, adds overlay to the NFS
     """
-    compatibility = 4
+    compatibility = 5
 
     def __init__(self, parent, parameters):
-        super(DeployMonitoredPyOCD, self).__init__(parent)
-        self.action = DeployMonitoredAction()
+        super(DeployQemuNfs, self).__init__(parent)
+        self.action = DeployQemuNfsAction()
         self.action.section = self.action_type
         self.action.job = self.job
         parent.add_action(self.action, parameters)
@@ -107,21 +90,108 @@ class DeployMonitoredPyOCD(Deployment):
         This is *not* the same as validation of the action
         which can use instance data.
         """
-        if 'pyocd' not in device['actions']['deploy']['methods']:
+        if 'nfs' not in device['actions']['deploy']['methods']:
             return False
-        if parameters['to'] != 'tmpfs':
+        if parameters['to'] != 'nfs':
             return False
-        # lookup if the job parameters match the available device methods
-        if 'images' not in parameters:
-            # python3 compatible
-            # FIXME: too broad
-            print("Parameters %s have not been implemented yet." % list(parameters.keys()))  # pylint: disable=superfluous-parens
-            return False
-        if 'type' not in parameters.keys():
+        if 'qemu-nfs' not in device['actions']['boot']['methods']:
             return False
+        if 'type' in parameters:
+            if parameters['type'] != 'monitor':
+                return False
         return True
 
 
+class DeployQemuNfsAction(DeployAction):
+
+    def __init__(self):
+        super(DeployQemuNfsAction, self).__init__()
+        self.name = 'deploy-qemu-nfs'
+        self.description = "deploy qemu with NFS"
+        self.summary = "deploy NFS for QEMU"
+
+    def populate(self, parameters):
+        self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
+        path = self.mkdtemp()
+        if 'uefi' in parameters:
+            uefi_path = self.mkdtemp()
+            self.internal_pipeline.add_action(DownloaderAction('uefi', uefi_path))
+            # uefi option of QEMU needs a directory, not the filename
+            self.set_namespace_data(action=self.name, label='image', key='uefi_dir', value=uefi_path, parameters=parameters)
+            # alternatively use the -bios option and standard image args
+        for image in parameters['images'].keys():
+            if image != 'yaml_line':
+                self.internal_pipeline.add_action(DownloaderAction(image, path))
+                if parameters['images'][image].get('format', '') == 'qcow2':
+                    self.internal_pipeline.add_action(QCowConversionAction(image))
+        self.internal_pipeline.add_action(ExtractNfsAction())
+        self.internal_pipeline.add_action(OverlayAction())
+        self.internal_pipeline.add_action(ApplyOverlayTftp())
+        self.internal_pipeline.add_action(DeployDeviceEnvironment())
+
+
+class ExtractNfsAction(Action):
+
+    def __init__(self):
+        super(ExtractNfsAction, self).__init__()
+        self.name = "qemu-nfs-deploy"
+        self.description = "deploy nfsrootfs for QEMU"
+        self.summary = "NFS deployment for QEMU"
+        self.param_key = 'nfsrootfs'
+        self.file_key = "nfsroot"
+        self.extra_compression = ['xz']
+        self.use_tarfile = True
+        self.use_lzma = False
+
+    def validate(self):
+        super(ExtractNfsAction, self).validate()
+        if not self.valid:
+            return
+        lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
+        lava_test_results_dir = lava_test_results_dir % self.job.job_id
+        self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir)
+        if not self.parameters['images'].get(self.param_key, None):  # idempotency
+            return
+        if not self.get_namespace_data(
+                action='download-action', label=self.param_key, key='file'):
+            self.errors = "no file specified extract as %s" % self.param_key
+        if not os.path.exists('/usr/sbin/exportfs'):
+            raise InfrastructureError("NFS job requested but nfs-kernel-server not installed.")
+        if 'prefix' in self.parameters['images'][self.param_key]:
+            prefix = self.parameters['images'][self.param_key]['prefix']
+            if prefix.startswith('/'):
+                self.errors = 'prefix must not be an absolute path'
+            if not prefix.endswith('/'):
+                self.errors = 'prefix must be a directory and end with /'
+
+    def run(self, connection, max_end_time, args=None):
+        if not self.parameters['images'].get(self.param_key, None):  # idempotency
+            return connection
+        connection = super(ExtractNfsAction, self).run(connection, max_end_time, args)
+        root = self.get_namespace_data(action='download-action', label=self.param_key, key='file')
+        root_dir = self.mkdtemp()
+        untar_file(root, root_dir)
+        self.set_namespace_data(action='extract-rootfs', label='file', key=self.file_key, value=root_dir)
+        self.logger.debug("Extracted %s to %s", self.file_key, root_dir)
+
+        if 'prefix' in self.parameters['images'][self.param_key]:
+            prefix = self.parameters['images'][self.param_key]['prefix']
+            self.logger.warning("Adding '%s' prefix, any other content will not be visible.",
+                                prefix)
+
+            # Grab the path already defined in super().run() and add the prefix
+            root_dir = self.get_namespace_data(
+                action='extract-rootfs',
+                label='file',
+                key=self.file_key
+            )
+            root_dir = os.path.join(root_dir, prefix)
+            # sets the directory into which the overlay is unpacked and which
+            # is used in the substitutions into the bootloader command string.
+            self.set_namespace_data(action='extract-rootfs', label='file', key=self.file_key, value=root_dir)
+        return connection
+
+
 # FIXME: needs to be renamed to DeployPosixImages
 class DeployImages(Deployment):
     """
@@ -132,11 +202,11 @@ class DeployImages(Deployment):
     Does not boot the device.
     Requires guestfs instead of loopback support.
     Prepares the following actions and pipelines:
-        retry_pipeline
-            download_action
-        report_checksum_action
-        customisation_action
-        test_definitions_action
+        retry-pipeline
+            download-action
+        report-checksum-action
+        customisation-action
+        test-definitions-action
     """
     compatibility = 4
 
