@@ -22,9 +22,10 @@ import os
 import re
 import glob
 from lava_dispatcher.pipeline.action import (
+    Action,
     JobError,
     InfrastructureError,
-    Action,
+    LAVABug,
     Pipeline
 )
 from lava_dispatcher.pipeline.logical import RetryAction
@@ -43,22 +44,22 @@ class OffsetAction(DeployAction):
 
     def __init__(self, key):
         super(OffsetAction, self).__init__()
-        self.name = "offset_action"
+        self.name = "offset-action"
         self.description = "calculate offset of the image"
         self.summary = "offset calculation"
         self.key = key
 
     def validate(self):
         super(OffsetAction, self).validate()
-        if not self.get_namespace_data(action='download_action', label=self.key, key='file'):
+        if not self.get_namespace_data(action='download-action', label=self.key, key='file'):
             self.errors = "no file specified to calculate offset"
 
-    def run(self, connection, args=None):
-        if self.get_namespace_data(action='download_action', label=self.key, key='offset'):
+    def run(self, connection, max_end_time, args=None):
+        if self.get_namespace_data(action='download-action', label=self.key, key='offset'):
             # idempotency
             return connection
-        connection = super(OffsetAction, self).run(connection, args)
-        image = self.get_namespace_data(action='download_action', label=self.key, key='file')
+        connection = super(OffsetAction, self).run(connection, max_end_time, args)
+        image = self.get_namespace_data(action='download-action', label=self.key, key='file')
         if not os.path.exists(image):
             raise JobError("Not able to mount %s: file does not exist" % image)
         part_data = self.run_command([
@@ -91,7 +92,7 @@ class LoopCheckAction(DeployAction):
 
     def __init__(self, key):
         super(LoopCheckAction, self).__init__()
-        self.name = "loop_check"
+        self.name = "loop-check"
         self.description = "ensure a loop back mount operation is possible"
         self.summary = "check available loop back support"
         self.key = key
@@ -104,10 +105,10 @@ class LoopCheckAction(DeployAction):
         available_loops = len(glob.glob('/sys/block/loop*'))
         self.set_namespace_data(action=self.name, label=self.key, key='available_loops', value=available_loops)
 
-    def run(self, connection, args=None):
-        connection = super(LoopCheckAction, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(LoopCheckAction, self).run(connection, max_end_time, args)
         if not self.get_namespace_data(action=self.name, label=self.key, key='available_loops'):
-            raise RuntimeError("Unable to check available loop devices")
+            raise LAVABug("Unable to check available loop devices")
         args = ['/sbin/losetup', '-a']
         pro = self.run_command(args)
         mounted_loops = len(pro.strip().split("\n")) if pro else 0
@@ -131,7 +132,7 @@ class LoopMountAction(RetryAction):
 
     def __init__(self, key):
         super(LoopMountAction, self).__init__()
-        self.name = "loop_mount"
+        self.name = "loop-mount"
         self.description = "Mount using a loopback device and offset"
         self.summary = "loopback mount"
         self.retries = 10
@@ -144,30 +145,31 @@ class LoopMountAction(RetryAction):
         lava_test_results_base = self.parameters['deployment_data']['lava_test_results_dir']
         lava_test_results_dir = lava_test_results_base % self.job.job_id
         self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir)
-        if not self.get_namespace_data(action='download_action', label=self.key, key='file'):
+        if not self.get_namespace_data(action='download-action', label=self.key, key='file'):
             self.errors = "no file specified to mount"
 
-    def run(self, connection, args=None):
-        connection = super(LoopMountAction, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(LoopMountAction, self).run(connection, max_end_time, args)
         self.mntdir = mkdtemp(autoremove=False)
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
-        test_mntdir = os.path.abspath("%s/%s" % (mntdir, lava_test_results_dir))
-        self.set_namespace_data(action=self.name, label='mntdir', key=mntdir)
-        self.set_namespace_data(action='mount_action', label='mntdir', key=test_mntdir)
-        offset = self.get_namespace_data(action='download_action', label=self.key, key='offset')
+        test_mntdir = os.path.abspath("%s/%s" % (self.mntdir, lava_test_results_dir))
+        self.set_namespace_data(action=self.name, label='mntdir', key='mntdir', value=self.mntdir)
+        self.set_namespace_data(action='mount-action', label='mntdir', key='mntdir', value=test_mntdir)
+        offset = self.get_namespace_data(action='download-action', label=self.key, key='offset')
         mount_cmd = [
             'mount',
             '-o',
             'loop,offset=%s' % offset,
-            self.get_namespace_data(action='download_action', label=self.key, key='file'),
-            mntdir
+            self.get_namespace_data(action='download-action', label=self.key, key='file'),
+            self.mntdir
         ]
         command_output = self.run_command(mount_cmd)
         if command_output and command_output is not '':
             raise JobError("Unable to mount: %s" % command_output)  # FIXME: JobError needs a unit test
         return connection
 
-    def cleanup(self):
+    def cleanup(self, connection):
+        super(LoopMountAction, self).cleanup(connection)
         self.logger.debug("%s cleanup", self.name)
         if self.mntdir:
             if os.path.ismount(self.mntdir):
@@ -185,7 +187,7 @@ class MountAction(DeployAction):
 
     def __init__(self, key):
         super(MountAction, self).__init__()
-        self.name = "mount_action"
+        self.name = "mount-action"
         self.description = "mount with offset"
         self.summary = "mount loop"
         self.key = key
@@ -197,7 +199,7 @@ class MountAction(DeployAction):
         as part of the deployment selection step.
         """
         if not self.job:
-            raise RuntimeError("No job object supplied to action")
+            raise LAVABug("No job object supplied to action")
         # FIXME: not all mount operations will need these actions
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.internal_pipeline.add_action(OffsetAction(self.key))
@@ -227,10 +229,10 @@ class Unmount(Action):
         self.description = "unmount the test image at end of deployment"
         self.summary = "unmount image"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         """
         rmtree is not a cleanup action - it needs to be umounted first.
         """
-        connection = super(Unmount, self).run(connection, args)
+        connection = super(Unmount, self).run(connection, max_end_time, args)
         # mntdir was never being set correctly
         return connection
