@@ -21,7 +21,7 @@
 # List just the subclasses supported for this base strategy
 # imported by the parser to populate the list of subclasses.
 
-from lava_dispatcher.pipeline.action import Pipeline
+from lava_dispatcher.pipeline.action import ConfigurationError, Pipeline
 from lava_dispatcher.pipeline.logical import Deployment
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.deploy.download import DownloaderAction
@@ -46,11 +46,11 @@ def nfs_accept(device, parameters):
     if not device:
         return False
     if 'actions' not in device:
-        raise RuntimeError("Invalid device configuration")
+        raise ConfigurationError("Invalid device configuration")
     if 'deploy' not in device['actions']:
         return False
     if 'methods' not in device['actions']['deploy']:
-        raise RuntimeError("Device misconfiguration")
+        raise ConfigurationError("Device misconfiguration")
     return True
 
 
@@ -73,6 +73,8 @@ class Nfs(Deployment):
     def accepts(cls, device, parameters):
         if not nfs_accept(device, parameters):
             return False
+        if 'image' in device['actions']['deploy']['methods']:
+            return False
         if 'nfs' in device['actions']['deploy']['methods']:
             return True
         return False
@@ -90,26 +92,24 @@ class NfsAction(DeployAction):  # pylint:disable=too-many-instance-attributes
         super(NfsAction, self).validate()
         if not self.valid:
             return
-        if 'nfsrootfs' in self.parameters and 'nfs_url' in self.parameters:
-            self.errors = "Only one of nfsrootfs or nfs_url can be specified"
-        lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
-        lava_test_results_dir = lava_test_results_dir % self.job.job_id
-        self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir)
+        if 'nfsrootfs' in self.parameters and 'persistent_nfs' in self.parameters:
+            self.errors = "Only one of nfsrootfs or persistent_nfs can be specified"
+        if self.test_needs_deployment(self.parameters):
+            lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
+            lava_test_results_dir = lava_test_results_dir % self.job.job_id
+            self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir)
 
     def populate(self, parameters):
         download_dir = self.mkdtemp()
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         if 'nfsrootfs' in parameters:
-            download = DownloaderAction('nfsrootfs', path=download_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
+            self.internal_pipeline.add_action(DownloaderAction('nfsrootfs', path=download_dir))
         if 'modules' in parameters:
-            download = DownloaderAction('modules', path=download_dir)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
+            self.internal_pipeline.add_action(DownloaderAction('modules', path=download_dir))
         # NfsAction is a deployment, so once the nfsrootfs has been deployed, just do the overlay
         self.internal_pipeline.add_action(ExtractNfsRootfs())
         self.internal_pipeline.add_action(OverlayAction())
         self.internal_pipeline.add_action(ExtractModules())
         self.internal_pipeline.add_action(ApplyOverlayTftp())
-        self.internal_pipeline.add_action(DeployDeviceEnvironment())
+        if self.test_needs_deployment(parameters):
+            self.internal_pipeline.add_action(DeployDeviceEnvironment())
