@@ -24,7 +24,12 @@ import glob
 import shutil
 import tarfile
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
-from lava_dispatcher.pipeline.action import Action, Pipeline
+from lava_dispatcher.pipeline.action import (
+    Action,
+    InfrastructureError,
+    LAVABug,
+    Pipeline
+)
 from lava_dispatcher.pipeline.actions.deploy.testdef import (
     TestDefinitionAction,
     get_test_action_namespaces,
@@ -44,8 +49,8 @@ class CustomisationAction(DeployAction):
         self.description = "customise image during deployment"
         self.summary = "customise image"
 
-    def run(self, connection, args=None):
-        connection = super(CustomisationAction, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(CustomisationAction, self).run(connection, max_end_time, args)
         self.logger.debug("Customising image...")
         # FIXME: implement
         return connection
@@ -132,7 +137,7 @@ class OverlayAction(DeployAction):
             self.internal_pipeline.add_action(CompressOverlay())
             self.internal_pipeline.add_action(PersistentNFSOverlay())  # idempotent
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals
         """
         Check if a lava-test-shell has been requested, implement the overlay
         * create test runner directories beneath the temporary location
@@ -142,7 +147,7 @@ class OverlayAction(DeployAction):
         namespace = self.parameters.get('namespace', None)
         if namespace:
             if namespace not in get_test_action_namespaces(self.job.parameters):
-                self.logger.debug("skipped %s", self.name)
+                self.logger.info("[%s] skipped %s - no test action.", namespace, self.name)
                 return connection
         self.set_namespace_data(action='test', label='shared', key='location', value=tmp_dir)
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
@@ -158,7 +163,13 @@ class OverlayAction(DeployAction):
         for fname in self.scripts_to_copy:
             with open(fname, 'r') as fin:
                 output_file = '%s/bin/%s' % (lava_path, os.path.basename(fname))
-                self.logger.debug("Creating %s", output_file)
+                if os.path.exists(output_file):
+                    self.logger.debug(
+                        "Applying distro script: distro/%s/%s",
+                        self.parameters['deployment_data']['distro'],
+                        os.path.basename(fname))
+                else:
+                    self.logger.debug("Creating %s", output_file)
                 with open(output_file, 'w') as fout:
                     fout.write("#!%s\n\n" % shell)
                     fout.write(fin.read())
@@ -186,7 +197,7 @@ class OverlayAction(DeployAction):
                         continue
                     fout.write("%s=%s\n" % (key, value))
 
-        connection = super(OverlayAction, self).run(connection, args)
+        connection = super(OverlayAction, self).run(connection, max_end_time, args)
         return connection
 
 
@@ -229,7 +240,7 @@ class MultinodeOverlayAction(OverlayAction):
         for script in glob.glob(os.path.join(self.lava_v2_multi_node_test_dir, 'lava-*')):
             self.v2_scripts_to_copy.append(script)
 
-    def run(self, connection, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
         if self.role is None:
             self.logger.debug("skipped %s", self.name)
             return connection
@@ -237,14 +248,14 @@ class MultinodeOverlayAction(OverlayAction):
         shell = self.get_namespace_data(action='test', label='shared', key='lava_test_sh_cmd')
         location = self.get_namespace_data(action='test', label='shared', key='location')
         if not location:
-            raise RuntimeError("Missing lava overlay location")
+            raise LAVABug("Missing lava overlay location")
         if not os.path.exists(location):
-            raise RuntimeError("Unable to find overlay location")
+            raise LAVABug("Unable to find overlay location")
 
         # the roles list can only be populated after the devices have been assigned
         # therefore, cannot be checked in validate which is executed at submission.
         if 'roles' not in self.job.parameters['protocols'][self.protocol]:
-            raise RuntimeError("multinode definition without complete list of roles after assignment")
+            raise LAVABug("multinode definition without complete list of roles after assignment")
 
         # Generic scripts
         lava_path = os.path.abspath("%s/%s" % (location, lava_test_results_dir))
@@ -353,8 +364,7 @@ class VlandOverlayAction(OverlayAction):
                     interface,
                     device_params[interface]['mac'],
                     device_params[interface]['sysfs'],
-                ])
-            )
+                ]))
         for interface in device_params:
             if not device_params[interface]['tags']:
                 # skip primary interface
@@ -363,7 +373,7 @@ class VlandOverlayAction(OverlayAction):
                 self.tags.append(",".join([interface, tag]))
 
     # pylint: disable=anomalous-backslash-in-string
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         """
         Writes out file contents from lists, across multiple lines
         VAR="VAL1\n\
@@ -381,9 +391,9 @@ class VlandOverlayAction(OverlayAction):
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
         shell = self.get_namespace_data(action='test', label='shared', key='lava_test_sh_cmd')
         if not location:
-            raise RuntimeError("Missing lava overlay location")
+            raise LAVABug("Missing lava overlay location")
         if not os.path.exists(location):
-            raise RuntimeError("Unable to find overlay location")
+            raise LAVABug("Unable to find overlay location")
 
         lava_path = os.path.abspath("%s/%s" % (location, lava_test_results_dir))
         scripts_to_copy = glob.glob(os.path.join(self.lava_vland_test_dir, 'lava-*'))
@@ -430,20 +440,20 @@ class CompressOverlay(Action):
         self.summary = "Compress the lava overlay files"
         self.description = "Create a lava overlay tarball and store alongside the job"
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         output = os.path.join(self.job.parameters['output_dir'],
                               "overlay-%s.tar.gz" % self.level)
         location = self.get_namespace_data(action='test', label='shared', key='location')
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
         self.set_namespace_data(action='test', label='shared', key='output', value=output)
         if not location:
-            raise RuntimeError("Missing lava overlay location")
+            raise LAVABug("Missing lava overlay location")
         if not os.path.exists(location):
-            raise RuntimeError("Unable to find overlay location")
+            raise LAVABug("Unable to find overlay location")
         if not self.valid:
             self.logger.error(self.errors)
             return connection
-        connection = super(CompressOverlay, self).run(connection, args)
+        connection = super(CompressOverlay, self).run(connection, max_end_time, args)
         cur_dir = os.getcwd()
         try:
             with tarfile.open(output, "w:gz") as tar:
@@ -453,8 +463,7 @@ class CompressOverlay(Action):
                 if os.path.exists('./root/'):
                     tar.add(".%s" % '/root/')
         except tarfile.TarError as exc:
-            self.errors = "Unable to create lava overlay tarball: %s" % exc
-            raise RuntimeError("Unable to create lava overlay tarball: %s" % exc)
+            raise InfrastructureError("Unable to create lava overlay tarball: %s" % exc)
         os.chdir(cur_dir)
         self.set_namespace_data(action=self.name, label='output', key='file', value=output)
         return connection
@@ -504,8 +513,8 @@ class SshAuthorize(Action):
                 # only secondary connections set active.
                 self.active = True
 
-    def run(self, connection, args=None):
-        connection = super(SshAuthorize, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(SshAuthorize, self).run(connection, max_end_time, args)
         if not self.identity_file:
             self.logger.debug("No authorisation required.")  # idempotency
             return connection
@@ -513,9 +522,9 @@ class SshAuthorize(Action):
         location = self.get_namespace_data(action='test', label='shared', key='location')
         lava_test_results_dir = self.get_namespace_data(action='test', label='results', key='lava_test_results_dir')
         if not location:
-            raise RuntimeError("Missing lava overlay location")
+            raise LAVABug("Missing lava overlay location")
         if not os.path.exists(location):
-            raise RuntimeError("Unable to find overlay location")
+            raise LAVABug("Unable to find overlay location")
         lava_path = os.path.abspath("%s/%s" % (location, lava_test_results_dir))
         output_file = '%s/%s' % (lava_path, os.path.basename(self.identity_file))
         shutil.copyfile(self.identity_file, output_file)
@@ -552,13 +561,17 @@ class PersistentNFSOverlay(Action):
 
     def validate(self):
         super(PersistentNFSOverlay, self).validate()
-        if 'nfs_url' not in self.parameters:
-            return None
-        if ':' not in self.parameters['nfs_url']:
-            self.errors = "Unrecognised NFS URL: '%s'" % self.parameters['nfs_url']
+        persist = self.parameters.get('persistent_nfs', None)
+        if not persist:
             return
-        nfs_server, dirname = self.parameters['nfs_url'].split(':')
+        if 'address' not in persist:
+            self.errors = "Missing address for persistent NFS"
+            return
+        if ':' not in persist['address']:
+            self.errors = "Unrecognised NFS URL: '%s'" % self.parameters['persistent_nfs']['address']
+            return
+        nfs_server, dirname = persist['address'].split(':')
         self.errors = infrastructure_error('rpcinfo')
         self.errors = rpcinfo_nfs(nfs_server)
-        self.set_namespace_data(action=self.name, label='nfs_url', key='nfsroot', value=dirname)
-        self.set_namespace_data(action=self.name, label='nfs_url', key='serverip', value=nfs_server)
+        self.set_namespace_data(action=self.name, label='nfs_address', key='nfsroot', value=dirname)
+        self.set_namespace_data(action=self.name, label='nfs_address', key='serverip', value=nfs_server)
