@@ -42,7 +42,7 @@ from lava_dispatcher.pipeline.actions.deploy.environment import DeployDeviceEnvi
 from lava_dispatcher.pipeline.utils.network import dispatcher_ip
 from lava_dispatcher.pipeline.utils.strings import substitute
 from lava_dispatcher.pipeline.utils.constants import (
-    SECONDARY_DEPLOYMENT_MSG,
+    DD_PROMPTS,
 )
 
 
@@ -70,27 +70,24 @@ class Removable(Deployment):
 
     @classmethod
     def accepts(cls, device, parameters):
-        job_device = None
-        media = None
-        if 'to' in parameters:
-            # connection support
-            # Which deployment method to use?
-            if parameters['to'] == 'usb':
-                if 'device' in parameters:
-                    job_device = parameters['device']
-                    media = 'usb'
-            if parameters['to'] == 'sata':
-                if 'device' in parameters:
-                    job_device = parameters['device']
-                    media = 'sata'
-        # Matching a method ?
+        media = parameters.get('to', None)
+        job_device = parameters.get('device', None)
+
+        # Is the media supported?
+        if media not in ['sata', 'sd', 'usb']:
+            return False
+        # Matching a method?
         if job_device is None:
             return False
-        # Is the device allowing this method ?
-        if job_device not in device['parameters']['media'][media]:
+        # "parameters.media" is not defined for every devices
+        if 'parameters' not in device or 'media' not in device['parameters']:
             return False
-        # TODO: what if the key does not exist for this device configuration?
-        if 'uuid' in device['parameters']['media'][media][job_device]:
+
+        # Is the device allowing this method?
+        if job_device not in device['parameters']['media'].get(media, {}):
+            return False
+        # Is the configuration correct?
+        if 'uuid' in device['parameters']['media'][media].get(job_device, {}):
             return True
         return False
 
@@ -108,6 +105,8 @@ class DDAction(Action):
         self.description = "deploy image to drive"
         self.timeout = Timeout(self.name, 600)
         self.boot_params = None
+        self.dd_prompts = None
+        self.dd_flags = None
 
     def validate(self):
         super(DDAction, self).validate()
@@ -121,14 +120,32 @@ class DDAction(Action):
             self.errors = "missing prompt for download tool"
         if not os.path.isabs(self.parameters['download']['tool']):
             self.errors = "download tool parameter needs to be an absolute path"
+
+        if self.parameters['to'] not in self.job.device['parameters'].get('media', {}):
+            self.errors = "media '%s' unavailable for this device" % self.parameters['to']
+
+        # No need to go further if an error was already detected
+        if not self.valid:
+            return
+
+        dd_params = self.parameters.get('dd', None)
+        if dd_params:
+            self.dd_prompts = self.parameters['dd'].get('prompts', DD_PROMPTS)
+            self.dd_flags = self.parameters['dd'].get('flags', None)
+        else:
+            self.dd_prompts = DD_PROMPTS
+
+        if not isinstance(self.dd_prompts, list):
+            self.errors = "'dd prompts' should be a list"
+        else:
+            for msg in self.dd_prompts:
+                if not msg:
+                    self.errors = "items of 'dd prompts' cannot be empty"
+
         uuid_required = False
         self.boot_params = self.job.device['parameters']['media'][self.parameters['to']]
-        if 'media' in self.job.device:
-            media_params = self.job.device['parameters']['media']
-            interface_params = [
-                interface for interface in media_params if interface == self.parameters['to']
-            ]
-            uuid_required = media_params[interface_params[0]]['UUID-required']
+        uuid_required = self.boot_params.get('UUID-required', False)
+
         if uuid_required:  # FIXME unit test required
             if 'uuid' not in self.boot_params[self.parameters['device']]:
                 self.errors = "A UUID is required for %s on %s" % (
@@ -143,18 +160,18 @@ class DDAction(Action):
                 value=self.boot_params[self.parameters['device']]['device_id']
             )
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):  # pylint: disable=too-many-locals
         """
         Retrieve the decompressed image from the dispatcher by calling the tool specified
         by the test writer, from within the test image of the first deployment, using the
         device to write directly to the secondary media, without needing to cache on the device.
         """
-        connection = super(DDAction, self).run(connection, args)
-        file = self.get_namespace_data(action='download_action', label='image', key='file')
-        if not file:
+        connection = super(DDAction, self).run(connection, max_end_time, args)
+        d_file = self.get_namespace_data(action='download-action', label='image', key='file')
+        if not d_file:
             self.logger.debug("Skipping %s - nothing downloaded")
             return connection
-        decompressed_image = os.path.basename(file)
+        decompressed_image = os.path.basename(d_file)
         try:
             device_path = os.path.realpath(
                 "/dev/disk/by-id/%s" %
@@ -170,8 +187,9 @@ class DDAction(Action):
         # As the test writer can use any tool we cannot predict where the
         # download URL will be positioned in the download command.
         # Providing the download URL as a substitution option gets round this
+        ip_addr = dispatcher_ip(self.job.parameters['dispatcher'])
         download_url = "http://%s/%s/%s" % (
-            dispatcher_ip(), suffix, decompressed_image
+            ip_addr, suffix, decompressed_image
         )
         substitutions = {
             '{DOWNLOAD_URL}': download_url
@@ -181,20 +199,20 @@ class DDAction(Action):
             self.parameters['download']['tool'], download_options
         )
         dd_cmd = "dd of='%s' bs=4M" % device_path  # busybox dd does not support other flags
+        if self.dd_flags:
+            dd_cmd = "%s %s" % (dd_cmd, self.dd_flags)
 
         # set prompt to download prompt to ensure that the secondary deployment has started
         prompt_string = connection.prompt_str
         connection.prompt_str = self.parameters['download']['prompt']
         self.logger.debug("Changing prompt to %s", connection.prompt_str)
-        connection.sendline("%s | %s ; echo %s" % (download_cmd, dd_cmd, SECONDARY_DEPLOYMENT_MSG))
+        connection.sendline("%s | %s" % (download_cmd, dd_cmd))
         self.wait(connection)
         if not self.valid:
             self.logger.error(self.errors)
 
-        # We must ensure that the secondary media deployment has completed before handing over
-        # the connection.  Echoing the SECONDARY_DEPLOYMENT_MSG after the deployment means we
-        # always have a constant string to match against
-        connection.prompt_str = SECONDARY_DEPLOYMENT_MSG
+        # change prompt string to list of dd outputs
+        connection.prompt_str = self.dd_prompts
         self.logger.debug("Changing prompt to %s", connection.prompt_str)
         self.wait(connection)
 
@@ -223,12 +241,14 @@ class MassStorage(DeployAction):  # pylint: disable=too-many-instance-attributes
             self.errors = "No device specified for mass storage deployment"
         if not self.valid:
             return
+
         if self.test_needs_deployment(self.parameters):
             lava_test_results_dir = self.parameters['deployment_data']['lava_test_results_dir']
-            self.set_namespace_data(action='lava-test-shell', label='shared', key='lava_test_results_dir', value=lava_test_results_dir % self.job.job_id)
-        if 'device' in self.parameters:
-            self.set_namespace_data(action=self.name, label='u-boot', key='device', value=self.parameters['device'])
+            self.set_namespace_data(action='test', label='results', key='lava_test_results_dir', value=lava_test_results_dir % self.job.job_id)
+
+        self.set_namespace_data(action=self.name, label='u-boot', key='device', value=self.parameters['device'])
         suffix = os.path.join(*self.image_path.split('/')[-2:])
+        suffix = os.path.join(suffix, "image")
         self.set_namespace_data(action=self.name, label='storage', key='suffix', value=suffix)
 
     def populate(self, parameters):
@@ -246,9 +266,7 @@ class MassStorage(DeployAction):  # pylint: disable=too-many-instance-attributes
         if self.test_needs_overlay(parameters):
             self.internal_pipeline.add_action(OverlayAction())  # idempotent, includes testdef
         if 'image' in parameters:
-            download = DownloaderAction('image', path=self.image_path)
-            download.max_retries = 3
-            self.internal_pipeline.add_action(download)
+            self.internal_pipeline.add_action(DownloaderAction('image', path=self.image_path))
             if self.test_needs_overlay(parameters):
                 self.internal_pipeline.add_action(ApplyOverlayImage())
             self.internal_pipeline.add_action(DDAction())
