@@ -23,10 +23,7 @@
 
 # pylint: disable=unused-import
 
-from lava_dispatcher.pipeline.actions.deploy.image import (
-    DeployImages,
-    DeployMonitoredPyOCD
-)
+from lava_dispatcher.pipeline.actions.deploy.image import DeployImages
 from lava_dispatcher.pipeline.actions.deploy.tftp import Tftp
 from lava_dispatcher.pipeline.actions.deploy.removable import MassStorage
 from lava_dispatcher.pipeline.actions.deploy.ssh import Ssh
@@ -34,3 +31,4 @@ from lava_dispatcher.pipeline.actions.deploy.fastboot import Fastboot
 from lava_dispatcher.pipeline.actions.deploy.lxc import Lxc
 from lava_dispatcher.pipeline.actions.deploy.iso import DeployIso
 from lava_dispatcher.pipeline.actions.deploy.nfs import Nfs
+from lava_dispatcher.pipeline.actions.deploy.vemsd import VExpressMsd
