@@ -24,12 +24,18 @@
 import os
 import tempfile
 
-from lava_dispatcher.pipeline.action import Pipeline
+from lava_dispatcher.pipeline.action import (
+    ConfigurationError,
+    JobError,
+    Pipeline
+)
 from lava_dispatcher.pipeline.logical import Deployment
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
+from lava_dispatcher.pipeline.actions.deploy.lxc import LxcAddDeviceAction
 from lava_dispatcher.pipeline.actions.deploy.download import DownloaderAction
 from lava_dispatcher.pipeline.actions.deploy.apply_overlay import PrepareOverlayTftp
 from lava_dispatcher.pipeline.actions.deploy.environment import DeployDeviceEnvironment
+from lava_dispatcher.pipeline.utils.constants import TFTP_SIZE_LIMIT
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.utils.filesystem import tftpd_dir
 
@@ -47,11 +53,11 @@ def tftp_accept(device, parameters):
     if not device:
         return False
     if 'actions' not in device:
-        raise RuntimeError("Invalid device configuration")
+        raise ConfigurationError("Invalid device configuration")
     if 'deploy' not in device['actions']:
         return False
     if 'methods' not in device['actions']['deploy']:
-        raise RuntimeError("Device misconfiguration")
+        raise ConfigurationError("Device misconfiguration")
     return True
 
 
@@ -96,8 +102,10 @@ class TftpAction(DeployAction):  # pylint:disable=too-many-instance-attributes
             self.errors = "%s needs a kernel to deploy" % self.name
         if not self.valid:
             return
-        if 'nfsrootfs' in self.parameters and 'nfs_url' in self.parameters:
-            self.errors = "Only one of nfsrootfs or nfs_url can be specified"
+        if 'nfs_url' in self.parameters:
+            self.errors = "Use a persistent_nfs dictionary instead of nfs_url"
+        if 'nfsrootfs' in self.parameters and 'persistent_nfs' in self.parameters:
+            self.errors = "Only one of nfsrootfs or persistent_nfs can be specified"
         if self.test_needs_deployment(self.parameters):
             lava_test_results_base = self.parameters['deployment_data']['lava_test_results_dir']
             lava_test_results_dir = lava_test_results_base % self.job.job_id
@@ -117,19 +125,35 @@ class TftpAction(DeployAction):  # pylint:disable=too-many-instance-attributes
             self.errors = "tftpd directory is not configured correctly, see /etc/default/tftpd-hpa"
 
     def populate(self, parameters):
-        self.tftp_dir = self.mkdtemp()
+        self.tftp_dir = self.mkdtemp(override=tftpd_dir())
         self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
         self.set_namespace_data(action=self.name, label='tftp', key='tftp_dir', value=self.tftp_dir, parameters=parameters)
 
         for key in ['ramdisk', 'kernel', 'dtb', 'nfsrootfs', 'modules', 'preseed']:
             if key in parameters:
-                download = DownloaderAction(key, path=self.tftp_dir)
-                download.max_retries = 3  # overridden by failure_retry in the parameters, if set.
-                self.internal_pipeline.add_action(download)
+                self.internal_pipeline.add_action(DownloaderAction(key, path=self.tftp_dir))
                 if key == 'ramdisk':
                     self.set_namespace_data(action=self.name, label='tftp', key='ramdisk', value=True, parameters=parameters)
 
         # TftpAction is a deployment, so once the files are in place, just do the overlay
         self.internal_pipeline.add_action(PrepareOverlayTftp())
+        self.internal_pipeline.add_action(LxcAddDeviceAction())
         if self.test_needs_deployment(parameters):
             self.internal_pipeline.add_action(DeployDeviceEnvironment())
+
+    def run(self, connection, max_end_time, args=None):
+        super(TftpAction, self).run(connection, max_end_time, args)
+        tftp_size_limit = self.job.parameters['dispatcher'].get('tftp_size_limit',
+                                                                TFTP_SIZE_LIMIT)
+        self.logger.debug("Checking files for TFTP limit of %s bytes.", tftp_size_limit)
+        for (action, key) in [('compress-ramdisk', 'ramdisk'),
+                              ('download-action', 'kernel'),
+                              ('download-action', 'dtb')]:
+            if key in self.parameters:
+                filename = self.get_namespace_data(action=action, label='file', key=key)
+                filename = os.path.join(tftpd_dir(), filename)
+                fsize = os.stat(filename).st_size
+                if fsize >= tftp_size_limit:
+                    raise JobError("Unable to send '%s' over tftp: file too large (%d > %d)" %
+                                   (os.path.basename(filename), fsize, tftp_size_limit))
+        return connection
