@@ -19,13 +19,11 @@
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
 
-import signal
 from lava_dispatcher.pipeline.action import JobError
 from lava_dispatcher.pipeline.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.action import Action
 from lava_dispatcher.pipeline.shell import ShellCommand, ShellSession
-from lava_dispatcher.pipeline.utils.constants import DEFAULT_SHELL_PROMPT
 
 
 # pylint: disable=too-many-public-methods,too-many-instance-attributes
@@ -127,16 +125,17 @@ class ConnectSsh(Action):
                 self.command.extend(['-i', self.identity_file])
             self.command.extend(self.ssh_port)
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         if connection:
             self.logger.debug("Already connected")
             return connection
-        signal.alarm(0)  # clear the timeouts used without connections.
         # ShellCommand executes the connection command
 
         params = self._check_params()
         command = self.command[:]  # local copy for idempotency
-        overrides = self.get_namespace_data(action='prepare-scp-overlay', label="prepare-scp-overlay", key=self.key)
+        overrides = None
+        if self.key:
+            overrides = self.get_namespace_data(action='prepare-scp-overlay', label="prepare-scp-overlay", key=self.key)
         host_address = None
         if overrides:
             host_address = str(self.get_namespace_data(
@@ -160,11 +159,12 @@ class ConnectSsh(Action):
                 self.command, shell.exitstatus, shell.readlines()))
         # SshSession monitors the pexpect
         connection = SShSession(self.job, shell)
-        connection = super(ConnectSsh, self).run(connection, args)
-        connection.sendline('export PS1="%s"' % DEFAULT_SHELL_PROMPT)
-        connection.prompt_str = [DEFAULT_SHELL_PROMPT]
+        connection = super(ConnectSsh, self).run(connection, max_end_time, args)
+        connection.sendline('export PS1="%s"' % self.job.device.get_constant(
+            'default-shell-prompt'))
+        connection.prompt_str = [self.job.device.get_constant(
+            'default-shell-prompt')]
         connection.connected = True
         self.wait(connection)
-        res = 'failed' if self.errors else 'success'
-        self.set_namespace_data(action='boot', label='shared', key='boot-result', value=res)
+        self.set_namespace_data(action='shared', label='shared', key='connection', value=connection)
         return connection
