@@ -21,7 +21,6 @@
 
 import datetime
 import logging
-import sys
 import yaml
 import zmq
 import zmq.auth
@@ -29,7 +28,7 @@ from zmq.utils.strtypes import b
 
 
 class ZMQPushHandler(logging.Handler):
-    def __init__(self, socket_addr, master_cert, slave_cert, job_id):
+    def __init__(self, logging_url, master_cert, slave_cert, job_id):
         super(ZMQPushHandler, self).__init__()
 
         # Create the PUSH socket
@@ -46,7 +45,7 @@ class ZMQPushHandler(logging.Handler):
             (server_public, _) = zmq.auth.load_certificate(master_cert)
             self.socket.curve_serverkey = server_public
 
-        self.socket.connect(socket_addr)
+        self.socket.connect(logging_url)
 
         self.job_id = str(job_id)
         self.action_level = '0'
@@ -74,8 +73,8 @@ class YAMLLogger(logging.Logger):
         super(YAMLLogger, self).__init__(name)
         self.handler = None
 
-    def addZMQHandler(self, socket_addr, master_cert, slave_cert, job_id):
-        self.handler = ZMQPushHandler(socket_addr, master_cert,
+    def addZMQHandler(self, logging_url, master_cert, slave_cert, job_id):
+        self.handler = ZMQPushHandler(logging_url, master_cert,
                                       slave_cert, job_id)
         self.addHandler(self.handler)
         return self.handler
@@ -93,10 +92,25 @@ class YAMLLogger(logging.Logger):
             data['msg'] = message % args
         else:
             data['msg'] = message
+
         # Set width to a really large value in order to always get one line.
-        self._log(level, yaml.dump(data, default_flow_style=True,
-                                   default_style='"',
-                                   width=sys.maxsize)[:-1], ())
+        # But keep this reasonable because the logs will be loaded by CLoader
+        # that is limited to around 10**7 chars
+        data_str = yaml.dump(data, default_flow_style=True,
+                             default_style='"',
+                             width=10 ** 6,
+                             Dumper=yaml.CDumper)[:-1]
+        # Test the limit and skip if the line is too long
+        if len(data_str) >= 10 ** 6:
+            if isinstance(message, str):
+                data['msg'] = "<line way too long ...>"
+            else:
+                data['msg'] = {"skip": "line way too long ..."}
+            data_str = yaml.dump(data, default_flow_style=True,
+                                 default_style='"',
+                                 width=10 ** 6,
+                                 Dumper=yaml.CDumper)[:-1]
+        self._log(level, data_str, ())
 
     def exception(self, exc, *args, **kwargs):
         self.log_message(logging.ERROR, 'exception', exc, *args, **kwargs)
@@ -113,8 +127,14 @@ class YAMLLogger(logging.Logger):
     def debug(self, message, *args, **kwargs):
         self.log_message(logging.DEBUG, 'debug', message, *args, **kwargs)
 
+    def input(self, message, *args, **kwargs):
+        self.log_message(logging.INFO, 'input', message, *args, **kwargs)
+
     def target(self, message, *args, **kwargs):
         self.log_message(logging.INFO, 'target', message, *args, **kwargs)
 
+    def feedback(self, message, *args, **kwargs):
+        self.log_message(logging.INFO, 'feedback', message, *args, **kwargs)
+
     def results(self, results, *args, **kwargs):
         self.log_message(logging.INFO, 'results', results, *args, **kwargs)
