@@ -21,7 +21,7 @@
 import re
 import pexpect
 from lava_dispatcher.pipeline.shell import ShellSession
-from lava_dispatcher.pipeline.action import Action, JobError
+from lava_dispatcher.pipeline.action import Action, JobError, LAVABug
 from lava_dispatcher.pipeline.connections.serial import ConnectDevice
 
 # pylint: disable=too-few-public-methods,too-many-branches
@@ -85,7 +85,7 @@ class SelectorMenu(object):
 
 class MenuSession(ShellSession):
 
-    def wait(self):
+    def wait(self, max_end_time=None):
         """
         Simple wait without sendling blank lines as that causes the menu
         to advance without data which can cause blank entries and can cause
@@ -117,23 +117,17 @@ class MenuConnect(ConnectDevice):
 
     def validate(self):
         hostname = self.job.device['hostname']
-        if self.job.device.power_state in ['on', 'off']:
-            # to enable power to a device, either power_on or hard_reset are needed.
-            if self.job.device.power_command is '':
-                self.errors = "Unable to power on or reset the device %s" % hostname
-            if self.job.device.connect_command is '':
-                self.errors = "Unable to connect to device %s" % hostname
-        else:
-            self.logger.warning("%s may need manual intervention to reboot", hostname)
+        if self.job.device.connect_command is '':
+            self.errors = "Unable to connect to device %s" % hostname
 
-    def run(self, connection, args=None):
-        connection = super(MenuConnect, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(MenuConnect, self).run(connection, max_end_time, args)
         if not connection:
-            raise RuntimeError("%s needs a Connection")
+            raise LAVABug("%s needs a Connection")
         connection.check_char = '\n'
         connection.sendline('\n')  # to catch the first prompt (remove for PDU?)
         connection.prompt_str = self.parameters['prompts']
-        if self.job.device.power_state not in ['on', 'off']:
+        if hasattr(self.job.device, 'power_state') and self.job.device.power_state not in ['on', 'off']:
             self.wait(connection)
         return connection
 
@@ -147,10 +141,10 @@ class MenuReset(ConnectDevice):
         self.summary = "reset to shell connection"
         self.description = "change out of menu session to a shell session"
 
-    def run(self, connection, args=None):
-        connection = super(MenuReset, self).run(connection, args)
+    def run(self, connection, max_end_time, args=None):
+        connection = super(MenuReset, self).run(connection, max_end_time, args)
         if not connection:
-            raise RuntimeError("%s needs a Connection")
+            raise LAVABug("%s needs a Connection")
 
         connection.check_char = '\n'
         connection.sendline('\n')  # to catch the first prompt (remove for PDU?)
@@ -166,6 +160,7 @@ class SelectorMenuAction(Action):
         self.description = 'select specified menu items'
         self.selector = SelectorMenu()
         self.items = []
+        self.line_sep = None
 
     def validate(self):
         super(SelectorMenuAction, self).validate()
@@ -186,7 +181,7 @@ class SelectorMenuAction(Action):
             self.logger.debug("Changing menu prompt to '%s'", connection.prompt_str)
             connection.wait()  # call MenuSession::wait directly for a tighter loop
 
-    def run(self, connection, args=None):
+    def run(self, connection, max_end_time, args=None):
         """
         iterate through the menu sequence:
         items: select
@@ -199,6 +194,7 @@ class SelectorMenuAction(Action):
         :param logger: Action logger
         :return: connection
         """
+        connection = super(SelectorMenuAction, self).run(connection, max_end_time, args)
         if not connection:
             self.logger.error("%s called without a Connection", self.name)
             return connection
@@ -217,6 +213,8 @@ class SelectorMenuAction(Action):
                             self.logger.debug("Selecting option %s", action)
                         elif 'fallback' in block['select']:
                             action = self.selector.select(menu_text, block['select']['fallback'])
+                        if not action:
+                            raise JobError("No selection was made")
                         connection.sendline(action, delay=self.character_delay)
                         self._change_prompt(connection, change_prompt)
                 if 'escape' in block['select']:
