@@ -29,15 +29,19 @@ import socket
 import time
 from lava_dispatcher.pipeline.connection import Protocol
 from lava_dispatcher.pipeline.action import (
-    Timeout,
-    JobError,
+    ConfigurationError,
     InfrastructureError,
-    TestError
+    JobError,
+    TestError,
+    MultinodeProtocolTimeoutError,
+    Timeout,
 )
 from lava_dispatcher.pipeline.utils.constants import LAVA_MULTINODE_SYSTEM_TIMEOUT
 
+# pylint: disable=misplaced-comparison-constant,too-many-branches
 
-class MultinodeProtocol(Protocol):
+
+class MultinodeProtocol(Protocol):  # pylint: disable=too-many-instance-attributes
     """
     Multinode API protocol - one instance per Multinode job
     """
@@ -93,7 +97,7 @@ class MultinodeProtocol(Protocol):
             try:
                 json_default = json.loads(jobdata)
             except ValueError as exc:
-                raise InfrastructureError("Invalid JSON settings for %s: %s" % (self.name, exc))
+                raise ConfigurationError("Invalid JSON settings for %s: %s" % (self.name, exc))
         if "port" in json_default:
             settings['port'] = json_default['port']
         if "blocksize" in json_default:
@@ -171,7 +175,7 @@ class MultinodeProtocol(Protocol):
         if isinstance(timeout, float):
             timeout = int(timeout)
         elif not isinstance(timeout, int):
-            raise RuntimeError("Invalid timeout duration type: %s %s" % (type(timeout), timeout))
+            raise ConfigurationError("Invalid timeout duration type: %s %s" % (type(timeout), timeout))
         msg_len = len(message)
         if msg_len > 0xFFFE:
             raise JobError("Message was too long to send!")
@@ -211,9 +215,20 @@ class MultinodeProtocol(Protocol):
             # apply the default timeout to each poll operation.
             if c_iter > timeout:
                 self.finalise_protocol()
-                raise JobError("protocol %s timed out" % self.name)
+                raise MultinodeProtocolTimeoutError(
+                    "protocol %s timed out" % self.name)
         return response
 
+    def configure(self, device, job):
+        """
+        Called by job.validate() to populate internal data
+        Returns True if configuration completed.
+        """
+        action_list = [action.section for action in job.pipeline.actions if action.section]
+        self.logger.debug("This MultiNode test job contains "
+                          "top level actions, in order, of: %s", ', '.join(action_list))
+        return True
+
     def set_up(self):
         """
         Called from the job at the start of the run step.
@@ -291,11 +306,13 @@ class MultinodeProtocol(Protocol):
         self._send(init_msg, True)
 
     def finalise_protocol(self, device=None):
-        fin_msg = {
-            "request": "clear_group",
-            "group_size": self.parameters['protocols'][self.name]['group_size']
-        }
-        self._send(fin_msg, True)
+        # If the protocol hasn't been setup correctly
+        if self.base_message is not None:
+            fin_msg = {
+                "request": "clear_group",
+                "group_size": self.parameters['protocols'][self.name]['group_size']
+            }
+            self._send(fin_msg, True)
         self.logger.debug("%s protocol finalised.", self.name)
 
     def _check_data(self, data):
@@ -329,7 +346,7 @@ class MultinodeProtocol(Protocol):
 
         return json_data
 
-    def _api_select(self, data):
+    def _api_select(self, data, action=None):
         """ Determines which API call has been requested, makes the call, blocks and returns the reply.
         :param json_data: Python object of the API call
         :return: Python object containing the reply dict.
@@ -381,9 +398,10 @@ class MultinodeProtocol(Protocol):
         else:
             return reply['response']
 
-    def __call__(self, args):
+    def __call__(self, *args, **kwargs):
+        # only the first argument is used.
         try:
-            return self._api_select(json.dumps(args))
+            return self._api_select(json.dumps(args[0]))
         except (ValueError, TypeError) as exc:
             msg = re.sub('\s+', ' ', ''.join(traceback.format_exc().split('\n')))
             logger = logging.getLogger("dispatcher")
@@ -466,6 +484,7 @@ class MultinodeProtocol(Protocol):
         and blocks until that messageID is available for this node
         """
         # use self.target as the node ID
+        self.logger.debug("request_wait %s", message_id)
         wait_msg = {"request": "lava_wait",
                     "messageID": message_id,
                     "nodeID": self.parameters['target']}
@@ -495,6 +514,7 @@ class MultinodeProtocol(Protocol):
         """
         Creates and send a message requesting lava_sync
         """
+        self.logger.debug("request_sync %s", msg)
         sync_msg = {"request": "lava_sync", "messageID": msg}
         return self._send(sync_msg)
 
@@ -503,6 +523,7 @@ class MultinodeProtocol(Protocol):
         Sends a message to the group via the Coordinator. All jobs with the matching role
         will receive the message and can then start the job.
         """
+        self.logger.debug("request_lava_start %s", message)
         send_msg = {"request": "lava_send",
                     "messageID": 'lava_start',
                     "message": message}
