@@ -223,7 +223,7 @@ class VlandProtocol(Protocol):
         reply = json.loads(response)
         if 'data' in reply:
             return reply['data']
-        raise RuntimeError(reply)
+        raise JobError("Deploy vlans failed for %s: %s" % (friendly_name, reply))
 
     def _declare_created(self, friendly_name, tag):
         if not self.configured:
@@ -406,7 +406,11 @@ class VlandProtocol(Protocol):
                     self.params[vlan_name]['iface'] = iface
                     self.nodes_seen.append(' '.join([device_info['switch'], str(device_info['port'])]))
                     break
-        self.logger.debug("[%s] vland params: %s", device['hostname'], self.params)
+        if self.sub_id == 0:
+            self.logger.info("[%s] Job will deploy requested VLANs.", self.name)
+        else:
+            self.logger.info("[%s] Job will wait for VLANs to be deployed.", self.name)
+        self.logger.debug("[%s] parameters: %s", self.name, self.params)
         super(VlandProtocol, self).configure(device, job)
         return True
 
@@ -427,15 +431,13 @@ class VlandProtocol(Protocol):
         # run_admin_command --create_vlan test30 -1 false
         if self.sub_id != 0:
             for friendly_name, _ in self.names.items():
+                self.logger.info("Waiting for vlan %s : %s to be deployed.", friendly_name, self.names[friendly_name])
                 self.vlans[friendly_name], tag = self._wait_on_create(friendly_name)
                 self.logger.debug("vlan name: %s vlan tag: %s", self.vlans[friendly_name], tag)
         else:
             for friendly_name, _ in self.names.items():
                 self.logger.info("Deploying vlan %s : %s", friendly_name, self.names[friendly_name])
-                try:
-                    self.vlans[friendly_name], tag = self._create_vlan(friendly_name)
-                except RuntimeError as exc:
-                    raise JobError("Deploy vlans failed for %s: %s" % (friendly_name, exc))
+                self.vlans[friendly_name], tag = self._create_vlan(friendly_name)
                 self.logger.debug("vlan name: %s vlan tag: %s", self.vlans[friendly_name], tag)
                 if not tag:  # error state from create_vlan
                     raise JobError("Unable to create vlan %s", friendly_name)
@@ -449,9 +451,10 @@ class VlandProtocol(Protocol):
             self._set_port_onto_vlan(self.vlans[friendly_name], port_id)
             self.ports.append(port_id)
 
-    def __call__(self, args):
+    def __call__(self, *args, **kwargs):
+        # only the first argument is used.
         try:
-            return self._api_select(args)
+            return self._api_select(args[0])
         except (ValueError, TypeError) as exc:
             msg = "Invalid call to %s %s" % (self.name, exc)
             self.logger.exception(msg)
