@@ -20,21 +20,46 @@
 
 import os
 import sys
-import glob
 import time
 import unittest
-import simplejson
+import logging
 import yaml
 
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
-from lava_dispatcher.pipeline.action import Pipeline, Action, JobError
+from lava_dispatcher.pipeline.action import Pipeline, Action, JobError, LAVABug, LAVAError
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.job import Job
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.actions.deploy.image import DeployImages
+from lava_dispatcher.pipeline.test.utils import DummyLogger
+
+# pylint: disable=superfluous-parens,too-few-public-methods
 
 
-class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class StdoutTestCase(unittest.TestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(StdoutTestCase, self).setUp()
+        logger = logging.getLogger('dispatcher')
+        logger.disabled = True
+        logger.propagate = False
+        # set to True to update pipeline_references automatically.
+        self.update_ref = False
+        self.job = None
+
+    def pipeline_reference(self, filename, job=None):
+        y_file = os.path.join(os.path.dirname(__file__), 'pipeline_refs', filename)
+        if self.update_ref:
+            if not job:
+                job = self.job
+            sys.stderr.write('WARNING: modifying pipeline references!')
+            with open(y_file, 'w') as describe:
+                yaml.dump(job.pipeline.describe(False), describe)
+        with open(y_file, 'r') as f_ref:
+            return yaml.load(f_ref)
+
+
+class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_references_a_device(self):
         device = object()
@@ -43,32 +68,36 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertIs(cmd.device, device)
 
 
-class TestPipelineInit(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestPipelineInit(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
-    class FakeAction(Action):  # pylint: disable=abstract-class-not-used
+    class FakeAction(Action):
 
         def __init__(self):
             self.ran = False
             super(TestPipelineInit.FakeAction, self).__init__()
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             self.ran = True
 
         def post_process(self):
             raise NotImplementedError("invalid")
 
     def setUp(self):
+        super(TestPipelineInit, self).setUp()
         self.sub0 = TestPipelineInit.FakeAction()
         self.sub1 = TestPipelineInit.FakeAction()
 
     def test_pipeline_init(self):
         self.assertIsNotNone(self.sub0)
         self.assertIsNotNone(self.sub1)
+        # prevent reviews leaving update_ref set to True.
+        self.assertFalse(self.update_ref)
 
 
-class TestJobParser(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestJobParser(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
+        super(TestJobParser, self).setUp()
         factory = Factory()
         self.job = factory.create_kvm_job('sample_jobs/basics.yaml', mkdtemp())
 
@@ -81,16 +110,10 @@ class TestJobParser(unittest.TestCase):  # pylint: disable=too-many-public-metho
     def test_pipeline_gets_multiple_actions_in_it(self):  # pylint: disable=invalid-name
         if not self.job:
             return unittest.skip("not all deployments have been implemented")
-        self.assertTrue(self.job.actions > 1)
+        self.assertTrue(self.job.actions > 1)  # pylint: disable=no-member
 
 
-def pipeline_reference(filename):
-    with open(os.path.join(os.path.dirname(__file__),
-              'pipeline_refs', filename), 'r') as f_ref:
-        return yaml.load(f_ref)
-
-
-class TestValidation(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestValidation(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_action_is_valid_if_there_are_not_errors(self):  # pylint: disable=invalid-name
         action = Action()
@@ -120,15 +143,25 @@ class Factory(object):
     Factory objects are dispatcher based classes, independent
     of any database objects.
     """
+
+    def __init__(self):
+        logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
+        logger = logging.getLogger('unittests')
+        logger.disabled = True
+        logger.propagate = False
+        logger = logging.getLogger('dispatcher')
+        logger.disabled = True
+        logger.propagate = False
+
     def create_fake_qemu_job(self, output_dir=None):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/basics.yaml')
         parser = JobParser()
         try:
             with open(sample_job_file) as sample_job_data:
-                job = parser.parse(sample_job_data, device, 4212, None, None, None,
+                job = parser.parse(sample_job_data, device, 4212, None, "",
                                    output_dir=output_dir)
-        except NotImplementedError:
+        except LAVAError:
             # some deployments listed in basics.yaml are not implemented yet
             return None
         return job
@@ -139,16 +172,17 @@ class Factory(object):
         parser = JobParser()
         try:
             with open(kvm_yaml) as sample_job_data:
-                job = parser.parse(sample_job_data, device, 4212, None, None, None,
+                job = parser.parse(sample_job_data, device, 4212, None, "",
                                    output_dir=output_dir)
-        except NotImplementedError as exc:
+            job.logger = DummyLogger()
+        except LAVAError as exc:
             print(exc)
             # some deployments listed in basics.yaml are not implemented yet
             return None
         return job
 
 
-class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestPipeline(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     class FakeAction(Action):
 
@@ -157,7 +191,7 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
             super(TestPipeline.FakeAction, self).__init__()
             self.name = "fake-action"
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             time.sleep(1)
             self.ran = True
 
@@ -173,20 +207,27 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertEqual(action.description, "test action only")
         self.assertEqual(action.summary, "starter")
         # action needs to be added to a top level pipe first
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             Pipeline(action)
         pipe = Pipeline()
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             pipe.add_action(None)
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             pipe.add_action(pipe)
         pipe.add_action(action)
         self.assertEqual(pipe.actions, [action])
         self.assertEqual(action.level, "1")
         try:
-            simplejson.loads(pipe.describe())
-        except:  # pylint: disable=bare-except
-            self.assertFalse(0)
+            description = pipe.describe()
+        except Exception as exc:  # pylint: disable=broad-except
+            self.fail(exc)
+        self.assertIsNotNone(description)
+        self.assertIsInstance(description, list)
+        self.assertIn('description', description[0])
+        self.assertIn('level', description[0])
+        self.assertIn('summary', description[0])
+        self.assertIn('max_retries', description[0])
+        self.assertIn('timeout', description[0])
 
     def test_create_internal_pipeline(self):
         action = Action()
@@ -201,7 +242,7 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
         action.name = "child_action"
         action.summary = "child"
         action.description = "action implementing an internal pipe"
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             Pipeline(action)
         pipe.add_action(action)
         self.assertEqual(action.level, "2")
@@ -309,7 +350,7 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
         parser = JobParser()
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
         try:
-            job = parser.parse(yaml.dump(job_def), device, 4212, None, None, None,
+            job = parser.parse(yaml.dump(job_def), device, 4212, None, "",
                                output_dir=mkdtemp())
         except NotImplementedError:
             # some deployments listed in basics.yaml are not implemented yet
@@ -317,18 +358,26 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertIsNotNone(job)
         job_def['compatibility'] = job.compatibility + 1
         self.assertRaises(
-            JobError, parser.parse, yaml.dump(job_def), device, 4212, None, None, None,
+            JobError, parser.parse, yaml.dump(job_def), device, 4212, None, "",
             mkdtemp()
         )
         job_def['compatibility'] = 0
         try:
-            job = parser.parse(yaml.dump(job_def), device, 4212, None, None, None,
+            job = parser.parse(yaml.dump(job_def), device, 4212, None, "",
                                output_dir=mkdtemp())
         except NotImplementedError:
             # some deployments listed in basics.yaml are not implemented yet
             pass
         self.assertIsNotNone(job)
 
+    def test_pipeline_actions(self):
+        factory = Factory()
+        job = factory.create_kvm_job('sample_jobs/kvm.yaml', mkdtemp())
+        self.assertEqual(
+            ['deploy', 'boot', 'test', 'finalize'],
+            [action.section for action in job.pipeline.actions]
+        )
+
     def test_namespace_data(self):
         factory = Factory()
         job = factory.create_kvm_job('sample_jobs/kvm.yaml', mkdtemp())
@@ -346,14 +395,14 @@ class TestPipeline(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertNotEqual(test_action.get_namespace_data('common', 'unknown', 'simple'), 1)
 
 
-class TestFakeActions(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestFakeActions(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
-    class KeepConnection(Action):  # pylint: disable=abstract-class-not-used
+    class KeepConnection(Action):
         def __init__(self):
             super(TestFakeActions.KeepConnection, self).__init__()
             self.name = "keep-connection"
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             pass
 
         def post_process(self):
@@ -364,11 +413,12 @@ class TestFakeActions(unittest.TestCase):  # pylint: disable=too-many-public-met
             super(TestFakeActions.MakeNewConnection, self).__init__()
             self.name = "make-new-connection"
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             new_connection = object()
             return new_connection
 
     def setUp(self):
+        super(TestFakeActions, self).setUp()
         self.sub0 = TestPipeline.FakeAction()
         self.sub1 = TestPipeline.FakeAction()
 
@@ -383,28 +433,28 @@ class TestFakeActions(unittest.TestCase):  # pylint: disable=too-many-public-met
         pipe = Pipeline()
         pipe.add_action(self.sub0)
         pipe.add_action(self.sub1)
-        pipe.run_actions(None)
+        pipe.run_actions(None, None)
         self.assertTrue(self.sub0.ran)
         self.assertTrue(self.sub1.ran)
-        self.assertNotEqual(self.sub0.elapsed_time, 0)
-        self.assertNotEqual(self.sub1.elapsed_time, 0)
+        self.assertNotEqual(self.sub0.timeout.elapsed_time, 0)
+        self.assertNotEqual(self.sub1.timeout.elapsed_time, 0)
 
     def test_keep_connection(self):
 
         pipe = Pipeline()
         pipe.add_action(TestFakeActions.KeepConnection())
         conn = object()
-        self.assertIs(conn, pipe.run_actions(conn))
+        self.assertIs(conn, pipe.run_actions(conn, None))
 
     def test_change_connection(self):
 
         pipe = Pipeline()
         pipe.add_action(TestFakeActions.MakeNewConnection())
         conn = object()
-        self.assertIsNot(conn, pipe.run_actions(conn))
+        self.assertIsNot(conn, pipe.run_actions(conn, None))
 
 
-class TestStrategySelector(unittest.TestCase):
+class TestStrategySelector(StdoutTestCase):
     """
     Check the lambda operation
     """
