@@ -23,7 +23,6 @@ import os
 import yaml
 import logging
 import unittest
-import subprocess
 from lava_dispatcher.pipeline.action import JobError
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
 from lava_dispatcher.pipeline.device import NewDevice
@@ -31,24 +30,27 @@ from lava_dispatcher.pipeline.action import Timeout
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.actions.boot.ssh import SchrootAction
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.filesystem import check_ssh_identity_file
 from lava_dispatcher.pipeline.protocols.multinode import MultinodeProtocol
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class ConnectionFactory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
     of any database objects.
     """
+
     def create_ssh_job(self, filename, output_dir=None):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/ssh-host-01.yaml'))
         kvm_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(kvm_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 0, socket_addr=None,
-                               master_cert=None, slave_cert=None, output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 0, None, dispatcher_config="",
+                               output_dir=output_dir)
+            job.logger = DummyLogger()
         return job
 
     def create_bbb_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
@@ -56,16 +58,17 @@ class Factory(object):  # pylint: disable=too-few-public-methods
         kvm_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(kvm_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None,
+            job = parser.parse(sample_job_data, device, 4212, None, "",
                                output_dir=output_dir)
+            job.logger = DummyLogger()
         return job
 
 
-class TestConnection(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestConnection(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestConnection, self).setUp()
-        factory = Factory()
+        factory = ConnectionFactory()
         self.job = factory.create_ssh_job('sample_jobs/ssh-deploy.yaml', mkdtemp())
         self.guest_job = factory.create_bbb_job('sample_jobs/bbb-ssh-guest.yaml', mkdtemp())
         logging.getLogger('dispatcher').addHandler(logging.NullHandler())
@@ -76,7 +79,7 @@ class TestConnection(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.job.validate()
         self.assertEqual([], self.job.pipeline.errors)
         # Check Pipeline
-        description_ref = pipeline_reference('ssh-deploy.yaml')
+        description_ref = self.pipeline_reference('ssh-deploy.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
     @unittest.skipIf(infrastructure_error('schroot'), "schroot not installed")
@@ -187,9 +190,18 @@ class TestConnection(unittest.TestCase):  # pylint: disable=too-many-public-meth
                     if 'boot' in boot and 'schroot' in boot['boot']][0]
         self.assertEqual(boot_act['schroot'], schroot.parameters['schroot'])
 
-    def test_guest_ssh(self):
+    def test_primary_ssh(self):
+        factory = ConnectionFactory()
+        job = factory.create_ssh_job('sample_jobs/primary-ssh.yaml', mkdtemp())
+        job.validate()
+        overlay = [action for action in job.pipeline.actions if action.name == 'scp-overlay'][0]
+        self.assertIsNotNone(overlay.parameters['deployment_data'])
+        tar_flags = overlay.parameters['deployment_data']['tar_flags'] if 'tar_flags' in overlay.parameters['deployment_data'].keys() else ''
+        self.assertIsNotNone(tar_flags)
+
+    def test_guest_ssh(self):  # pylint: disable=too-many-locals,too-many-statements
         self.assertIsNotNone(self.guest_job)
-        description_ref = pipeline_reference('bbb-ssh-guest.yaml')
+        description_ref = self.pipeline_reference('bbb-ssh-guest.yaml', job=self.guest_job)
         self.assertEqual(description_ref, self.guest_job.pipeline.describe(False))
         self.guest_job.validate()
         multinode = [protocol for protocol in self.guest_job.protocols if protocol.name == MultinodeProtocol.name][0]
@@ -259,7 +271,7 @@ class TestConnection(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertIn('timeout', overlay[0].parameters['protocols'][MultinodeProtocol.name][0])
         msg_dict = overlay[0].parameters['protocols'][MultinodeProtocol.name][0]['message']
         for key, value in msg_dict.items():
-            if 'yaml_line' == key:
+            if key == 'yaml_line':
                 continue
             self.assertTrue(value.startswith('$'))
             self.assertFalse(key.startswith('$'))
@@ -267,11 +279,11 @@ class TestConnection(unittest.TestCase):  # pylint: disable=too-many-public-meth
         multinode = [item for item in overlay[0].internal_pipeline.actions if item.name == 'lava-multinode-overlay']
         self.assertEqual(len(multinode), 1)
         # Check Pipeline
-        description_ref = pipeline_reference('ssh-guest.yaml')
+        description_ref = self.pipeline_reference('ssh-guest.yaml', job=self.guest_job)
         self.assertEqual(description_ref, self.guest_job.pipeline.describe(False))
 
 
-class TestTimeouts(unittest.TestCase):
+class TestTimeouts(StdoutTestCase):
     """
     Test action and connection timeout parsing.
     """
@@ -279,30 +291,34 @@ class TestTimeouts(unittest.TestCase):
     def create_custom_job(self, data, output_dir='/tmp/'):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         parser = JobParser()
-        job = parser.parse(data, device, 4212, None, None, None,
+        job = parser.parse(data, device, 4212, None, "",
                            output_dir=output_dir)
         return job
 
     def test_action_timeout(self):
-        factory = Factory()
+        factory = ConnectionFactory()
         job = factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
         job.validate()
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         test_action = [action for action in job.pipeline.actions if action.name == 'lava-test-retry'][0]
         test_shell = [action for action in test_action.internal_pipeline.actions if action.name == 'lava-test-shell'][0]
         self.assertEqual(test_shell.connection_timeout.duration, 240)  # job specifies 4 minutes
-        self.assertEqual(test_shell.timeout.duration, 420)  # job specifies 7 minutes
+        self.assertEqual(test_shell.timeout.duration, 300)  # job (test action block) specifies 5 minutes
         self.assertEqual(deploy.timeout.duration, 120)  # job specifies 2 minutes
         self.assertNotEqual(deploy.connection_timeout.duration, Timeout.default_duration())
         self.assertNotEqual(deploy.connection_timeout.duration, test_shell.connection_timeout)
         self.assertEqual(test_action.timeout.duration, 300)
+        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
+        retry = [action for action in uboot.internal_pipeline.actions if action.name == 'uboot-retry'][0]
+        auto = [action for action in retry.internal_pipeline.actions if action.name == 'auto-login-action'][0]
+        self.assertEqual(auto.timeout.duration / 60, 9)  # 9 minutes in the job def
 
     def test_job_connection_timeout(self):
         """
         Test connection timeout specified in the submission YAML
         """
-        with open(os.path.join(
-                os.path.dirname(__file__), './sample_jobs/uboot-ramdisk.yaml'), 'r') as uboot_ramdisk:
+        y_file = os.path.join(os.path.dirname(__file__), './sample_jobs/uboot-ramdisk.yaml')
+        with open(y_file, 'r') as uboot_ramdisk:
             data = yaml.load(uboot_ramdisk)
         data['timeouts']['connection'] = {'seconds': 20}
         job = self.create_custom_job(yaml.dump(data))
@@ -314,27 +330,27 @@ class TestTimeouts(unittest.TestCase):
                         # lava-test-shell from the job, uboot-retry from the device
                         self.assertEqual(check_action.connection_timeout.duration, 20)
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        self.assertIsNotNone(deploy)
         test_action = [action for action in job.pipeline.actions if action.name == 'lava-test-retry'][0]
         test_shell = [action for action in test_action.internal_pipeline.actions if action.name == 'lava-test-shell'][0]
         self.assertEqual(test_shell.connection_timeout.duration, 20)
+        self.assertEqual(test_shell.timeout.duration, 300)
+        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
+        retry = [action for action in uboot.internal_pipeline.actions if action.name == 'uboot-retry'][0]
+        auto = [action for action in retry.internal_pipeline.actions if action.name == 'auto-login-action'][0]
+        self.assertEqual(auto.connection_timeout.duration / 60, 12)
 
     def test_action_connection_timeout(self):
         """
         Test connection timeout specified for a particular action
         """
-        with open(os.path.join(
-                os.path.dirname(__file__), './sample_jobs/uboot-ramdisk.yaml'), 'r') as uboot_ramdisk:
+        y_file = os.path.join(os.path.dirname(__file__), './sample_jobs/uboot-ramdisk.yaml')
+        with open(y_file, 'r') as uboot_ramdisk:
             data = yaml.load(uboot_ramdisk)
         connection_timeout = Timeout.parse(data['timeouts']['connection'])
         self.assertEqual(connection_timeout, 240)
-        data['timeouts']['connections'] = {'uboot-retry': {}}
-        data['timeouts']['connections']['uboot-retry'] = {'seconds': 20}
         job = self.create_custom_job(yaml.dump(data))
         boot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
         retry = [action for action in boot.internal_pipeline.actions if action.name == 'uboot-retry'][0]
-        self.assertEqual(retry.timeout.duration, Timeout.parse(job.device['timeouts']['actions'][retry.name]))
-        self.assertEqual(
-            Timeout.parse(data['timeouts']['connections'][retry.name]),
-            retry.connection_timeout.duration
-        )
-        self.assertEqual(90, retry.timeout.duration)
+        self.assertEqual(retry.timeout.duration, 90)  # Set by the job global action timeout
+        self.assertEqual(retry.connection_timeout.duration, 45)
