@@ -24,15 +24,19 @@ import sys
 import glob
 import stat
 import yaml
+import shutil
 import pexpect
+import tempfile
 import unittest
+import subprocess
+from nose.tools import nottest
 from lava_dispatcher.pipeline.power import FinalizeAction
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.action import InfrastructureError
 from lava_dispatcher.pipeline.actions.test.shell import TestShellRetry, PatternFixup
-from lava_dispatcher.pipeline.test.test_basic import Factory
-from lava_dispatcher.pipeline.test.test_uboot import Factory as BBBFactory
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.test_uboot import UBootFactory
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.deploy.image import DeployImagesAction
 from lava_dispatcher.pipeline.actions.deploy.testdef import (
@@ -42,7 +46,6 @@ from lava_dispatcher.pipeline.actions.deploy.testdef import (
     TestInstallAction,
     TestRunnerAction,
 )
-from lava_dispatcher.pipeline.actions.test import TestAction
 from lava_dispatcher.pipeline.actions.boot import BootAction
 from lava_dispatcher.pipeline.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.pipeline.actions.deploy.download import DownloaderAction
@@ -66,7 +69,7 @@ def check_missing_path(testcase, exception, path):
             testcase.fail(exception)
 
 
-class TestDefinitionHandlers(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestDefinitionHandlers(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestDefinitionHandlers, self).setUp()
@@ -86,7 +89,7 @@ class TestDefinitionHandlers(unittest.TestCase):  # pylint: disable=too-many-pub
         self.assertIsInstance(testdef, TestDefinitionAction)
         testdef.validate()
         self.assertEqual(testdef.run_levels,
-                         {'smoke-tests': 0, 'singlenode-advanced': 1})
+                         {'smoke-tests': 0, 'singlenode-advanced': 0})
         if not testdef.valid:
             # python3 compatible
             print(testdef.errors)  # pylint: disable=superfluous-parens
@@ -133,7 +136,7 @@ class TestDefinitionHandlers(unittest.TestCase):  # pylint: disable=too-many-pub
         data = [block['test'] for block in content['actions'] if 'test' in block][0]
         definitions = [block for block in data['definitions'] if 'path' in block][0]
         definitions['name'] = 'smoke tests'
-        job = parser.parse(yaml.dump(content), device, 4212, None, None, None,
+        job = parser.parse(yaml.dump(content), device, 4212, None, "",
                            output_dir='/tmp/')
         deploy = [action for action in job.pipeline.actions if action.name == 'deployimages'][0]
         overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
@@ -197,11 +200,11 @@ class TestDefinitionHandlers(unittest.TestCase):  # pylint: disable=too-many-pub
         if sys.version_info[0] == 2:
             self.assertItemsEqual(check_list, script_list)
         elif sys.version_info[0] == 3:
-            self.assertCountEqual(check_list, script_list)
+            self.assertCountEqual(check_list, script_list)  # pylint: disable=no-member
         self.assertEqual(overlay.xmod, stat.S_IRWXU | stat.S_IXGRP | stat.S_IRGRP | stat.S_IXOTH | stat.S_IROTH)
 
 
-class TestDefinitionSimple(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestDefinitionSimple(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestDefinitionSimple, self).setUp()
@@ -225,12 +228,12 @@ class TestDefinitionSimple(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertEqual(len(deploy.pipeline.actions), 1)  # deploy without test only needs DownloaderAction
 
 
-class TestDefinitionParams(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestDefinitionParams(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestDefinitionParams, self).setUp()
-        factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm-params.yaml')
+        self.factory = Factory()
+        self.job = self.factory.create_kvm_job('sample_jobs/kvm-params.yaml')
 
     def test_job_without_tests(self):
         boot = finalize = None
@@ -283,8 +286,32 @@ class TestDefinitionParams(unittest.TestCase):  # pylint: disable=too-many-publi
             }
         )
 
+    @unittest.skipIf(infrastructure_error('git'), 'git not installed')
+    def test_install_repos(self):
+        job = self.factory.create_kvm_job('sample_jobs/kvm-install.yaml')
+        allow_missing_path(self.job.pipeline.validate_actions, self, 'qemu-system-x86_64')
+        deploy = [action for action in job.pipeline.actions if action.name == 'deployimages'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
+        testdef = [action for action in overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
+        test_install = [
+            action for action in testdef.internal_pipeline.actions
+            if action.name == 'test-install-overlay'][0]
+        self.assertIsNotNone(test_install)
+        yaml_file = os.path.join(os.path.dirname(__file__), './testdefs/install.yaml')
+        self.assertTrue(os.path.exists(yaml_file))
+        with open(yaml_file, 'r') as test_file:
+            testdef = yaml.safe_load(test_file)
+        repos = testdef['install'].get('git-repos', [])
+        self.assertIsNotNone(repos)
+        self.assertIsInstance(repos, list)
+        for repo in repos:
+            self.assertIsNotNone(repo)
+        runner_path = tempfile.mkdtemp()
+        test_install.install_git_repos(testdef, runner_path)
+        shutil.rmtree(runner_path)
 
-class TestDefinitionRepeat(unittest.TestCase):  # pylint: disable=too-many-public-methods
+
+class TestDefinitionRepeat(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestDefinitionRepeat, self).setUp()
@@ -313,11 +340,11 @@ class TestDefinitionRepeat(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertEqual(len(finalize), 1)
 
 
-class TestSkipInstall(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestSkipInstall(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestSkipInstall, self).setUp()
-        factory = BBBFactory()
+        factory = UBootFactory()
         self.job = factory.create_bbb_job("sample_jobs/bbb-skip-install.yaml")
 
     def test_skip_install_params(self):
@@ -346,7 +373,21 @@ class TestSkipInstall(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertEqual(single_testdef.skip_options, ['deps'])
 
 
-class TestDefinitions(unittest.TestCase):
+@nottest
+def check_rpcinfo(server='127.0.0.1'):
+    """
+    Supports the unittest.SkipIf
+    needs to return True if skip, so
+    returns True on failure.
+    """
+    try:
+        subprocess.check_output(['/usr/sbin/rpcinfo', '-u', server, 'nfs'])
+    except (OSError, subprocess.CalledProcessError):
+        return True
+    return False
+
+
+class TestDefinitions(StdoutTestCase):
     """
     For compatibility until the V1 code is removed and we can start
     cleaning up Lava Test Shell.
@@ -359,7 +400,7 @@ class TestDefinitions(unittest.TestCase):
         super(TestDefinitions, self).setUp()
         self.testdef = os.path.join(os.path.dirname(__file__), 'testdefs', 'params.yaml')
         self.res_data = os.path.join(os.path.dirname(__file__), 'testdefs', 'result-data.txt')
-        factory = BBBFactory()
+        factory = UBootFactory()
         self.job = factory.create_bbb_job("sample_jobs/bbb-nfs-url.yaml")
 
     def test_pattern(self):
@@ -369,7 +410,7 @@ class TestDefinitions(unittest.TestCase):
         self.assertIn('parse', params.keys())
         line = 'test1a: pass'
         self.assertEqual(
-            '(?P<test_case_id>.*-*):\s+(?P<result>(pass|fail))',
+            r'(?P<test_case_id>.*-*):\s+(?P<result>(pass|fail))',
             params['parse']['pattern'])
         match = re.search(params['parse']['pattern'], line)
         self.assertIsNotNone(match)
@@ -386,12 +427,16 @@ class TestDefinitions(unittest.TestCase):
         pattern = PatternFixup(testdef=params, count=0)
         self.assertTrue(pattern.valid())
 
-    def test_definition_lists(self):
+    @unittest.skipIf(check_rpcinfo(), "rpcinfo returns non-zero for nfs")
+    def test_definition_lists(self):  # pylint: disable=too-many-locals
         self.job.validate()
         tftp_deploy = [action for action in self.job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
         overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'lava-overlay'][0]
-        definition = [action for action in overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
+        apply_o = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
+        self.assertIsNone(apply_o.parameters.get('nfs_url', None))
+        self.assertIsInstance(apply_o.parameters.get('persistent_nfs', None), dict)
+        self.assertIsInstance(apply_o.parameters['persistent_nfs'].get('address', None), str)
         definition = [action for action in overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
         git_repos = [action for action in definition.internal_pipeline.actions if action.name == 'git-repo-action']
         self.assertIn('common', self.job.context)
@@ -420,7 +465,7 @@ class TestDefinitions(unittest.TestCase):
         with open(self.testdef, 'r') as par:
             params = yaml.load(par)
         self.assertEqual(
-            '(?P<test_case_id>.*-*):\s+(?P<result>(pass|fail))',
+            r'(?P<test_case_id>.*-*):\s+(?P<result>(pass|fail))',
             params['parse']['pattern'])
         self.job.context.setdefault('test', {})
         for git_repo in git_repos:
