@@ -20,18 +20,20 @@
 
 import os
 import unittest
-from lava_dispatcher.pipeline.action import Action, JobError
+from lava_dispatcher.pipeline.action import Action, ConfigurationError, JobError
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.boot import BootAction
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.actions.boot.u_boot import UBootInterrupt, UBootAction
+from lava_dispatcher.pipeline.test.test_basic import StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 
 # Test the loading of test definitions within the deploy stage
 
 
-class TestDeviceParser(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestDeviceParser(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_new_device(self):
         kvm01 = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
@@ -60,7 +62,7 @@ class FakeAction(Action):
         self.summary = "fake action"
 
 
-class TestJobDeviceParameters(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestJobDeviceParameters(StdoutTestCase):  # pylint: disable=too-many-public-methods
     """
     Test parsing of device configuration into job parameters
     """
@@ -69,13 +71,12 @@ class TestJobDeviceParameters(unittest.TestCase):  # pylint: disable=too-many-pu
         job_parser = JobParser()
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         self.assertIn('power_state', device)
-        self.assertEqual(device.power_state, 'off')
-        self.assertTrue(hasattr(device, 'power_state'))
+        self.assertFalse(hasattr(device, 'power_state'))
         self.assertFalse(hasattr(device, 'hostname'))
         self.assertIn('hostname', device)
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
-            job = job_parser.parse(sample_job_data, device, 4212, None, None, None)
+            job = job_parser.parse(sample_job_data, device, 4212, None, "")
         uboot_action = None
         for action in job.pipeline.actions:
             if isinstance(action, DeployAction):
@@ -98,7 +99,7 @@ class TestJobDeviceParameters(unittest.TestCase):  # pylint: disable=too-many-pu
         self.assertTrue(uboot_action.valid)
         for action in uboot_action.internal_pipeline.actions:
             if isinstance(action, UBootInterrupt):
-                self.assertIn('power_on', action.job.device['commands'])
+                self.assertIn('power-on', action.job.device['commands'])
                 self.assertIn('hard_reset', action.job.device['commands'])
                 self.assertIn('connect', action.job.device['commands'])
                 self.assertEqual(action.job.device['commands']['connect'].split(' ')[0], 'telnet')
@@ -109,27 +110,27 @@ class TestJobDeviceParameters(unittest.TestCase):  # pylint: disable=too-many-pu
                 self.assertIn(action.parameters['commands'], action.parameters[action.parameters['method']])
                 self.assertIn('commands', action.parameters[action.parameters['method']][action.parameters['commands']])
                 self.assertIsNotNone(action.parameters['u-boot']['ramdisk'])
-                self.assertTrue(type(action.parameters['u-boot']['ramdisk']['commands']) == list)
+                self.assertIsInstance(action.parameters['u-boot']['ramdisk']['commands'], list)
                 self.assertTrue(len(action.parameters['u-boot']['ramdisk']['commands']) > 2)
 
     def test_device_power(self):
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
-        self.assertEqual(device.power_state, 'off')
         self.assertNotEqual(device.hard_reset_command, '')
         self.assertNotEqual(device.power_command, '')
         self.assertIn('on', device.power_command)
-        device.power_state = 'on'
-        self.assertEqual(device.power_state, 'on')
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
-        self.assertEqual(device.power_state, '')
         self.assertEqual(device.hard_reset_command, '')
         self.assertEqual(device.power_command, '')
-        with self.assertRaises(RuntimeError):
-            device.power_state = ''
-        self.assertEqual(device.power_command, '')
+
+    def test_device_constants(self):
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
+        self.assertIn('constants', device)
+        self.assertEqual(device.get_constant('boot-message'), "Booting Linux")
+        self.assertRaises(ConfigurationError,
+                          device.get_constant, ('non-existing-const'))
 
 
-class TestDeviceEnvironment(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestDeviceEnvironment(StdoutTestCase):  # pylint: disable=too-many-public-methods
     """
     Test parsing of device environment support
     """
@@ -141,8 +142,7 @@ class TestDeviceEnvironment(unittest.TestCase):  # pylint: disable=too-many-publ
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(
-                sample_job_data, device, 4212, None,
-                None, None,
+                sample_job_data, device, 4212, None, "",
                 output_dir='/tmp', env_dut=data)
         self.assertEqual(
             job.parameters['env_dut'],
@@ -150,7 +150,7 @@ class TestDeviceEnvironment(unittest.TestCase):  # pylint: disable=too-many-publ
         )
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
-    def test_device_environment_validity(self):
+    def test_device_environment_validity(self):  # pylint: disable=invalid-name
         """
         Use non-YAML syntax a bit like existing device config syntax.
         Ensure this syntax is picked up as invalid.
@@ -166,9 +166,9 @@ overrides:
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(
-                sample_job_data, device, 4212, None,
-                None, None,
+                sample_job_data, device, 4212, None, "",
                 output_dir='/tmp', env_dut=data)
+        job.logger = DummyLogger()
         self.assertEqual(
             job.parameters['env_dut'],
             data
@@ -189,9 +189,9 @@ overrides:
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(
-                sample_job_data, device, 4212, None,
-                None, None,
+                sample_job_data, device, 4212, None, "",
                 output_dir='/tmp', env_dut=data)
+        job.logger = DummyLogger()
         self.assertEqual(
             job.parameters['env_dut'],
             data
@@ -210,7 +210,7 @@ overrides:
         self.assertTrue(found)
 
 
-class TestCommand(unittest.TestCase):
+class TestCommand(StdoutTestCase):
 
     def test_silent(self):
         fake = FakeAction()
@@ -232,7 +232,7 @@ class TestCommand(unittest.TestCase):
         command = 'false'
         # sets return code non-zero with no output
         log = fake.run_command(command.split(' '))
-        self.assertIsNone(log)
+        self.assertFalse(log)
         self.assertNotEqual([], fake.errors)
 
     def test_allow_silent_error(self):
@@ -246,7 +246,7 @@ class TestCommand(unittest.TestCase):
         fake = FakeAction()
         command = './no-script'
         log = fake.run_command(command.split(' '))
-        self.assertIsNone(log)
+        self.assertFalse(log)
         self.assertNotEqual([], fake.errors)
 
     def test_allow_silent_invalid(self):
