@@ -20,28 +20,23 @@
 
 
 import os
-import yaml
 import unittest
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
-from lava_dispatcher.pipeline.actions.boot.grub import (
-    GrubMainAction,
-    BootloaderCommandOverlay
-)
+from lava_dispatcher.pipeline.test.utils import DummyLogger
+from lava_dispatcher.pipeline.actions.boot.grub import GrubMainAction
+from lava_dispatcher.pipeline.actions.boot import BootloaderCommandOverlay
 from lava_dispatcher.pipeline.actions.deploy.tftp import TftpAction
 from lava_dispatcher.pipeline.job import Job
-from lava_dispatcher.pipeline.action import Pipeline, InfrastructureError
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.action import JobError, Pipeline
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
 from lava_dispatcher.pipeline.utils.network import dispatcher_ip
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, tftpd_dir
 from lava_dispatcher.pipeline.utils.strings import substitute
-from lava_dispatcher.pipeline.utils.constants import (
-    BOOT_MESSAGE
-)
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class GrubFactory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
@@ -49,32 +44,55 @@ class Factory(object):  # pylint: disable=too-few-public-methods
     """
     def create_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/d02-01.yaml'))
-        yaml = os.path.join(os.path.dirname(__file__), filename)
-        with open(yaml) as sample_job_data:
+        y_file = os.path.join(os.path.dirname(__file__), filename)
+        with open(y_file) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None,
+            job = parser.parse(sample_job_data, device, 4212, None, "",
                                output_dir=output_dir)
+        job.logger = DummyLogger()
+        return job
+
+    def create_mustang_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/mustang-grub-efi.yaml'))
+        y_file = os.path.join(os.path.dirname(__file__), filename)
+        with open(y_file) as sample_job_data:
+            parser = JobParser()
+            job = parser.parse(sample_job_data, device, 4212, None, "",
+                               output_dir=output_dir)
+        job.logger = DummyLogger()
+        return job
+
+    def create_hikey_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/hi6220-hikey-01.yaml'))
+        y_file = os.path.join(os.path.dirname(__file__), filename)
+        with open(y_file) as sample_job_data:
+            parser = JobParser()
+            job = parser.parse(sample_job_data, device, 4212, None, "",
+                               output_dir=output_dir)
+        job.logger = DummyLogger()
         return job
 
 
-class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestGrubAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(TestGrubAction, self).setUp()
+        self.factory = GrubFactory()
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
     def test_simulated_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk.yaml')
         self.assertIsNotNone(job)
 
         # uboot and uboot-ramdisk have the same pipeline structure
-        description_ref = pipeline_reference('grub.yaml')
+        description_ref = self.pipeline_reference('grub.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
         self.assertIsNone(job.validate())
         self.assertEqual(job.device['device_type'], 'd02')
 
     def test_tftp_pipeline(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk.yaml')
         self.assertEqual(
             [action.name for action in job.pipeline.actions],
             ['tftp-deploy', 'grub-main-action', 'lava-test-retry', 'finalize']
@@ -84,7 +102,7 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertIsNotNone(tftp.internal_pipeline)
         self.assertEqual(
             [action.name for action in tftp.internal_pipeline.actions],
-            ['download_retry', 'download_retry', 'download_retry', 'prepare-tftp-overlay', 'deploy-device-env']
+            ['download-retry', 'download-retry', 'download-retry', 'prepare-tftp-overlay', 'lxc-add-device-action', 'deploy-device-env']
         )
         self.assertIn('ramdisk', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('kernel', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
@@ -92,8 +110,7 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertNotIn('=', tftpd_dir())
 
     def test_device_d02(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk.yaml')
         self.assertEqual(
             job.device['commands']['connect'],
             'telnet ratchet 7003'
@@ -105,13 +122,13 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
     def test_grub_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk.yaml')
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
         self.assertIn('grub', job.device['actions']['boot']['methods'])
         params = job.device['actions']['boot']['methods']['grub']['parameters']
-        boot_message = params.get('boot_message', BOOT_MESSAGE)
+        boot_message = params.get('boot_message',
+                                  job.device.get_constant('boot-message'))
         self.assertIsNotNone(boot_message)
         for action in job.pipeline.actions:
             action.validate()
@@ -147,16 +164,13 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
             }
         }
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/d02-01.yaml'))
-        job = Job(4212, None, None, None, parameters)
+        job = Job(4212, parameters, None)
         job.device = device
         pipeline = Pipeline(job=job, parameters=parameters['actions']['boot'])
         job.pipeline = pipeline
         overlay = BootloaderCommandOverlay()
         pipeline.add_action(overlay)
-        try:
-            ip_addr = dispatcher_ip()
-        except InfrastructureError as exc:
-            raise RuntimeError("Unable to get dispatcher IP address: %s" % exc)
+        ip_addr = dispatcher_ip(None)
         parsed = []
         kernel = parameters['actions']['deploy']['kernel']
         ramdisk = parameters['actions']['deploy']['ramdisk']
@@ -186,12 +200,11 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertNotIn('devicetree (tftp,{SERVER_IP})/{DTB}', parsed)
 
     def test_download_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-nfs.yaml')
+        job = self.factory.create_job('sample_jobs/grub-nfs.yaml')
         for action in job.pipeline.actions:
             action.validate()
             if not action.valid:
-                raise RuntimeError(action.errors)
+                raise JobError(action.errors)
             self.assertTrue(action.valid)
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
@@ -213,11 +226,10 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
         self.assertIsNotNone(test_dir)
         self.assertIn('/lava-', test_dir)
         self.assertIsNotNone(extract)
-        self.assertEqual(extract.timeout.duration, job.parameters['timeouts'][extract.name]['seconds'])
+        self.assertEqual(extract.timeout.duration, 600)
 
     def test_reset_actions(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk.yaml')
         grub_action = None
         for action in job.pipeline.actions:
             action.validate()
@@ -226,14 +238,37 @@ class TestGrubAction(unittest.TestCase):  # pylint: disable=too-many-public-meth
                 grub_action = action
         names = [r_action.name for r_action in grub_action.internal_pipeline.actions]
         self.assertIn('connect-device', names)
-        self.assertIn('reboot-device', names)
+        self.assertIn('reset-device', names)
         self.assertIn('bootloader-interrupt', names)
         self.assertIn('expect-shell-connection', names)
         self.assertIn('bootloader-commands', names)
 
     def test_grub_with_monitor(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/grub-ramdisk-monitor.yaml')
+        job = self.factory.create_job('sample_jobs/grub-ramdisk-monitor.yaml')
         job.validate()
-        description_ref = pipeline_reference('grub-ramdisk-monitor.yaml')
+        description_ref = self.pipeline_reference('grub-ramdisk-monitor.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
+
+    def test_grub_via_efi(self):
+        job = self.factory.create_mustang_job('sample_jobs/mustang-grub-efi-nfs.yaml')
+        self.assertIsNotNone(job)
+        job.validate()
+        description_ref = self.pipeline_reference('mustang-grub-efi-nfs.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+        grub = [action for action in job.pipeline.actions if action.name == 'grub-main-action'][0]
+        menu = [action for action in grub.internal_pipeline.actions if action.name == 'uefi-menu-interrupt'][0]
+        self.assertIn('item_class', menu.params)
+        grub_efi = [action for action in grub.internal_pipeline.actions if action.name == 'grub-efi-menu-selector'][0]
+        self.assertEqual('pxe-grub', grub_efi.commands)
+
+    def test_hikey_grub_efi(self):
+        job = self.factory.create_hikey_job('sample_jobs/hikey-grub-lxc.yaml')
+        self.assertIsNotNone(job)
+        job.validate()
+        description_ref = self.pipeline_reference('hikey-grub-efi.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+        grub = [action for action in job.pipeline.actions if action.name == 'grub-main-action'][0]
+        menu = [action for action in grub.internal_pipeline.actions if action.name == 'uefi-menu-interrupt'][0]
+        self.assertIn('item_class', menu.params)
+        grub_efi = [action for action in grub.internal_pipeline.actions if action.name == 'grub-efi-menu-selector'][0]
+        self.assertEqual('fastboot', grub_efi.commands)
