@@ -22,62 +22,60 @@
 import os
 import sys
 import yaml
-import tarfile
 import unittest
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
-from lava_dispatcher.pipeline.actions.boot.ipxe import (
-    BootloaderAction,
-    BootloaderCommandsAction,
+from lava_dispatcher.pipeline.actions.boot.ipxe import BootloaderAction
+from lava_dispatcher.pipeline.actions.boot import (
     BootloaderCommandOverlay
 )
-from lava_dispatcher.pipeline.actions.deploy.apply_overlay import CompressRamdisk
 from lava_dispatcher.pipeline.actions.deploy.tftp import TftpAction
 from lava_dispatcher.pipeline.job import Job
-from lava_dispatcher.pipeline.action import Pipeline, InfrastructureError, JobError
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.action import Pipeline, JobError
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.network import dispatcher_ip
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
-from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, tftpd_dir
+from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
 from lava_dispatcher.pipeline.utils.strings import substitute
-from lava_dispatcher.pipeline.utils.constants import (
-    SHUTDOWN_MESSAGE,
-    BOOT_MESSAGE,
-)
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class X86Factory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
     of any database objects.
     """
+
     def create_job(self, filename, output_dir='/tmp/'):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/x86-01.yaml'))
-        yaml = os.path.join(os.path.dirname(__file__), filename)
-        with open(yaml) as sample_job_data:
+        y_file = os.path.join(os.path.dirname(__file__), filename)
+        with open(y_file) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None,
+            job = parser.parse(sample_job_data, device, 4212, None, "",
                                output_dir=output_dir)
+        job.logger = DummyLogger()
         return job
 
 
-class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestBootloaderAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(TestBootloaderAction, self).setUp()
+        self.factory = X86Factory()
 
     def test_simulated_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
         self.assertIsNotNone(job)
 
-        description_ref = pipeline_reference('ipxe.yaml')
+        description_ref = self.pipeline_reference('ipxe.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
         self.assertIsNone(job.validate())
         self.assertEqual(job.device['device_type'], 'x86')
 
     def test_tftp_pipeline(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
         self.assertEqual(
             [action.name for action in job.pipeline.actions],
             ['tftp-deploy', 'bootloader-action', 'lava-test-retry', 'finalize']
@@ -87,14 +85,13 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertIsNotNone(tftp.internal_pipeline)
         self.assertEqual(
             [action.name for action in tftp.internal_pipeline.actions],
-            ['download_retry', 'download_retry', 'download_retry', 'prepare-tftp-overlay', 'deploy-device-env']
+            ['download-retry', 'download-retry', 'download-retry', 'prepare-tftp-overlay', 'lxc-add-device-action', 'deploy-device-env']
         )
         self.assertIn('ramdisk', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('kernel', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
 
     def test_device_x86(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
         self.assertEqual(
             job.device['commands']['connect'],
             'telnet bumblebee 8003'
@@ -105,13 +102,13 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertEqual(methods['ipxe']['parameters'].get('bootloader_prompt', None), 'iPXE>')
 
     def test_bootloader_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
         self.assertIn('ipxe', job.device['actions']['boot']['methods'])
         params = job.device['actions']['boot']['methods']['ipxe']['parameters']
-        boot_message = params.get('boot_message', BOOT_MESSAGE)
+        boot_message = params.get('boot_message',
+                                  job.device.get_constant('boot-message'))
         self.assertIsNotNone(boot_message)
         bootloader_action = [action for action in job.pipeline.actions if action.name == 'bootloader-action'][0]
         bootloader_retry = [action for action in bootloader_action.internal_pipeline.actions if action.name == 'bootloader-retry'][0]
@@ -124,7 +121,7 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
                 self.assertEqual('ipxe', action.parameters['method'])
                 self.assertEqual(
                     'reboot: Restarting system',
-                    action.parameters.get('parameters', {}).get('shutdown-message', SHUTDOWN_MESSAGE)
+                    action.parameters.get('parameters', {}).get('shutdown-message', job.device.get_constant('shutdown-message'))
                 )
             if isinstance(action, TftpAction):
                 self.assertIn('ramdisk', action.parameters)
@@ -154,17 +151,13 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
             }
         }
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/x86-01.yaml'))
-        job = Job(4212, None, None, None, parameters)
+        job = Job(4212, parameters, None)
         job.device = device
         pipeline = Pipeline(job=job, parameters=parameters['actions']['boot'])
         job.pipeline = pipeline
         overlay = BootloaderCommandOverlay()
         pipeline.add_action(overlay)
-        try:
-            ip_addr = dispatcher_ip()
-        except InfrastructureError as exc:
-            raise RuntimeError("Unable to get dispatcher IP address: %s" % exc)
-        parsed = []
+        ip_addr = dispatcher_ip(None)
         kernel = parameters['actions']['deploy']['kernel']
         ramdisk = parameters['actions']['deploy']['ramdisk']
 
@@ -187,8 +180,7 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertIn("boot", commands)
 
     def test_download_action(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe.yaml')
         for action in job.pipeline.actions:
             action.validate()
             self.assertTrue(action.valid)
@@ -212,11 +204,10 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertIsNotNone(test_dir)
         self.assertIn('/lava-', test_dir)
         self.assertIsNotNone(extract)
-        self.assertEqual(extract.timeout.duration, job.parameters['timeouts'][extract.name]['seconds'])
+        self.assertEqual(extract.timeout.duration, 120)
 
     def test_reset_actions(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe.yaml')
         bootloader_action = None
         bootloader_retry = None
         reset_action = None
@@ -232,28 +223,25 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
             if action.name == 'bootloader-retry':
                 bootloader_retry = action
         names = [r_action.name for r_action in bootloader_retry.internal_pipeline.actions]
-        self.assertIn('reboot-device', names)
+        self.assertIn('reset-device', names)
         self.assertIn('bootloader-interrupt', names)
         self.assertIn('expect-shell-connection', names)
         self.assertIn('bootloader-commands', names)
         for action in bootloader_retry.internal_pipeline.actions:
-            if action.name == 'reboot-device':
+            if action.name == 'reset-device':
                 reset_action = action
         names = [r_action.name for r_action in reset_action.internal_pipeline.actions]
-        self.assertIn('soft-reboot', names)
-        self.assertIn('pdu_reboot', names)
-        self.assertIn('power_on', names)
+        self.assertIn('pdu-reboot', names)
 
     @unittest.skipIf(infrastructure_error('telnet'), "telnet not installed")
-    def test_prompt_from_job(self):
+    def test_prompt_from_job(self):  # pylint: disable=too-many-locals
         """
         Support setting the prompt after login via the job
 
         Loads a known YAML, adds a prompt to the dict and re-parses the job.
         Checks that the prompt is available in the expect_shell_connection action.
         """
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-ramdisk.yaml')
         job.validate()
         bootloader = [action for action in job.pipeline.actions if action.name == 'bootloader-action'][0]
         retry = [action for action in bootloader.internal_pipeline.actions
@@ -268,9 +256,11 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         parser = JobParser()
         sample_job_data = yaml.load(sample_job_string)
         boot = [item['boot'] for item in sample_job_data['actions'] if 'boot' in item][0]
+        self.assertIsNotNone(boot)
         sample_job_string = yaml.dump(sample_job_data)
-        job = parser.parse(sample_job_string, device, 4212, None, None, None,
+        job = parser.parse(sample_job_string, device, 4212, None, "",
                            output_dir='/tmp')
+        job.logger = DummyLogger()
         job.validate()
         bootloader = [action for action in job.pipeline.actions if action.name == 'bootloader-action'][0]
         retry = [action for action in bootloader.internal_pipeline.actions
@@ -282,8 +272,7 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
             self.assertNotEqual(check, expect.parameters)
 
     def test_xz_nfs(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-nfs.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-nfs.yaml')
         # this job won't validate as the .xz nfsrootfs URL is a fiction
         self.assertRaises(JobError, job.validate)
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -293,8 +282,7 @@ class TestBootloaderAction(unittest.TestCase):  # pylint: disable=too-many-publi
         self.assertEqual(nfs.parameters['nfsrootfs']['compression'], 'xz')
 
     def test_ipxe_with_monitor(self):
-        factory = Factory()
-        job = factory.create_job('sample_jobs/ipxe-monitor.yaml')
+        job = self.factory.create_job('sample_jobs/ipxe-monitor.yaml')
         job.validate()
-        description_ref = pipeline_reference('ipxe-monitor.yaml')
+        description_ref = self.pipeline_reference('ipxe-monitor.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
