@@ -20,17 +20,17 @@
 
 import os
 import yaml
-import unittest
 from lava_dispatcher.pipeline.action import Pipeline, Timeout
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.job import Job
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.strings import substitute
 
 
-class InstallerFactory(object):  # pylint: disable=too-few-public-methods
+class InstallerFactory(Factory):  # pylint: disable=too-few-public-methods
 
     def create_qemu_installer_job(self, output_dir=None):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm01.yaml'))
@@ -38,16 +38,18 @@ class InstallerFactory(object):  # pylint: disable=too-few-public-methods
         parser = JobParser()
         try:
             with open(sample_job_file) as sample_job_data:
-                job = parser.parse(sample_job_data, device, 4212, None, None, None, output_dir=output_dir)
+                job = parser.parse(sample_job_data, device, 4212, None, "", output_dir=output_dir)
+            job.logger = DummyLogger()
         except NotImplementedError:
             # some deployments listed in basics.yaml are not implemented yet
             return None
         return job
 
 
-class TestIsoJob(unittest.TestCase):
+class TestIsoJob(StdoutTestCase):
 
     def setUp(self):
+        super(TestIsoJob, self).setUp()
         factory = InstallerFactory()
         self.job = factory.create_qemu_installer_job(mkdtemp())
         self.assertIsNotNone(self.job)
@@ -55,7 +57,7 @@ class TestIsoJob(unittest.TestCase):
         self.assertIsInstance(self.job.pipeline, Pipeline)
 
     def test_job_reference(self):
-        description_ref = pipeline_reference('qemu-debian-installer.yaml')
+        description_ref = self.pipeline_reference('qemu-debian-installer.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
     def test_iso_preparation(self):
