@@ -21,6 +21,7 @@
 
 import os
 import glob
+import time
 import unittest
 import yaml
 import pexpect
@@ -32,7 +33,7 @@ from lava_dispatcher.pipeline.action import (
     JobError,
     InfrastructureError,
 )
-from lava_dispatcher.pipeline.test.test_basic import Factory, pipeline_reference
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
 from lava_dispatcher.pipeline.job import Job
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.boot.qemu import BootAction
@@ -41,12 +42,14 @@ from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.test.test_messages import FakeConnection
 from lava_dispatcher.pipeline.utils.messages import LinuxKernelMessages
 from lava_dispatcher.pipeline.test.test_defs import allow_missing_path, check_missing_path
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
+from lava_dispatcher.pipeline.utils.strings import substitute
 
 # pylint: disable=invalid-name
 
 
-class TestBasicJob(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestBasicJob(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_basic_actions(self):
         factory = Factory()
@@ -57,7 +60,7 @@ class TestBasicJob(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertIsInstance(job.pipeline, Pipeline)
 
 
-class TestKVMSimulation(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKVMSimulation(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_kvm_simulation(self):  # pylint: disable=too-many-statements
         """
@@ -151,7 +154,7 @@ class TestKVMSimulation(unittest.TestCase):  # pylint: disable=too-many-public-m
         self.assertEqual(len(pipe.describe()), 4)
 
 
-class TestKVMBasicDeploy(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKVMBasicDeploy, self).setUp()
@@ -165,7 +168,7 @@ class TestKVMBasicDeploy(unittest.TestCase):  # pylint: disable=too-many-public-
                 self.assertEqual(action.job, self.job)
 
     def test_pipeline(self):
-        description_ref = pipeline_reference('kvm.yaml')
+        description_ref = self.pipeline_reference('kvm.yaml')
         deploy = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
         overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
         self.assertIn('persistent-nfs-overlay', [action.name for action in overlay.internal_pipeline.actions])
@@ -219,7 +222,7 @@ class TestKVMBasicDeploy(unittest.TestCase):  # pylint: disable=too-many-public-
                 self.assertEqual(len(action.parameters['definitions']), 2)
 
 
-class TestKVMQcow2Deploy(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKVMQcow2Deploy(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKVMQcow2Deploy, self).setUp()
@@ -233,7 +236,7 @@ class TestKVMQcow2Deploy(unittest.TestCase):  # pylint: disable=too-many-public-
                 self.assertEqual(action.job, self.job)
 
     def test_pipeline(self):
-        description_ref = pipeline_reference('kvm-qcow2.yaml')
+        description_ref = self.pipeline_reference('kvm-qcow2.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
     def test_validate(self):
@@ -245,7 +248,7 @@ class TestKVMQcow2Deploy(unittest.TestCase):  # pylint: disable=too-many-public-
             self.assertEqual([], action.errors)
 
 
-class TestKVMDownloadLocalDeploy(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKVMDownloadLocalDeploy(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKVMDownloadLocalDeploy, self).setUp()
@@ -259,7 +262,7 @@ class TestKVMDownloadLocalDeploy(unittest.TestCase):  # pylint: disable=too-many
                 self.assertEqual(action.job, self.job)
 
     def test_pipeline(self):
-        description_ref = pipeline_reference('kvm-local.yaml')
+        description_ref = self.pipeline_reference('kvm-local.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
 
@@ -272,7 +275,7 @@ def prepare_test_connection():
     return FakeConnection(child, message_list)
 
 
-class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKVMInlineTestDeploy, self).setUp()
@@ -310,20 +313,21 @@ class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-pu
                   """)
         self.assertIsInstance(device['actions']['boot']['methods']['qemu']['parameters']['extra'][1], int)
         parser = JobParser()
-        job = parser.parse(yaml.dump(job_data), device, 4212, None, None, None,
+        job = parser.parse(yaml.dump(job_data), device, 4212, None, "",
                            output_dir='/tmp/')
+        job.logger = DummyLogger()
         job.validate()
-        boot_image = [action for action in job.pipeline.actions if action.name == 'boot_image_retry'][0]
-        boot_qemu = [action for action in boot_image.internal_pipeline.actions if action.name == 'boot_qemu_image'][0]
+        boot_image = [action for action in job.pipeline.actions if action.name == 'boot-image-retry'][0]
+        boot_qemu = [action for action in boot_image.internal_pipeline.actions if action.name == 'boot-qemu-image'][0]
         qemu = [action for action in boot_qemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
         self.assertIsInstance(qemu.sub_command, list)
-        [self.assertIsInstance(item, str) for item in qemu.sub_command]
+        [self.assertIsInstance(item, str) for item in qemu.sub_command]  # pylint: disable=expression-not-assigned
         self.assertIn('virtio-blk-device.scsi=off', qemu.sub_command)
         self.assertIn('1', qemu.sub_command)
         self.assertNotIn(1, qemu.sub_command)
 
     def test_pipeline(self):
-        description_ref = pipeline_reference('kvm-inline.yaml')
+        description_ref = self.pipeline_reference('kvm-inline.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -346,7 +350,7 @@ class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-pu
         inline_repo.set_namespace_data(action='test', label='shared', key='location', value=location)
         inline_repo.set_namespace_data(action='test', label='test-definiton', key='overlay_dir', value=location)
 
-        inline_repo.run(None)
+        inline_repo.run(None, None)
         yaml_file = os.path.join(location, '0/tests/0_smoke-tests-inline/inline/smoke-tests-basic.yaml')
         self.assertTrue(os.path.exists(yaml_file))
         with open(yaml_file, 'r') as f_in:
@@ -371,18 +375,20 @@ class TestKVMInlineTestDeploy(unittest.TestCase):  # pylint: disable=too-many-pu
         self.assertEqual(set(testdef), set(expected_testdef))
 
 
-class TestAutoLogin(unittest.TestCase):
+class TestAutoLogin(StdoutTestCase):
 
     def setUp(self):
         super(TestAutoLogin, self).setUp()
         factory = Factory()
         self.job = factory.create_kvm_job('sample_jobs/kvm-inline.yaml', mkdtemp())
+        self.job.logger = DummyLogger()
+        self.max_end_time = time.time() + 30
 
     def test_autologin_prompt_patterns(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
         self.job.validate()
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'auto_login': {'login_prompt': 'login:',
@@ -396,7 +402,7 @@ class TestAutoLogin(unittest.TestCase):
             key='line_separator', value='testsep')
 
         # Test the AutoLoginAction directly
-        conn = autologinaction.run(shell_connection)
+        conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
         self.assertEqual(shell_connection.raw_connection.linesep, 'testsep')
 
         self.assertIn('lava-test: # ', conn.prompt_str)
@@ -405,7 +411,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_autologin_void_login_prompt(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'auto_login': {'login_prompt': '',
@@ -419,7 +425,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_void_prompts_list(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': []})
@@ -431,7 +437,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_void_prompts_list_item(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['']})
@@ -443,7 +449,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_void_prompts_list_item2(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#', '']})
@@ -455,7 +461,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_prompts_list(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#']})
@@ -464,7 +470,7 @@ class TestAutoLogin(unittest.TestCase):
         shell_connection = prepare_test_connection()
 
         # Test the AutoLoginAction directly
-        conn = autologinaction.run(shell_connection)
+        conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
 
         self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
@@ -472,7 +478,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_void_prompts_str(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ''})
@@ -484,7 +490,7 @@ class TestAutoLogin(unittest.TestCase):
     def test_missing_autologin_prompts_str(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
         autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
 
         autologinaction.parameters.update({'prompts': ['root@debian:~#']})
@@ -493,13 +499,36 @@ class TestAutoLogin(unittest.TestCase):
         shell_connection = prepare_test_connection()
 
         # Test the AutoLoginAction directly
-        conn = autologinaction.run(shell_connection)
+        conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
 
         self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
 
+    def test_autologin_login_incorrect(self):
+        self.assertEqual(len(self.job.pipeline.describe()), 4)
 
-class TestChecksum(unittest.TestCase):
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
+        autologinaction = [action for action in bootaction.internal_pipeline.actions if action.name == 'auto-login-action'][0]
+
+        autologinaction.parameters.update({'prompts': ['root@debian:~#']})
+        autologinaction.parameters.update({'auto_login': {
+            'login_prompt': 'debian login:',
+            'username': 'root'
+        }})
+
+        # initialise the first Connection object, a command line shell
+        shell_connection = prepare_test_connection()
+
+        # Test the AutoLoginAction directly
+        conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
+
+        self.assertIn('lava-test: # ', conn.prompt_str)
+        self.assertIn('root@debian:~#', conn.prompt_str)
+        self.assertIn('Login incorrect', conn.prompt_str)
+        self.assertIn('Login timed out', conn.prompt_str)
+
+
+class TestChecksum(StdoutTestCase):
 
     def setUp(self):
         super(TestChecksum, self).setUp()
@@ -510,8 +539,8 @@ class TestChecksum(unittest.TestCase):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
         deployimagesaction = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download_retry'][0]
-        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http_download'][0]
+        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download-retry'][0]
+        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http-download'][0]
 
         # Just a small image
         httpdownloadaction.url = 'http://images.validation.linaro.org/unit-tests/rootfs.gz'
@@ -520,14 +549,14 @@ class TestChecksum(unittest.TestCase):
             'md5sum': '6ea432ac3c23210c816551782346ed1c',
             'sha256sum': '1a76b17701b9fdf6346b88eb49b0143a9c6912701b742a6e5826d6856edccd21'}}})
         httpdownloadaction.validate()
-        httpdownloadaction.run(None)
+        httpdownloadaction.run(None, None)
 
     def test_download_checksum_match_fail(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
         deployimagesaction = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download_retry'][0]
-        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http_download'][0]
+        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download-retry'][0]
+        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http-download'][0]
 
         # Just a small image
         httpdownloadaction.url = 'http://images.validation.linaro.org/unit-tests/rootfs.gz'
@@ -537,28 +566,28 @@ class TestChecksum(unittest.TestCase):
             'sha256sum': '92d6ff900d0c3656ab3f214ce6efd708f898fc5e259111111111111111111111'}}})
         httpdownloadaction.validate()
 
-        self.assertRaises(JobError, httpdownloadaction.run, None)
+        self.assertRaises(JobError, httpdownloadaction.run, None, None)
 
     def test_download_no_images_no_checksum(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
         deployimagesaction = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download_retry'][0]
-        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http_download'][0]
+        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download-retry'][0]
+        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http-download'][0]
 
         # Just a small image
         httpdownloadaction.url = 'http://images.validation.linaro.org/unit-tests/rootfs.gz'
         del httpdownloadaction.parameters['images']
         httpdownloadaction.parameters.update({'rootfs': {'url': httpdownloadaction.url}})
         httpdownloadaction.validate()
-        httpdownloadaction.run(None)
+        httpdownloadaction.run(None, None)
 
     def test_download_no_images_match_success(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
         deployimagesaction = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download_retry'][0]
-        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http_download'][0]
+        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download-retry'][0]
+        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http-download'][0]
 
         # Just a small image
         httpdownloadaction.url = 'http://images.validation.linaro.org/unit-tests/rootfs.gz'
@@ -568,14 +597,14 @@ class TestChecksum(unittest.TestCase):
                        'md5sum': '6ea432ac3c23210c816551782346ed1c',
                        'sha256sum': '1a76b17701b9fdf6346b88eb49b0143a9c6912701b742a6e5826d6856edccd21'}})
         httpdownloadaction.validate()
-        httpdownloadaction.run(None)
+        httpdownloadaction.run(None, None)
 
     def test_download_no_images_match_fail(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
 
         deployimagesaction = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download_retry'][0]
-        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http_download'][0]
+        downloadretryaction = [action for action in deployimagesaction.internal_pipeline.actions if action.name == 'download-retry'][0]
+        httpdownloadaction = [action for action in downloadretryaction.internal_pipeline.actions if action.name == 'http-download'][0]
 
         # Just a small image
         httpdownloadaction.url = 'http://images.validation.linaro.org/unit-tests/rootfs.gz'
@@ -585,7 +614,7 @@ class TestChecksum(unittest.TestCase):
                        'md5sum': '6ea432ac3c232122222221782346ed1c',
                        'sha256sum': '1a76b17701b9fdf63444444444444444446912701b742a6e5826d6856edccd21'}})
         httpdownloadaction.validate()
-        self.assertRaises(JobError, httpdownloadaction.run, None)
+        self.assertRaises(JobError, httpdownloadaction.run, None, None)
 
     def test_no_test_action_validate(self):
         self.assertEqual(len(self.job.pipeline.describe()), 4)
@@ -606,10 +635,10 @@ class TestChecksum(unittest.TestCase):
         bbb_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/bbb-ramdisk-nfs.yaml')
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None, output_dir='/tmp/')
+            job = parser.parse(sample_job_data, device, 4212, None, "", output_dir='/tmp/')
         deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
-        download = [action for action in deploy.internal_pipeline.actions if action.name == 'download_retry'][0]
-        helper = [action for action in download.internal_pipeline.actions if action.name == 'file_download'][0]
+        download = [action for action in deploy.internal_pipeline.actions if action.name == 'download-retry'][0]
+        helper = [action for action in download.internal_pipeline.actions if action.name == 'file-download'][0]
         remote = helper.parameters[helper.key]
         md5sum = remote.get('md5sum', None)
         self.assertIsNone(md5sum)
@@ -617,7 +646,7 @@ class TestChecksum(unittest.TestCase):
         self.assertIsNotNone(sha256sum)
 
 
-class TestKvmGuest(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKvmGuest(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKvmGuest, self).setUp()
@@ -629,7 +658,7 @@ class TestKvmGuest(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertEqual(512, self.job.device['actions']['deploy']['methods']['image']['parameters']['guest']['size'])
 
 
-class TestKvmUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKvmUefi(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestKvmUefi, self).setUp()
@@ -640,7 +669,7 @@ class TestKvmUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
                      'qemu-system-x86_64 not installed')
     def test_uefi_path(self):
         deploy = [action for action in self.job.pipeline.actions if action.name == 'deployimages'][0]
-        downloaders = [action for action in deploy.internal_pipeline.actions if action.name == 'download_retry']
+        downloaders = [action for action in deploy.internal_pipeline.actions if action.name == 'download-retry']
         self.assertEqual(len(downloaders), 2)
         uefi_download = downloaders[0]
         image_download = downloaders[1]
@@ -649,23 +678,87 @@ class TestKvmUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertIsNotNone(uefi_dir)
         self.assertTrue(os.path.exists(uefi_dir))  # no download has taken place, but the directory needs to exist
         self.assertFalse(uefi_dir.endswith('bios-256k.bin'))
-        boot = [action for action in self.job.pipeline.actions if action.name == 'boot_image_retry'][0]
-        qemu = [action for action in boot.internal_pipeline.actions if action.name == 'boot_qemu_image'][0]
+        boot = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
+        qemu = [action for action in boot.internal_pipeline.actions if action.name == 'boot-qemu-image'][0]
         execute = [action for action in qemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
         self.job.validate()
         self.assertIn('-L', execute.sub_command)
         self.assertIn(uefi_dir, execute.sub_command)
 
 
-# class TestMonitor(unittest.TestCase):  # pylint: disable=too-many-public-methods
-#
-#     def setUp(self):
-#         super(TestMonitor, self).setUp()
-#         factory = Factory()
-#         self.job = factory.create_kvm_job('sample_jobs/qemu-monitor.yaml', mkdtemp())
-#
-#     def test_qemu_monitor(self):
-#         self.assertIsNotNone(self.job)
-#         self.assertIsNotNone(self.job.pipeline)
-#         self.assertIsNotNone(self.job.pipeline.actions)
-#         self.job.validate()
+class TestQemuNFS(StdoutTestCase):
+
+    def setUp(self):
+        super(TestQemuNFS, self).setUp()
+        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/kvm03.yaml'))
+        kvm_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/qemu-nfs.yaml')
+        parser = JobParser()
+        try:
+            with open(kvm_yaml) as sample_job_data:
+                job = parser.parse(sample_job_data, device, 4212, None, "",
+                                   output_dir=mkdtemp())
+        except NotImplementedError as exc:
+            print(exc)
+            # some deployments listed in basics.yaml are not implemented yet
+            return None
+        self.job = job
+        self.job.logger = DummyLogger()
+
+    @unittest.skipIf(infrastructure_error('qemu-system-aarch64'),
+                     'qemu-system-arm not installed')
+    def test_qemu_nfs(self):
+        self.assertIsNotNone(self.job)
+        description_ref = self.pipeline_reference('qemu-nfs.yaml')
+        self.assertEqual(description_ref, self.job.pipeline.describe(False))
+
+        boot = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
+        qemu = [action for action in boot.internal_pipeline.actions if action.name == 'boot-qemu-image'][0]
+        execute = [action for action in qemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
+        self.job.validate()
+        self.assertNotEqual([], [line for line in execute.sub_command if line.startswith('-kernel')])
+        self.assertEqual(1, len([line for line in execute.sub_command if line.startswith('-kernel')]))
+        self.assertIn('vmlinuz', [line for line in execute.sub_command if line.startswith('-kernel')][0])
+
+        self.assertNotEqual([], [line for line in execute.sub_command if line.startswith('-initrd')])
+        self.assertEqual(1, len([line for line in execute.sub_command if line.startswith('-initrd')]))
+        self.assertIn('initrd.img', [line for line in execute.sub_command if line.startswith('-initrd')][0])
+
+        self.assertEqual([], [line for line in execute.sub_command if '/dev/nfs' in line])
+        self.assertEqual([], [line for line in execute.sub_command if 'nfsroot' in line])
+
+        args = execute.methods['qemu-nfs']['parameters']['append']['nfsrootargs']
+        self.assertIn('{NFS_SERVER_IP}', args)
+        self.assertIn('{NFSROOTFS}', args)
+
+        substitutions = execute.substitutions
+        substitutions["{NFSROOTFS}"] = 'root_dir'
+        params = execute.methods['qemu-nfs']['parameters']['append']
+        # console=ttyAMA0 root=/dev/nfs nfsroot=10.3.2.1:/var/lib/lava/dispatcher/tmp/dirname,tcp,hard,intr ip=dhcp
+        append = [
+            'console=%s' % params['console'],
+            'root=/dev/nfs',
+            '%s' % substitute([params['nfsrootargs']], substitutions)[0],
+            "%s" % params['ipargs']
+        ]
+        execute.sub_command.append('--append')
+        execute.sub_command.append('"%s"' % ' '.join(append))
+        kernel_cmdline = ' '.join(execute.sub_command)
+        self.assertIn('console=ttyAMA0', kernel_cmdline)
+        self.assertIn('/dev/nfs', kernel_cmdline)
+        self.assertIn('root_dir,tcp,hard,intr', kernel_cmdline)
+        self.assertIn('smp', kernel_cmdline)
+        self.assertIn('cortex-a57', kernel_cmdline)
+
+
+class TestMonitor(StdoutTestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(TestMonitor, self).setUp()
+        factory = Factory()
+        self.job = factory.create_kvm_job('sample_jobs/qemu-monitor.yaml', mkdtemp())
+
+    def test_qemu_monitor(self):
+        self.assertIsNotNone(self.job)
+        self.assertIsNotNone(self.job.pipeline)
+        self.assertIsNotNone(self.job.pipeline.actions)
+        self.job.validate()
