@@ -22,22 +22,23 @@
 import os
 import re
 import logging
-import unittest
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.action import Timeout, JobError
 from lava_dispatcher.pipeline.shell import ShellSession, ShellCommand
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.strings import substitute
 from lava_dispatcher.pipeline.menus.menus import SelectorMenu
 
 # pylint: disable=too-many-public-methods
 
 
-class TestSelectorMenu(unittest.TestCase):
+class TestSelectorMenu(StdoutTestCase):
 
     def setUp(self):
+        super(TestSelectorMenu, self).setUp()
         self.menu = SelectorMenu()
         self.menu.item_markup = (r'\[', r'\]')
         self.menu.item_class = '0-9'
@@ -74,7 +75,7 @@ class TestSelectorMenu(unittest.TestCase):
                     self.assertEqual(match.group(1), selection)
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class MenuFactory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
@@ -85,16 +86,17 @@ class Factory(object):  # pylint: disable=too-few-public-methods
         mustang_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(mustang_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 0, socket_addr=None,
-                               master_cert=None, slave_cert=None, output_dir=output_dir)
+            job = parser.parse(sample_job_data, device, 0, None, dispatcher_config="",
+                               output_dir=output_dir)
+            job.logger = DummyLogger()
         return job
 
 
-class TestUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestUefi(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
         super(TestUefi, self).setUp()
-        factory = Factory()
+        factory = MenuFactory()
         self.job = factory.create_uefi_job('sample_jobs/mustang-menu-ramdisk.yaml', mkdtemp())
 
     def test_check_char(self):
@@ -132,7 +134,7 @@ class TestUefi(unittest.TestCase):  # pylint: disable=too-many-public-methods
             "Start:"
         )
         self.assertIsInstance(selector.items, list)
-        description_ref = pipeline_reference('mustang-uefi.yaml')
+        description_ref = self.pipeline_reference('mustang-uefi.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
         # just dummy strings
         substitution_dictionary = {
