@@ -20,33 +20,35 @@
 
 import os
 import unittest
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
-from lava_dispatcher.pipeline.action import JobError, action_namespaces
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.action import JobError
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.actions.deploy import DeployAction
 from lava_dispatcher.pipeline.actions.deploy.removable import MassStorage
-from lava_dispatcher.pipeline.utils.strings import substitute
+from lava_dispatcher.pipeline.test.utils import DummyLogger
+from lava_dispatcher.pipeline.utils.strings import substitute, map_kernel_uboot
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class RemovableFactory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
     of any database objects.
     """
+
     def create_job(self, sample_job, device_file, output_dir='/tmp/'):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), device_file))
         yaml = os.path.join(os.path.dirname(__file__), sample_job)
         with open(yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None,
+            job = parser.parse(sample_job_data, device, 4212, None, "",
                                output_dir=output_dir)
         return job
 
 
-class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_device_parameters(self):
         """
@@ -78,13 +80,14 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         cubie = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/cubie1.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         with open(sample_job_file) as sample_job_data:
-            job = job_parser.parse(sample_job_data, cubie, 4212, None, None, None, output_dir='/tmp/')
+            job = job_parser.parse(sample_job_data, cubie, 4212, None, "", output_dir='/tmp/')
+        job.logger = DummyLogger()
         try:
             job.validate()
         except JobError:
             self.fail(job.pipeline.errors)
         sample_job_data.close()
-        description_ref = pipeline_reference('cubietruck-removable.yaml')
+        description_ref = self.pipeline_reference('cubietruck-removable.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
         mass_storage = None  # deploy
@@ -108,7 +111,8 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         cubie = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/cubie1.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         with open(sample_job_file) as sample_job_data:
-            job = job_parser.parse(sample_job_data, cubie, 4212, None, None, None, output_dir='/tmp/')
+            job = job_parser.parse(sample_job_data, cubie, 4212, None, "", output_dir='/tmp/')
+        job.logger = DummyLogger()
         job.validate()
         self.assertIn('usb', cubie['parameters']['media'].keys())
         deploy_params = [methods for methods in job.parameters['actions'] if 'deploy' in methods.keys()][1]['deploy']
@@ -132,12 +136,13 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.assertIsNotNone(dd_action.get_namespace_data(action=dd_action.name, label='u-boot', key='boot_part'))
         self.assertIsNotNone(dd_action.get_namespace_data(action='uboot-from-media', label='uuid', key='boot_part'))
         self.assertEqual('0', '%s' % dd_action.get_namespace_data(action=dd_action.name, label='u-boot', key='boot_part'))
-        self.assertTrue(type(dd_action.get_namespace_data(action='uboot-from-media', label='uuid', key='boot_part')) is str)
+        self.assertIsInstance(dd_action.get_namespace_data(action='uboot-from-media', label='uuid', key='boot_part'), str)
         self.assertEqual('0:1', dd_action.get_namespace_data(action='uboot-from-media', label='uuid', key='boot_part'))
 
     def test_juno_deployment(self):
-        factory = Factory()
+        factory = RemovableFactory()
         job = factory.create_job('sample_jobs/juno-uboot-removable.yaml', '../devices/juno-uboot.yaml')
+        job.logger = DummyLogger()
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
         self.assertIn('usb', job.device['parameters']['media'].keys())
@@ -159,7 +164,7 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.assertNotIn('deployment_data', second_deploy.parameters)
         storage_deploy_action = [action for action in job.pipeline.actions if action.name == 'storage-deploy'][0]
         download_action = [
-            action for action in storage_deploy_action.internal_pipeline.actions if action.name == 'download_retry'][0]
+            action for action in storage_deploy_action.internal_pipeline.actions if action.name == 'download-retry'][0]
         self.assertIsNotNone(download_action)
         self.assertEqual('android', storage_deploy_action.parameters['namespace'])
 
@@ -172,7 +177,8 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         bbb = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
-            job = job_parser.parse(sample_job_data, bbb, 4212, None, None, None, output_dir='/tmp/')
+            job = job_parser.parse(sample_job_data, bbb, 4212, None, "", output_dir='/tmp/')
+        job.logger = DummyLogger()
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
         self.assertIn('usb', bbb['parameters']['media'].keys())
@@ -189,7 +195,8 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         cubie = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/cubie1.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         with open(sample_job_file) as sample_job_data:
-            job = job_parser.parse(sample_job_data, cubie, 4212, None, None, None, output_dir='/tmp/')
+            job = job_parser.parse(sample_job_data, cubie, 4212, None, "", output_dir='/tmp/')
+        job.logger = DummyLogger()
         job.validate()
         boot_params = [
             methods for methods in job.parameters['actions'] if 'boot' in methods.keys()][1]['boot']
@@ -198,12 +205,12 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.assertIn('dtb', boot_params)
         self.assertIn('root_uuid', boot_params)
         self.assertIn('boot_part', boot_params)
-        self.assertIn('type', boot_params)
+        self.assertNotIn('type', boot_params)
         self.assertGreater(len(job.pipeline.actions), 1)
         self.assertIsNotNone(job.pipeline.actions[1].internal_pipeline)
-        u_boot_action = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1].internal_pipeline.actions[2]
-        self.assertIsNotNone(u_boot_action.get_namespace_data(action='storage-deploy', label='u-boot', key='device'))
-        self.assertEqual(u_boot_action.name, "uboot-overlay")
+        u_boot_action = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1]
+        overlay = [action for action in u_boot_action.internal_pipeline.actions if action.name == 'bootloader-overlay'][0]
+        self.assertIsNotNone(overlay.get_namespace_data(action='storage-deploy', label='u-boot', key='device'))
 
         methods = cubie['actions']['boot']['methods']
         self.assertIn('u-boot', methods)
@@ -212,12 +219,14 @@ class TestRemovable(unittest.TestCase):  # pylint: disable=too-many-public-metho
         commands_list = methods['u-boot']['usb']['commands']
         device_id = u_boot_action.get_namespace_data(action='storage-deploy', label='u-boot', key='device')
         self.assertIsNotNone(device_id)
+        kernel_type = u_boot_action.parameters['kernel_type']
+        bootcommand = map_kernel_uboot(kernel_type, device_params=cubie.get('parameters', None))
         substitutions = {
             '{BOOTX}': "%s %s %s %s" % (
-                u_boot_action.parameters['type'],
-                cubie['parameters'][u_boot_action.parameters['type']]['kernel'],
-                cubie['parameters'][u_boot_action.parameters['type']]['ramdisk'],
-                cubie['parameters'][u_boot_action.parameters['type']]['dtb'],),
+                bootcommand,
+                cubie['parameters'][bootcommand]['kernel'],
+                cubie['parameters'][bootcommand]['ramdisk'],
+                cubie['parameters'][bootcommand]['dtb'],),
             '{RAMDISK}': boot_params['ramdisk'],
             '{KERNEL}': boot_params['kernel'],
             '{DTB}': boot_params['dtb'],
