@@ -19,16 +19,15 @@
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
 
-import unittest
 from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMUImageAction
 from lava_dispatcher.pipeline.actions.test.shell import TestShellRetry
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
-from lava_dispatcher.pipeline.test.test_basic import Factory, pipeline_reference
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
 from lava_dispatcher.pipeline.actions.deploy.testdef import get_deployment_testdefs
 from lava_dispatcher.pipeline.test.test_defs import allow_missing_path
 
 
-class TestRepeatBootTest(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestRepeatBootTest(StdoutTestCase):  # pylint: disable=too-many-public-methods
     """
     Test repeat counts with nested test stanzas
     """
@@ -41,7 +40,7 @@ class TestRepeatBootTest(unittest.TestCase):  # pylint: disable=too-many-public-
         self.assertIsNotNone(self.job)
         allow_missing_path(self.job.validate, self, 'qemu-system-x86_64')
         self.assertEqual([], self.job.pipeline.errors)
-        description_ref = pipeline_reference('kvm-repeat.yaml')
+        description_ref = self.pipeline_reference('kvm-repeat.yaml')
         self.assertEqual(description_ref, self.job.pipeline.describe(False))
 
     def test_deploy_norepeat(self):
@@ -59,7 +58,7 @@ class TestRepeatBootTest(unittest.TestCase):  # pylint: disable=too-many-public-
         self.assertIn('test', repeat_block['actions'][1])
         self.assertIn('boot', repeat_block['actions'][2])
         self.assertIn('test', repeat_block['actions'][3])
-        self.assertEqual(len(params), 5)
+        self.assertEqual(len(params), 3)  # count the "namespace" parameter
 
     def test_nested_structure(self):
         self.assertIn(['repeat'], [list(actions.keys()) for actions in self.job.parameters['actions']])
