@@ -18,34 +18,33 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-
 import time
-import unittest
 from lava_dispatcher.pipeline.action import (
     Action,
     Pipeline,
     Timeout,
     JobError,
+    LAVABug,
 )
 from lava_dispatcher.pipeline.logical import (
-    AdjuvantAction,
     RetryAction,
     DiagnosticAction,
 )
 from lava_dispatcher.pipeline.power import FinalizeAction
 from lava_dispatcher.pipeline.job import Job
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp
+from lava_dispatcher.pipeline.test.test_basic import StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 
 
 # pylint: disable=too-few-public-methods
 
-
-class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     class FakeJob(Job):
 
         def __init__(self, parameters):
-            super(TestAction.FakeJob, self).__init__(4212, None, None, None, parameters)
+            super(TestAction.FakeJob, self).__init__(4212, parameters, None)
 
     class FakeDeploy(object):
         """
@@ -86,7 +85,7 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.summary = "fake action for unit tests"
             self.description = "fake, do not use outside unit tests"
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             self.count += 1
             raise JobError("fake error")
 
@@ -102,8 +101,9 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.section = 'internal'
             self.summary = "fake trigger action for unit tests"
             self.description = "fake, do not use outside unit tests"
+            self.parameters['namespace'] = 'common'
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             self.count += 1
             self.job.triggers.append(TestAction.DiagnoseCheck.trigger())
             raise JobError("fake error")
@@ -142,7 +142,7 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.internal_pipeline = Pipeline(parent=self, job=self.job)
             self.internal_pipeline.add_action(TestAction.FakeAction(), parameters)
 
-        def cleanup(self):
+        def cleanup(self, connection):
             pass
 
     class DiagnoseCheck(DiagnosticAction):
@@ -155,18 +155,22 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
             return 'fake-check'
 
     def setUp(self):
+        super(TestAction, self).setUp()
         self.parameters = {
             "job_name": "fakejob",
             'output_dir': mkdtemp(),
             "actions": [
                 {
                     'deploy': {
+                        'namespace': 'common',
                         'failure_retry': 3
                     },
                     'boot': {
+                        'namespace': 'common',
                         'failure_retry': 4
                     },
                     'test': {
+                        'namespace': 'common',
                         'failure_retry': 5
                     }
                 }
@@ -177,9 +181,12 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
     def lookup_deploy(self, params):  # pylint: disable=no-self-use
         actions = iter(params)
         while actions:
-            action = next(actions)
-            if 'deploy' in action:
-                yield action['deploy']
+            try:
+                action = next(actions)
+                if 'deploy' in action:
+                    yield action['deploy']
+            except StopIteration:
+                break
 
     def test_fakeaction_fails_joberror(self):
         fakepipeline = TestAction.FakePipeline(job=self.fakejob)
@@ -192,7 +199,7 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
     def test_fakeretry_action(self):
         fakepipeline = TestAction.FakePipeline(job=self.fakejob)
         fakepipeline.add_action(TestAction.FakeRetryAction())
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             # first fake retry has no internal pipeline
             self.assertTrue(fakepipeline.validate_actions())
 
@@ -228,7 +235,7 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
         fakepipeline = TestAction.FakePipeline(job=self.fakejob)
         fakepipeline.add_action(TestAction.FakeTriggerAction())
         self.assertIsNone(fakepipeline.validate_actions())
-        with self.assertRaises(RuntimeError):
+        with self.assertRaises(LAVABug):
             fakepipeline.run_actions(None, None)
 
     def test_diagnostic(self):
@@ -240,184 +247,6 @@ class TestAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
         with self.assertRaises(JobError):
             fakepipeline.run_actions(None, None)
 
-
-class TestAdjuvant(unittest.TestCase):  # pylint: disable=too-many-public-methods
-
-    class FakeJob(Job):
-
-        def __init__(self, parameters):
-            super(TestAdjuvant.FakeJob, self).__init__(4212, None, None, None, parameters)
-
-        def validate(self, simulate=False):
-            self.pipeline.validate_actions()
-
-    class FakeDeploy(object):
-        """
-        Derived from object, *not* Deployment as this confuses python -m unittest discover
-        - leads to the FakeDeploy being called instead.
-        """
-        def __init__(self, parent):
-            self.__parameters__ = {}
-            self.pipeline = parent
-            self.job = parent.job
-            self.action = TestAdjuvant.FakeAction()
-
-    class FakeConnection(object):
-        def __init__(self):
-            self.name = "fake-connect"
-
-    class FakeDevice(object):
-        def __init__(self):
-            self.parameters = {}
-
-    class FakePipeline(Pipeline):
-
-        def __init__(self, parent=None, job=None):
-            super(TestAdjuvant.FakePipeline, self).__init__(parent, job)
-
-    class FailingAdjuvant(AdjuvantAction):  # pylint: disable=abstract-method
-        """
-        Added to the pipeline but only runs if FakeAction sets a suitable key.
-        """
-        def __init__(self):
-            super(TestAdjuvant.FailingAdjuvant, self).__init__()
-            self.name = "fake-adjuvant"
-            self.summary = "fake helper"
-            self.description = "fake adjuvant helper"
-
-    class FakeAdjuvant(AdjuvantAction):
-        """
-        Added to the pipeline but only runs if FakeAction sets a suitable key.
-        """
-        def __init__(self):
-            super(TestAdjuvant.FakeAdjuvant, self).__init__()
-            self.name = "fake-adjuvant"
-            self.summary = "fake helper"
-            self.description = "fake adjuvant helper"
-
-        @classmethod
-        def key(cls):
-            return "fake-key"
-
-        def run(self, connection, args=None):
-            connection = super(TestAdjuvant.FakeAdjuvant, self).run(connection, args)
-            if not self.valid:
-                raise RuntimeError("fakeadjuvant should be valid")
-            if self.data[self.key()]:
-                self.data[self.key()] = 'triggered'
-            if self.adjuvant:
-                self.data[self.key()] = 'base class trigger'
-            return connection
-
-    class FakeAction(Action):
-        """
-        Isolated Action which can be used to generate artificial exceptions.
-        """
-
-        def __init__(self):
-            super(TestAdjuvant.FakeAction, self).__init__()
-            self.count = 1
-            self.name = "fake-action"
-            self.summary = "fake action for unit tests"
-            self.description = "fake, do not use outside unit tests"
-
-        def populate(self, parameters):
-            self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-            self.internal_pipeline.add_action(TestAdjuvant.FakeAdjuvant())
-
-        def run(self, connection, args=None):
-            if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
-            connection = TestAdjuvant.FakeConnection()
-            self.count += 1
-            self.results = {'status': "failed"}
-            self.data[TestAdjuvant.FakeAdjuvant.key()] = True
-            return connection
-
-    class SafeAction(Action):
-        """
-        Isolated test action which does not trigger the adjuvant
-        """
-        def __init__(self):
-            super(TestAdjuvant.SafeAction, self).__init__()
-            self.name = "passing-action"
-            self.summary = "fake action without adjuvant"
-            self.description = "fake action runs without calling adjuvant"
-
-        def populate(self, parameters):
-            self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-            self.internal_pipeline.add_action(TestAdjuvant.FakeAdjuvant())
-
-        def run(self, connection, args=None):
-            if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
-            connection = TestAdjuvant.FakeConnection()
-            self.results = {'status': "passed"}
-            self.data[TestAdjuvant.FakeAdjuvant.key()] = False
-            return connection
-
-    def setUp(self):
-        self.parameters = {
-            "job_name": "fakejob",
-            'output_dir': mkdtemp(),
-            "actions": [
-                {
-                    'deploy': {
-                        'failure_retry': 3
-                    },
-                    'boot': {
-                        'failure_retry': 4
-                    },
-                    'test': {
-                        'failure_retry': 5
-                    }
-                }
-            ]
-        }
-        self.fakejob = TestAdjuvant.FakeJob(self.parameters)
-
-    def test_adjuvant_key(self):
-        pipeline = TestAction.FakePipeline(job=self.fakejob)
-        pipeline.add_action(TestAdjuvant.FakeAction())
-        pipeline.add_action(TestAdjuvant.FailingAdjuvant())
-        self.fakejob.pipeline = pipeline
-        self.fakejob.device = TestAdjuvant.FakeDevice()
-        with self.assertRaises(JobError):
-            self.fakejob.validate()
-
-    def test_adjuvant(self):
-        pipeline = TestAction.FakePipeline(job=self.fakejob)
-        pipeline.add_action(TestAdjuvant.FakeAction())
-        pipeline.add_action(TestAdjuvant.FakeAdjuvant())
-        self.fakejob.pipeline = pipeline
-        self.fakejob.device = TestAdjuvant.FakeDevice()
-        actions = []
-        for action in self.fakejob.pipeline.actions:
-            actions.append(action.name)
-        self.assertIn('fake-action', actions)
-        self.assertIn('fake-adjuvant', actions)
-        self.assertEqual(self.fakejob.pipeline.actions[1].key(), TestAdjuvant.FakeAdjuvant.key())
-
-    def test_run_adjuvant_action(self):
-        pipeline = TestAction.FakePipeline(job=self.fakejob)
-        pipeline.add_action(TestAdjuvant.FakeAction())
-        pipeline.add_action(TestAdjuvant.FakeAdjuvant())
-        self.fakejob.pipeline = pipeline
-        self.fakejob.device = TestAdjuvant.FakeDevice()
-        self.fakejob.run()
-        self.assertEqual(self.fakejob.context, {'fake-key': 'base class trigger'})
-
-    def test_run_action(self):
-        pipeline = TestAction.FakePipeline(job=self.fakejob)
-        pipeline.add_action(TestAdjuvant.SafeAction())
-        pipeline.add_action(TestAdjuvant.FakeAdjuvant())
-        self.fakejob.pipeline = pipeline
-        self.fakejob.device = TestAdjuvant.FakeDevice()
-        self.fakejob.run()
-        self.assertNotEqual(self.fakejob.context, {'fake-key': 'triggered'})
-        self.assertNotEqual(self.fakejob.context, {'fake-key': 'base class trigger'})
-        self.assertEqual(self.fakejob.context, {'fake-key': False})
-
     def test_namespace_data(self):
         """
         namespace data uses copies, not references
@@ -425,8 +254,9 @@ class TestAdjuvant(unittest.TestCase):  # pylint: disable=too-many-public-method
         This allows actions to refer to the common data and manipulate it without affecting other actions.
         """
         pipeline = TestAction.FakePipeline(job=self.fakejob)
-        action = TestAdjuvant.SafeAction()
+        action = TestAction.FakeTriggerAction()
         pipeline.add_action(action)
+        self.assertEqual({'namespace': 'common'}, action.parameters)
         action.set_namespace_data(action='test', label="fake", key="string value", value="test string")
         self.assertEqual(action.get_namespace_data(action='test', label='fake', key='string value'), 'test string')
         test_string = action.get_namespace_data(action='test', label='fake', key='string value')
@@ -458,20 +288,23 @@ class TestAdjuvant(unittest.TestCase):  # pylint: disable=too-many-public-method
         self.assertNotEqual(reference_list, [1, 2, 3, 7, 8, 9])
 
 
-class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestTimeout(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     class FakeJob(Job):
 
         def __init__(self, parameters):
-            super(TestTimeout.FakeJob, self).__init__(4212, None, None, None, parameters)
+            super(TestTimeout.FakeJob, self).__init__(4212, parameters, None)
+            self.logger = DummyLogger()
 
         def validate(self, simulate=False):
             self.pipeline.validate_actions()
 
-    class FakeDevice(object):
+    class FakeDevice(dict):
         def __init__(self):
-            self.parameters = {}
-            self.power_state = ''
+            self.update({'parameters': {}, 'commands': {}})
+
+        def __get_item__(self):
+            return {}
 
     class FakePipeline(Pipeline):
 
@@ -491,11 +324,11 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
         def populate(self, parameters):
             self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-            self.internal_pipeline.add_action(TestAdjuvant.FakeAdjuvant())
+            self.internal_pipeline.add_action(TestAction.FakeAction())
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
+                raise LAVABug("Fake action not meant to have a real connection")
             time.sleep(3)
             self.results = {'status': "failed"}
             return connection
@@ -509,10 +342,11 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.name = "passing-action"
             self.summary = "fake action without adjuvant"
             self.description = "fake action runs without calling adjuvant"
+            self.parameters['namespace'] = 'common'
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
+                raise LAVABug("Fake action not meant to have a real connection")
             self.results = {'status': "passed"}
             return connection
 
@@ -525,10 +359,11 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.name = "long-action"
             self.summary = "fake action with overly long sleep"
             self.description = "fake action"
+            self.parameters['namespace'] = 'common'
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
+                raise LAVABug("Fake action not meant to have a real connection")
             time.sleep(5)
             self.results = {'status': "passed"}
             return connection
@@ -543,8 +378,8 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
             self.summary = "fake action without adjuvant"
             self.description = "fake action runs without calling adjuvant"
 
-        def run(self, connection, args=None):
-            raise RuntimeError("Fake action not meant to actually run - should have timed out")
+        def run(self, connection, max_end_time, args=None):
+            raise LAVABug("Fake action not meant to actually run - should have timed out")
 
     class FakeSafeAction(Action):
         """
@@ -560,16 +395,17 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
         def populate(self, parameters):
             self.internal_pipeline = Pipeline(parent=self, job=self.job, parameters=parameters)
-            self.internal_pipeline.add_action(TestAdjuvant.FakeAdjuvant())
+            self.internal_pipeline.add_action(TestAction.FakeAction())
 
-        def run(self, connection, args=None):
+        def run(self, connection, max_end_time, args=None):
             if connection:
-                raise RuntimeError("Fake action not meant to have a real connection")
+                raise LAVABug("Fake action not meant to have a real connection")
             time.sleep(3)
             self.results = {'status': "failed"}
             return connection
 
     def setUp(self):
+        super(TestTimeout, self).setUp()
         self.parameters = {
             "job_name": "fakejob",
             'output_dir': mkdtemp(),
@@ -581,12 +417,15 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
             "actions": [
                 {
                     'deploy': {
+                        'namespace': 'common',
                         'failure_retry': 3
                     },
                     'boot': {
+                        'namespace': 'common',
                         'failure_retry': 4
                     },
                     'test': {
+                        'namespace': 'common',
                         'failure_retry': 5
                     }
                 }
@@ -612,8 +451,8 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
         pipeline.add_action(action)
         self.fakejob.pipeline = pipeline
         self.fakejob.device = TestTimeout.FakeDevice()
-        with self.assertRaises(JobError):
-            self.fakejob.run()
+        # run() returns 2 for JobError
+        self.assertEqual(self.fakejob.run(), 2)
 
     def test_action_complete(self):
         self.assertIsNotNone(self.fakejob.timeout)
@@ -624,10 +463,7 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
         pipeline.add_action(action)
         self.fakejob.pipeline = pipeline
         self.fakejob.device = TestTimeout.FakeDevice()
-        try:
-            self.fakejob.run()
-        except JobError as exc:
-            self.fail(exc)
+        self.assertEqual(self.fakejob.run(), 0)
 
     def test_job_timeout(self):
         self.assertIsNotNone(self.fakejob.timeout)
@@ -635,11 +471,13 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
         action = TestTimeout.LongAction()
         pipeline.add_action(action)
         pipeline.add_action(TestTimeout.SafeAction())
-        pipeline.add_action(FinalizeAction())
+        finalize = FinalizeAction()
+        finalize.parameters['namespace'] = 'common'
+        pipeline.add_action(finalize)
         self.fakejob.pipeline = pipeline
         self.fakejob.device = TestTimeout.FakeDevice()
-        with self.assertRaises(JobError):
-            self.fakejob.run()
+        # run() returns 2 for JobError
+        self.assertEqual(self.fakejob.run(), 2)
 
     def test_job_safe(self):
         self.assertIsNotNone(self.fakejob.timeout)
@@ -647,13 +485,13 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
         action = TestTimeout.SafeAction()
         pipeline.add_action(action)
         pipeline.add_action(TestTimeout.SafeAction())
-        pipeline.add_action(FinalizeAction())
+        finalize = FinalizeAction()
+        finalize.parameters['namespace'] = 'common'
+        pipeline.add_action(finalize)
         self.fakejob.pipeline = pipeline
         self.fakejob.device = TestTimeout.FakeDevice()
-        try:
-            self.fakejob.run()
-        except JobError as exc:
-            self.fail(exc)
+        # run() returns 0 in case of success
+        self.assertEqual(self.fakejob.run(), 0)
 
     def test_long_job_safe(self):
         self.fakejob.timeout.duration = 8
@@ -666,10 +504,9 @@ class TestTimeout(unittest.TestCase):  # pylint: disable=too-many-public-methods
         pipeline.add_action(action)
         pipeline.add_action(TestTimeout.FakeSafeAction())
         pipeline.add_action(TestTimeout.FakeSafeAction())
-        pipeline.add_action(FinalizeAction())
+        finalize = FinalizeAction()
+        finalize.parameters['namespace'] = 'common'
+        pipeline.add_action(finalize)
         self.fakejob.pipeline = pipeline
         self.fakejob.device = TestTimeout.FakeDevice()
-        try:
-            self.fakejob.run()
-        except JobError as exc:
-            self.fail(exc)
+        self.assertEqual(self.fakejob.run(), 0)
