@@ -22,30 +22,28 @@
 import os
 import sys
 import yaml
+import logging
 import unittest
 from lava_dispatcher.pipeline.device import NewDevice
 from lava_dispatcher.pipeline.parser import JobParser
 from lava_dispatcher.pipeline.actions.boot.u_boot import (
     UBootAction,
-    UBootCommandOverlay,
     UBootSecondaryMedia
 )
+from lava_dispatcher.pipeline.actions.boot import BootloaderCommandOverlay
 from lava_dispatcher.pipeline.actions.deploy.apply_overlay import CompressRamdisk
 from lava_dispatcher.pipeline.actions.deploy.tftp import TftpAction
 from lava_dispatcher.pipeline.job import Job
-from lava_dispatcher.pipeline.action import Pipeline, InfrastructureError, JobError
-from lava_dispatcher.pipeline.test.test_basic import pipeline_reference
+from lava_dispatcher.pipeline.action import Pipeline, JobError
+from lava_dispatcher.pipeline.test.test_basic import Factory, StdoutTestCase
+from lava_dispatcher.pipeline.test.utils import DummyLogger
 from lava_dispatcher.pipeline.utils.network import dispatcher_ip
 from lava_dispatcher.pipeline.utils.shell import infrastructure_error
 from lava_dispatcher.pipeline.utils.filesystem import mkdtemp, tftpd_dir
 from lava_dispatcher.pipeline.utils.strings import substitute
-from lava_dispatcher.pipeline.utils.constants import (
-    SHUTDOWN_MESSAGE,
-    BOOT_MESSAGE,
-)
 
 
-class Factory(object):  # pylint: disable=too-few-public-methods
+class UBootFactory(Factory):  # pylint: disable=too-few-public-methods
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
@@ -56,29 +54,32 @@ class Factory(object):  # pylint: disable=too-few-public-methods
         bbb_yaml = os.path.join(os.path.dirname(__file__), filename)
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
-            job = parser.parse(sample_job_data, device, 4212, None, None, None,
+            job = parser.parse(sample_job_data, device, 4212, None, "",
                                output_dir=output_dir)
+            job.logger = DummyLogger()
         return job
 
 
-class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super(TestUbootAction, self).setUp()
+        self.factory = UBootFactory()
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
     def test_simulated_action(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
         self.assertIsNotNone(job)
 
         # uboot and uboot-ramdisk have the same pipeline structure
-        description_ref = pipeline_reference('uboot.yaml')
+        description_ref = self.pipeline_reference('uboot.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
         self.assertIsNone(job.validate())
         self.assertEqual(job.device['device_type'], 'beaglebone-black')
 
     def test_tftp_pipeline(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
         self.assertEqual(
             [action.name for action in job.pipeline.actions],
             ['tftp-deploy', 'uboot-action', 'lava-test-retry', 'finalize']
@@ -88,16 +89,18 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertIsNotNone(tftp.internal_pipeline)
         self.assertEqual(
             [action.name for action in tftp.internal_pipeline.actions],
-            ['download_retry', 'download_retry', 'download_retry', 'prepare-tftp-overlay', 'deploy-device-env']
+            ['download-retry', 'download-retry', 'download-retry', 'prepare-tftp-overlay', 'lxc-add-device-action', 'deploy-device-env']
         )
         self.assertIn('ramdisk', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('kernel', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertIn('dtb', [action.key for action in tftp.internal_pipeline.actions if hasattr(action, 'key')])
         self.assertNotIn('=', tftpd_dir())
+        job.validate()
+        tftp.validate()
+        self.assertEqual([], tftp.errors)
 
     def test_device_bbb(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot.yaml')
         self.assertEqual(
             job.device['commands']['connect'],
             'telnet localhost 6000'
@@ -109,14 +112,14 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
     def test_uboot_action(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
         job.validate()
         self.assertEqual(job.pipeline.errors, [])
         self.assertIn('u-boot', job.device['actions']['boot']['methods'])
-        params = job.device['actions']['boot']['methods']['u-boot']['parameters']
+        params = job.device['actions']['deploy']['parameters']
         self.assertIn('mkimage_arch', params)
-        boot_message = params.get('boot_message', BOOT_MESSAGE)
+        boot_message = params.get('boot_message',
+                                  job.device.get_constant('boot-message'))
         self.assertIsNotNone(boot_message)
         for action in job.pipeline.actions:
             action.validate()
@@ -125,7 +128,7 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
                 self.assertEqual('u-boot', action.parameters['method'])
                 self.assertEqual(
                     'reboot: Restarting system',
-                    action.parameters.get('parameters', {}).get('shutdown-message', SHUTDOWN_MESSAGE)
+                    action.parameters.get('parameters', {}).get('shutdown-message', job.device.get_constant('shutdown-message'))
                 )
             if isinstance(action, TftpAction):
                 self.assertIn('ramdisk', action.parameters)
@@ -159,16 +162,13 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
             }
         }
         device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
-        job = Job(4212, None, None, None, parameters)
+        job = Job(4212, parameters, None)
         job.device = device
         pipeline = Pipeline(job=job, parameters=parameters['actions']['boot'])
         job.pipeline = pipeline
-        overlay = UBootCommandOverlay()
+        overlay = BootloaderCommandOverlay()
         pipeline.add_action(overlay)
-        try:
-            ip_addr = dispatcher_ip()
-        except InfrastructureError as exc:
-            raise RuntimeError("Unable to get dispatcher IP address: %s" % exc)
+        ip_addr = dispatcher_ip(None)
         parsed = []
         kernel_addr = job.device['parameters'][overlay.parameters['type']]['ramdisk']
         ramdisk_addr = job.device['parameters'][overlay.parameters['type']]['ramdisk']
@@ -220,9 +220,30 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertNotIn("setenv initrd_addr_r '{RAMDISK_ADDR}'", parsed)
         self.assertNotIn("setenv fdt_addr_r '{DTB_ADDR}'", parsed)
 
+    def test_boot_commands(self):
+        job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk-inline-commands.yaml')
+        job.validate()
+        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
+        overlay = [action for action in uboot.internal_pipeline.actions if action.name == 'bootloader-overlay'][0]
+        self.assertEqual(overlay.commands, ['a list', 'of commands', 'with a {KERNEL_ADDR} substitution'])
+
+    def test_transfer_media(self):
+        """
+        Test adding the overlay to existing rootfs
+        """
+        job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk-inline-commands.yaml')
+        job.validate()
+        description_ref = self.pipeline_reference('uboot-ramdisk-inline-commands.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
+        retry = [action for action in uboot.internal_pipeline.actions if action.name == 'uboot-retry'][0]
+        transfer = [action for action in retry.internal_pipeline.actions if action.name == 'overlay-unpack'][0]
+        self.assertIn('transfer_overlay', transfer.parameters)
+        self.assertIn('download_command', transfer.parameters['transfer_overlay'])
+        self.assertIn('unpack_command', transfer.parameters['transfer_overlay'])
+
     def test_download_action(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot.yaml')
         for action in job.pipeline.actions:
             action.validate()
             self.assertTrue(action.valid)
@@ -246,11 +267,10 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertIsNotNone(test_dir)
         self.assertIn('/lava-', test_dir)
         self.assertIsNotNone(extract)
-        self.assertEqual(extract.timeout.duration, job.parameters['timeouts'][extract.name]['seconds'])
+        self.assertEqual(extract.timeout.duration, 240)
 
     def test_reset_actions(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot.yaml')
         uboot_action = None
         uboot_retry = None
         reset_action = None
@@ -266,17 +286,15 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
             if action.name == 'uboot-retry':
                 uboot_retry = action
         names = [r_action.name for r_action in uboot_retry.internal_pipeline.actions]
-        self.assertIn('reboot-device', names)
+        self.assertIn('reset-device', names)
         self.assertIn('u-boot-interrupt', names)
         self.assertIn('expect-shell-connection', names)
-        self.assertIn('u-boot-commands', names)
+        self.assertIn('bootloader-commands', names)
         for action in uboot_retry.internal_pipeline.actions:
-            if action.name == 'reboot-device':
+            if action.name == 'reset-device':
                 reset_action = action
         names = [r_action.name for r_action in reset_action.internal_pipeline.actions]
-        self.assertIn('soft-reboot', names)
-        self.assertIn('pdu_reboot', names)
-        self.assertIn('power_on', names)
+        self.assertIn('pdu-reboot', names)
 
     def test_secondary_media(self):
         """
@@ -286,11 +304,12 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         cubie = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/cubie1.yaml'))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         sample_job_data = open(sample_job_file)
-        job = job_parser.parse(sample_job_data, cubie, 4212, None, None, None,
+        job = job_parser.parse(sample_job_data, cubie, 4212, None, "",
                                output_dir='/tmp/')
+        job.logger = DummyLogger()
         job.validate()
         sample_job_data.close()
-        u_boot_media = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1].internal_pipeline.actions[1]
+        u_boot_media = [action for action in job.pipeline.actions if action.name == 'uboot-action'][1].internal_pipeline.actions[0]
         self.assertIsInstance(u_boot_media, UBootSecondaryMedia)
         self.assertEqual([], u_boot_media.errors)
         self.assertEqual(u_boot_media.parameters['kernel'], '/boot/vmlinuz-3.16.0-4-armmp-lpae')
@@ -312,47 +331,8 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
             action=u_boot_media.name, label='uuid', key='boot_part'))
         self.assertEqual(part_reference, "0:1")
 
-    @unittest.skipIf(infrastructure_error('telnet'), "telnet not installed")
-    def test_prompt_from_job(self):  # pylint: disable=too-many-locals
-        """
-        Support setting the prompt after login via the job
-
-        Loads a known YAML, adds a prompt to the dict and re-parses the job.
-        Checks that the prompt is available in the expect_shell_connection action.
-        """
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot.yaml')
-        job.validate()
-        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
-        retry = [action for action in uboot.internal_pipeline.actions
-                 if action.name == 'uboot-retry'][0]
-        expect = [action for action in retry.internal_pipeline.actions
-                  if action.name == 'expect-shell-connection'][0]
-        check = expect.parameters
-        device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
-        extra_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot.yaml')
-        with open(extra_yaml) as data:
-            sample_job_string = data.read()
-        parser = JobParser()
-        sample_job_data = yaml.load(sample_job_string)
-        boot = [item['boot'] for item in sample_job_data['actions'] if 'boot' in item][0]
-        self.assertIsNotNone(boot)
-        sample_job_string = yaml.dump(sample_job_data)
-        job = parser.parse(sample_job_string, device, 4212, None, None, None,
-                           output_dir='/tmp')
-        job.validate()
-        uboot = [action for action in job.pipeline.actions if action.name == 'uboot-action'][0]
-        retry = [action for action in uboot.internal_pipeline.actions
-                 if action.name == 'uboot-retry'][0]
-        expect = [action for action in retry.internal_pipeline.actions
-                  if action.name == 'expect-shell-connection'][0]
-        if sys.version < '3':
-            # skipping in 3 due to "RecursionError: maximum recursion depth exceeded in comparison"
-            self.assertNotEqual(check, expect.parameters)
-
     def test_xz_nfs(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/uboot-nfs.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/uboot-nfs.yaml')
         # this job won't validate as the .xz nfsrootfs URL is a fiction
         self.assertRaises(JobError, job.validate)
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
@@ -362,8 +342,7 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertEqual(nfs.parameters['nfsrootfs']['compression'], 'xz')
 
     def test_prefix(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/bbb-skip-install.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/bbb-skip-install.yaml')
         job.validate()
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
@@ -373,11 +352,94 @@ class TestUbootAction(unittest.TestCase):  # pylint: disable=too-many-public-met
         self.assertEqual(nfs.param_key, 'nfsrootfs')
 
 
-class TestOverlayCommands(unittest.TestCase):  # pylint: disable=too-many-public-methods
+class TestKernelConversion(StdoutTestCase):
+
+    def setUp(self):
+        logger = logging.getLogger('dispatcher')
+        logger.disabled = True
+        logger.propagate = False
+        logger = logging.getLogger('lava-dispatcher')
+        logger.disabled = True
+        logger.propagate = False
+        self.device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
+        bbb_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
+        with open(bbb_yaml) as sample_job_data:
+            self.base_data = yaml.load(sample_job_data)
+        self.deploy_block = [block for block in self.base_data['actions'] if 'deploy' in block][0]['deploy']
+        self.boot_block = [block for block in self.base_data['actions'] if 'boot' in block][0]['boot']
+        self.parser = JobParser()
+
+    def test_zimage_bootz(self):
+        self.deploy_block['kernel']['type'] = 'zimage'
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job.logger = DummyLogger()
+        job.validate()
+        deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        prepare = [action for action in overlay.internal_pipeline.actions if action.name == 'prepare-kernel'][0]
+        uboot_prepare = [action for action in prepare.internal_pipeline.actions if action.name == 'uboot-prepare-kernel'][0]
+        self.assertEqual('zimage', uboot_prepare.kernel_type)
+        self.assertEqual('bootz', uboot_prepare.bootcommand)
+        self.assertFalse(uboot_prepare.mkimage_conversion)
+
+    def test_image(self):
+        self.deploy_block['kernel']['type'] = 'image'
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job.logger = DummyLogger()
+        job.validate()
+        deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        prepare = [action for action in overlay.internal_pipeline.actions if action.name == 'prepare-kernel'][0]
+        uboot_prepare = [action for action in prepare.internal_pipeline.actions if action.name == 'uboot-prepare-kernel'][0]
+        self.assertEqual('image', uboot_prepare.kernel_type)
+        # bbb-01.yaml does not contain booti parameters, try to convert to a uImage
+        self.assertEqual('bootm', uboot_prepare.bootcommand)
+        self.assertTrue(uboot_prepare.mkimage_conversion)
+
+    def test_uimage(self):
+        self.deploy_block['kernel']['type'] = 'uimage'
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job.logger = DummyLogger()
+        job.validate()
+        deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        prepare = [action for action in overlay.internal_pipeline.actions if action.name == 'prepare-kernel'][0]
+        uboot_prepare = [action for action in prepare.internal_pipeline.actions if action.name == 'uboot-prepare-kernel'][0]
+        self.assertEqual('uimage', uboot_prepare.kernel_type)
+        self.assertEqual('bootm', uboot_prepare.bootcommand)
+        self.assertFalse(uboot_prepare.mkimage_conversion)
+
+    def test_zimage_nobootz(self):
+        # drop bootz from the device for this part of the test
+        del self.device['parameters']['bootz']
+        self.deploy_block['kernel']['type'] = 'zimage'
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job.logger = DummyLogger()
+        job.validate()
+        deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        prepare = [action for action in overlay.internal_pipeline.actions if action.name == 'prepare-kernel'][0]
+        uboot_prepare = [action for action in prepare.internal_pipeline.actions if action.name == 'uboot-prepare-kernel'][0]
+        self.assertEqual('zimage', uboot_prepare.kernel_type)
+        self.assertEqual('bootm', uboot_prepare.bootcommand)
+        self.assertTrue(uboot_prepare.mkimage_conversion)
+
+    def test_uimage_boot_type(self):
+        # uimage in boot type
+        del self.deploy_block['kernel']['type']
+        self.boot_block['type'] = 'bootm'
+        job = self.parser.parse(yaml.dump(self.base_data), self.device, 4212, None, "", output_dir=mkdtemp())
+        job.logger = DummyLogger()
+        job.validate()
+        deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
+        self.assertNotIn('uboot-prepare-kernel', [action.name for action in overlay.internal_pipeline.actions])
+
+
+class TestOverlayCommands(TestUbootAction):  # pylint: disable=too-many-public-methods
 
     def test_combined_ramdisk_nfs(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/bbb-ramdisk-nfs.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/bbb-ramdisk-nfs.yaml')
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
         nfs = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-nfsrootfs'][0]
@@ -396,8 +458,7 @@ class TestOverlayCommands(unittest.TestCase):  # pylint: disable=too-many-public
         self.assertIsNotNone(overlay.parameters.get('ramdisk', None))
 
     def test_ramdisk_nfs_nomodules(self):
-        factory = Factory()
-        job = factory.create_bbb_job('sample_jobs/bbb-uinitrd-nfs.yaml')
+        job = self.factory.create_bbb_job('sample_jobs/bbb-uinitrd-nfs.yaml')
         tftp_deploy = [action for action in job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
         nfs = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-nfsrootfs'][0]
