@@ -23,23 +23,11 @@
 
 # pylint: disable=anomalous-backslash-in-string
 
-# Default timeout for shell operations
-SHELL_DEFAULT_TIMEOUT = 60
-
-# Default timeout when downloading over http/https
-HTTP_DOWNLOAD_TIMEOUT = 15
-
 # Retry at most 5 times
 MAX_RETRY = 5
 
-# u-boot auto boot prompt
-UBOOT_AUTOBOOT_PROMPT = "Hit any key to stop autoboot"
-
-# u-boot interrupt character
-UBOOT_INTERRUPT_CHARACTER = ' '
-
-# u-boot default timeout for commands
-UBOOT_DEFAULT_CMD_TIMEOUT = 90
+# size of u-boot header to be removed from ramdisks, in bytes.
+UBOOT_DEFAULT_HEADER_LENGTH = 64
 
 # Ramdisk default filenames
 RAMDISK_FNAME = 'ramdisk.cpio'
@@ -56,24 +44,11 @@ SCP_DOWNLOAD_CHUNK_SIZE = 32768
 # Clamp on the maximum timeout allowed for overrides
 OVERRIDE_CLAMP_DURATION = 300
 
-# Auto-login prompt timeout default
-AUTOLOGIN_DEFAULT_TIMEOUT = 120
-
 # dispatcher temporary directory
 # This is distinct from the TFTP daemon directory
 # Files here are for download using the Apache /tmp alias.
 DISPATCHER_DOWNLOAD_DIR = "/var/lib/lava/dispatcher/tmp"
 
-# OS shutdown message
-# Override: set as the shutdown-message parameter of an Action.
-SHUTDOWN_MESSAGE = 'The system is going down for reboot NOW'
-
-# Kernel starting message
-BOOT_MESSAGE = 'Booting Linux'
-
-# Default shell prompt for AutoLogin
-DEFAULT_SHELL_PROMPT = 'lava-test: # '
-
 # Distinctive prompt characters which can
 # help distinguish status messages from shell prompts.
 DISTINCTIVE_PROMPT_CHARACTERS = "\\:"
@@ -91,11 +66,14 @@ ANDROID_TMP_DIR = '/data/local/tmp'
 LXC_PATH = "/var/lib/lxc"
 
 # LXC finalize timeout
-LAVA_LXC_TIMEOUT = 30
+LAVA_LXC_TIMEOUT = 90
 
 # LXC templates with mirror option
 LXC_TEMPLATE_WITH_MIRROR = ['debian', 'ubuntu']
 
+# LXC default packages
+LXC_DEFAULT_PACKAGES = "systemd,systemd-sysv"
+
 # Timeout used by the vland protocol when waiting for vland to
 # respond to the api.create_vlan request, in seconds.
 VLAND_DEPLOY_TIMEOUT = 120
@@ -106,11 +84,6 @@ IPXE_BOOT_PROMPT = "Press Ctrl-B for the iPXE command line"
 # bootloader default timeout for commands
 BOOTLOADER_DEFAULT_CMD_TIMEOUT = 90
 
-GRUB_BOOT_PROMPT = "Press enter to boot the selected OS"
-
-# Timeout for USB devices to settle and show up
-USB_SHOW_UP_TIMEOUT = 30
-
 # kernel boot monitoring
 # Some successful kernel builds end the boot with this string
 KERNEL_FREE_UNUSED_MSG = 'Freeing unused kernel memory'
@@ -127,6 +100,11 @@ KERNEL_PANIC_MSG = "Kernel panic - (.*) end Kernel panic"
 # init dropping to a shell - often needs a sendline
 KERNEL_INIT_ALERT = 'ALERT! .* does not exist.\s+Dropping to a shell!'
 
+# Login incorrect message
+LOGIN_INCORRECT_MSG = 'Login incorrect'
+# Login incorrect message
+LOGIN_TIMED_OUT_MSG = 'Login timed out'
+
 # qemu installer size limit in Mb
 # (i.e. size * 1024 * 1024)
 INSTALLER_IMAGE_MAX_SIZE = 8 * 1024  # 8Gb
@@ -136,8 +114,8 @@ INSTALLER_QUIET_MSG = 'Loading initial ramdisk'
 DEFAULT_V1_PATTERN = "(?P<test_case_id>.*-*)\\s+:\\s+(?P<result>(PASS|pass|FAIL|fail|SKIP|skip|UNKNOWN|unknown))"
 DEFAULT_V1_FIXUP = {'PASS': 'pass', 'FAIL': 'fail', 'SKIP': 'skip', 'UNKNOWN': 'unknown'}
 
-# Message for notifying completion of secondary deployment
-SECONDARY_DEPLOYMENT_MSG = "Secondary media deployment complete"
+# List of DD output prompts for notifying completion of secondary deployment
+DD_PROMPTS = ['[0-9]+\+[0-9]+ records out', '[0-9]+ bytes \(.*\) copied']
 
 # fallback UEFI menu label class
 DEFAULT_UEFI_LABEL_CLASS = 'a-zA-Z0-9\s\:'
@@ -153,3 +131,15 @@ DEFAULT_TESTDEF_NAME_CLASS = r'^[\w\d\_\-]+$'
 
 # Limit repetitive messages
 METADATA_MESSAGE_LIMIT = 8192
+
+# Versatile Express autorun interrupt character
+VEXPRESS_AUTORUN_INTERRUPT_CHARACTER = ' '
+
+# sys class kvm path
+SYS_CLASS_KVM = '/sys/class/misc/kvm'
+
+# default reboot commands
+REBOOT_COMMAND_LIST = ['reboot', 'reboot -n', 'reboot -nf']
+
+# Default size limit for tftp in bytes (4Gb)
+TFTP_SIZE_LIMIT = 4 * 1024 * 1024 * 1024
