@@ -25,9 +25,11 @@ import tarfile
 import tempfile
 import guestfs
 import subprocess
+import glob
+import logging
 from configobj import ConfigObj
 
-from lava_dispatcher.pipeline.action import JobError
+from lava_dispatcher.pipeline.action import InfrastructureError, JobError, LAVABug
 from lava_dispatcher.pipeline.utils.constants import LXC_PATH
 from lava_dispatcher.pipeline.utils.compression import (
     decompress_file,
@@ -39,15 +41,15 @@ def rmtree(directory):
     """
     Wrapper around shutil.rmtree to remove a directory tree while ignoring most
     errors.
-    If called on a symbolic link, this function will raise a RuntimeError.
+    If called on a symbolic link, this function will raise a LAVABug.
     """
     # TODO: consider how to handle problems if the directory has already been removed -
     # coding bugs may trigger this Runtime exception - implement before moving to production.
     try:
         shutil.rmtree(directory)
     except OSError as exc:
-        raise RuntimeError("Error when trying to remove '%s': %s"
-                           % (directory, exc))
+        raise LAVABug("Error when trying to remove '%s': %s"
+                      % (directory, exc))
 
 
 def mkdtemp(autoremove=True, basedir='/tmp'):
@@ -92,14 +94,14 @@ def tftpd_dir():
     read in 'TFTP_DIRECTORY' from /etc/default/tftpd-hpa
     Any file to be offered using tftp must use this directory or a
     subdirectory of it. Default installation value: /srv/tftp/
-    :return: real path to the TFTP directory or raises RuntimeError
+    :return: real path to the TFTP directory or raises InfrastructureError
     """
     var_name = 'TFTP_DIRECTORY'
     if os.path.exists('/etc/default/tftpd-hpa'):
         config = ConfigObj('/etc/default/tftpd-hpa')
         value = config.get(var_name)
         return os.path.realpath(value)
-    raise RuntimeError("Unable to identify tftpd directory")
+    raise InfrastructureError("Unable to identify tftpd directory")
 
 
 def write_bootscript(commands, filename):
@@ -128,7 +130,7 @@ def prepare_guestfs(output, overlay, size):
     guest.launch()
     devices = guest.list_devices()
     if len(devices) != 1:
-        raise RuntimeError("Unable to prepare guestfs")
+        raise InfrastructureError("Unable to prepare guestfs")
     guest_device = devices[0]
     guest.mke2fs(guest_device, label='LAVA')
     # extract to a temp location
@@ -162,7 +164,7 @@ def prepare_install_base(output, size):
     guest.launch()
     devices = guest.list_devices()
     if len(devices) != 1:
-        raise RuntimeError("Unable to prepare guestfs")
+        raise InfrastructureError("Unable to prepare guestfs")
     guest.shutdown()
 
 
@@ -175,13 +177,13 @@ def copy_out_files(image, filenames, destination):
     source files exist in separate directories.
     """
     if not isinstance(filenames, list):
-        raise RuntimeError('filenames must be a list')
+        raise LAVABug('filenames must be a list')
     guest = guestfs.GuestFS(python_return_dict=True)
     guest.add_drive_ro(image)
     guest.launch()
     devices = guest.list_devices()
     if len(devices) != 1:
-        raise RuntimeError("Unable to prepare guestfs")
+        raise InfrastructureError("Unable to prepare guestfs")
     guest.mount_ro(devices[0], '/')
     for filename in filenames:
         file_buf = guest.read_file(filename)
@@ -200,7 +202,7 @@ def copy_in_overlay(image, root_partition, overlay):
     guest.launch()
     partitions = guest.list_partitions()
     if not partitions:
-        raise RuntimeError("Unable to prepare guestfs")
+        raise InfrastructureError("Unable to prepare guestfs")
     guest_partition = partitions[root_partition]
     guest.mount(guest_partition, '/')
     # FIXME: max message length issues when using tar_in
@@ -213,13 +215,24 @@ def copy_in_overlay(image, root_partition, overlay):
     guest.umount(guest_partition)
 
 
-def copy_to_lxc(lxc_name, src):
+def lxc_path(dispatcher_config):
+    """
+    Returns LXC_PATH which is a constant, unless a dispatcher specific path is
+    configured via lxc_path key in dispatcher_config.
+    """
+    try:
+        return dispatcher_config["lxc_path"]
+    except (KeyError, TypeError):
+        return LXC_PATH
+
+
+def copy_to_lxc(lxc_name, src, dispatcher_config):
     """Copies given file in SRC to lxc filesystem '/' with the provided
-    LXC_NAME and configured LXC_PATH
+    LXC_NAME and configured lxc_path
 
     For example, SRC such as '/var/lib/lava/dispatcher/tmp/tmpuuI_U0/system.img'
     will get copied to '/var/lib/lxc/lxc-nexus4-test-None/rootfs/system.img'
-    where, '/var/lib/lxc' is the LXC_PATH and 'lxc-nexus4-test-None' is the
+    where, '/var/lib/lxc' is the lxc_path and 'lxc-nexus4-test-None' is the
     LXC_NAME
 
     Returns the destination path within lxc. For example, '/boot.img'
@@ -227,7 +240,8 @@ def copy_to_lxc(lxc_name, src):
     Raises JobError if the copy failed.
     """
     filename = os.path.basename(src)
-    dst = os.path.join(LXC_PATH, lxc_name, 'rootfs', filename)
+    dst = os.path.join(lxc_path(dispatcher_config), lxc_name, 'rootfs',
+                       filename)
     try:
         shutil.copyfile(src, dst)
     except IOError:
@@ -253,3 +267,50 @@ def copy_overlay_to_sparse_fs(image, overlay):
     subprocess.check_output(['/usr/bin/img2simg', ext4_img, image],
                             stderr=subprocess.STDOUT)
     os.remove(ext4_img)
+
+
+def debian_package_version(pkg='lava-dispatcher', split=True):
+    """
+    Relies on Debian Policy rules for the existence of the
+    changelog. Distributions not derived from Debian will
+    return an empty string.
+    """
+    changelog = '/usr/share/doc/%s/changelog.Debian.gz' % pkg
+    if os.path.exists(changelog):
+        deb_version = subprocess.check_output((
+            'dpkg-query', '-W', "-f=${Version}\n",
+            "%s" % pkg)).strip().decode('utf-8')
+        # example version returned would be '2016.11'
+        if split:
+            return deb_version.split('-')[0]
+        return deb_version
+    return ''
+
+
+def copy_directory_contents(root_dir, dst_dir):
+    """
+    Copies the contents of the root directory to the destination directory
+    but excludes the root directory's top level folder
+    """
+    files_to_copy = glob.glob(os.path.join(root_dir, '*'))
+    logger = logging.getLogger('dispatcher')
+    for fname in files_to_copy:
+        logger.debug("copying %s to %s" % (fname, os.path.join(dst_dir, os.path.basename(fname))))
+        if os.path.isdir(fname):
+            shutil.copytree(fname, os.path.join(dst_dir, os.path.basename(fname)))
+        else:
+            shutil.copy(fname, dst_dir)
+
+
+def remove_directory_contents(root_dir):
+    """
+    Removes the contents of the root directory but not the root itself
+    """
+    files_to_remove = glob.glob(os.path.join(root_dir, '*'))
+    logger = logging.getLogger('dispatcher')
+    for fname in files_to_remove:
+        logger.debug("removing %s" % fname)
+        if os.path.isdir(fname):
+            shutil.rmtree(fname)
+        else:
+            os.remove(fname)
