@@ -27,6 +27,9 @@ def test_suite():
         'lava_dispatcher.pipeline.test.test_ipxe',
         'lava_dispatcher.pipeline.test.test_grub',
         'lava_dispatcher.pipeline.test.test_messages',
+        'lava_dispatcher.pipeline.test.test_auto_login',
+        'lava_dispatcher.pipeline.test.test_power',
+        'lava_dispatcher.pipeline.test.test_uefi_shell',
     ]
     loader = unittest.TestLoader()
     return loader.loadTestsFromNames(module_names)
